/*
 * EnumValue.java
 * Part of the B'am library - a collection of general-purpose utility classes.
 * 
 * Copyright (C)2006 Stephen Battey
 * Contact: resource@lucidviews.net
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

package net.lucidviews.util;

import java.lang.reflect.Field;
import java.lang.reflect.Modifier;


/**
 * Base-class for defining an enumeration type.
 * <br />
 * <br />
 * To define an enumeration extend this class, implementing the
 * <code>getEnumeration</code> method, and create instances for each value in
 * your enumeration.
 * <br />
 * In Java5, use of generics allows the exact type (sub-class) of EnumValue to be
 * associated with the Enumeration.
 * <br />
 * <br />
 * For example:
 * <code>
 * <br />
 * <br />public class DistanceUnit extends EnumValue&lt;DistanceUnit&gt;
 * <br />{
 * <br />&nbsp;&nbsp;// create a list for the enumeration values
 * <br />&nbsp;&nbsp;public static final Enumeration&lt;DistanceUnit&gt; ENUMERATION =&nbsp;new&nbsp;Enumeration&lt;DistanceUnit&gt;();
 * <br />&nbsp;&nbsp;
 * <br />&nbsp;&nbsp;// provide the EnumValue base class with access to the enumeration
 * <br />&nbsp;&nbsp;public Enumeration&lt;DistanceUnit&gt; getEnumeration()
 * <br />&nbsp;&nbsp;{
 * <br />&nbsp;&nbsp;&nbsp;&nbsp;return ENUMERATION;
 * <br />&nbsp;&nbsp;}
 * <br />&nbsp;&nbsp;
 * <br />&nbsp;&nbsp;// define the enumeration values
 * <br />&nbsp;&nbsp;public static final DistanceUnit METRE = new DistanceUnit();
 * <br />&nbsp;&nbsp;public static final DistanceUnit YARD = new DistanceUnit();
 * <br />&nbsp;&nbsp;public static final DistanceUnit KILOMETRE = new DistanceUnit();
 * <br />&nbsp;&nbsp;public static final DistanceUnit MILE = new DistanceUnit();
 * <br />&nbsp;&nbsp;
 * <br />&nbsp;&nbsp;// make the constructor private to protect this enum
 * <br />&nbsp;&nbsp;private DistanceUnit()
 * <br />&nbsp;&nbsp;{
 * <br />&nbsp;&nbsp;}
 * <br />}
 * <br />
 * <br />
 * </code>
 * It is essential that all constructors of the class are <code>protected</code>
 * or <code>private</code> so that no other instances can be created, which
 * would extend your enumeration with values that you did not define. This
 * includes the default constructor, which must be declared
 * <code>protected</code> or <code>private</code> if you define no other
 * constructor.
 * <br />
 * However, it is recommended that you assign a user-friendly text string to
 * each of your enumeration values. To do this you will need to define a
 * constructor in your enum class like this:
 * <code>
 * <br />&nbsp;&nbsp;...
 * <br />&nbsp;&nbsp;// define the enumeration values
 * <br />&nbsp;&nbsp;public static final DistanceUnit METRE = new DistanceUnit( "metre" );
 * <br />&nbsp;&nbsp;public static final DistanceUnit YARD = new DistanceUnit( "yard" );
 * <br />&nbsp;&nbsp;public static final DistanceUnit KILOMETRE = new DistanceUnit( "kilometre" );
 * <br />&nbsp;&nbsp;public static final DistanceUnit MILE = new DistanceUnit( "mile" );
 * <br />&nbsp;&nbsp;
 * <br />&nbsp;&nbsp;// make the constructor private to protect this enum
 * <br />&nbsp;&nbsp;private DistanceUnit( String unitName )
 * <br />&nbsp;&nbsp;{
 * <br />&nbsp;&nbsp;&nbsp;&nbsp;super( unitName );
 * <br />&nbsp;&nbsp;}
 * <br />&nbsp;&nbsp;...
 * </code>
 * <br />
 * <br />
 * The enumeration type is now ready to use. Using the enumeration type makes
 * the source code more readable and ensures the compiler performs some type
 * checking on parameter values.
 * <br />
 * For example, the method:
 * <br /><code>public void setDistanceUnit( int distanceUnit )</code>
 * <br />would become
 * <br /><code>public void setDistanceUnit( DistanceUnit newValue )</code>
 * <br />
 * The primary advantage is that the <code>newValue</code> must be one of the
 * constants defined by the <code>DistanceUnit</code> class - in the first
 * interface, the <code>distanceUnit</code> could be any <code>int</code> value
 * and the method would need to perform some range checks first and throw an
 * exception. With enumerations the checks are done at compile time and no
 * exception needs to be caught.
 * <br />
 * <br />
 * Enumeration values can be taken from the exported constants in the
 * <code>EnumValue</code> sub class or derived from their index or user-friendly
 * name.
 * <br />
 * For example, if a string value "<code>mile</code>" is read from a
 * configuration file, the enumeration value can be looked-up:
 * <code>
 * <br />String distanceUnitName = ...;
 * <br />DistanceUnit distanceUnit = DistanceUnit.ENUMERATION.lookupValue( distanceUnitName );
 * </code>
 * <br />
 * <br />
 * <b>IMPORTANT</b>
 * <br />
 * Use the <code>==</code> operator, not the
 * <code>equals</code> method to evaluate or compare <code>EnumValue</code>s.
 * <br />
 * The <code>equals</code> method performs an internal representation
 * comparision and is provided so that string and integer values will match
 * the text-value and index of an enumeration value.
 * <br />
 * Each enumeration value is a new, unique constant and should be compared
 * directly. For example:
 * <code>
 * <br />String distanceUnitName = ...;
 * <br />DistanceUnit distanceUnit = DistanceUnit.ENUMERATION.lookupValue( distanceUnitName );
 * <br />if (distanceUnit == DistanceUnit.MILE)
 * <br />{
 * <br />&nbsp;&nbsp;...
 * <br />}
 * </code>
 * <br />
 * <br />
 * <b>Sub Enumerations</b>
 * <br />
 * In Java5 the <code>EnumValue</code> class supports sub enumerations
 * - sub lists within an enumeration.
 * <br />
 * To support sub lists you should define your enum class slightly differently to the
 * definition described above.
 * <br />
 * <br />
 * Like this:
 * <code>
 * <br />
 * <br />public class Department&lt;T extends Department&gt; extends EnumValue&lt;T&gt;
 * <br />{
 * <br />&nbsp;&nbsp;// create a list for the enumeration values
 * <br />&nbsp;&nbsp;public static final Enumeration&lt;Department&gt; ENUMERATION =&nbsp;new&nbsp;Enumeration&lt;Department&gt;();
 * <br />&nbsp;&nbsp;
 * <br />&nbsp;&nbsp;// provide the EnumValue base class with access to the enumeration
 * <br />&nbsp;&nbsp;public Enumeration&lt;T&gt; getEnumeration()
 * <br />&nbsp;&nbsp;{
 * <br />&nbsp;&nbsp;&nbsp;&nbsp;// The following cast works only when T = Department.
 * <br />&nbsp;&nbsp;&nbsp;&nbsp;// Sub-classes MUST override this method
 * <br />&nbsp;&nbsp;&nbsp;&nbsp;// and return an Enumeration of values of that sub type.
 * <br />&nbsp;&nbsp;&nbsp;&nbsp;return (Enumeration&lt;T&gt;)ENUMERATION;
 * <br />&nbsp;&nbsp;}
 * <br />&nbsp;&nbsp;
 * <br />&nbsp;&nbsp;// define the enumeration values
 * <br />&nbsp;&nbsp;public static final Department&lt;Department&gt; SALES = new Department&lt;Department&gt;( "sales" );
 * <br />&nbsp;&nbsp;public static final Department&lt;Department&gt; ACCOUNTS = new Department&lt;Department&gt;( "accounts" );
 * <br />&nbsp;&nbsp;public static final Department&lt;Department&gt; OFFICE_SERVICES = new Department&lt;Department&gt;( "office services" );
 * <br />&nbsp;&nbsp;
 * <br />&nbsp;&nbsp;// protect the constructor to prevent new departments being created
 * <br />&nbsp;&nbsp;protected Department( String name )
 * <br />&nbsp;&nbsp;{
 * <br />&nbsp;&nbsp;&nbsp;&nbsp;super( name );
 * <br />&nbsp;&nbsp;}
 * <br />}
 * <br />
 * <br />
 * </code>
 * Sub lists can then be defined like so:
 * <code>
 * <br />
 * <br />public class TechnicalDepartment&lt;T extends TechnicalDepartment&gt; extends Department&lt;T&gt;
 * <br />{
 * <br />&nbsp;&nbsp;// create a list for the enumeration values
 * <br />&nbsp;&nbsp;public static final Enumeration&lt;TechnicalDepartment&gt; ENUMERATION =&nbsp;new&nbsp;Enumeration&lt;TechnicalDepartment&gt;( Department.ENUMERATION );
 * <br />&nbsp;&nbsp;
 * <br />&nbsp;&nbsp;// provide the EnumValue base class with access to the enumeration
 * <br />&nbsp;&nbsp;public Enumeration&lt;T&gt; getEnumeration()
 * <br />&nbsp;&nbsp;{
 * <br />&nbsp;&nbsp;&nbsp;&nbsp;// The following cast works only when T = TechnicalDepartment.
 * <br />&nbsp;&nbsp;&nbsp;&nbsp;// Sub-classes MUST override this method
 * <br />&nbsp;&nbsp;&nbsp;&nbsp;// and return an Enumeration of values of that sub type.
 * <br />&nbsp;&nbsp;&nbsp;&nbsp;return (Enumeration&lt;T&gt;)ENUMERATION;
 * <br />&nbsp;&nbsp;}
 * <br />&nbsp;&nbsp;
 * <br />&nbsp;&nbsp;// define the enumeration values
 * <br />&nbsp;&nbsp;public static final TechnicalDepartment&lt;TechnicalDepartment&gt; DEVELOPMENT = new TechnicalDepartment&lt;TechnicalDepartment&gt;( "development" );
 * <br />&nbsp;&nbsp;public static final TechnicalDepartment&lt;TechnicalDepartment&gt; SUPPORT = new TechnicalDepartment&lt;TechnicalDepartment&gt;( "support" );
 * <br />&nbsp;&nbsp;public static final TechnicalDepartment&lt;TechnicalDepartment&gt; TESTING = new TechnicalDepartment&lt;TechnicalDepartment&gt;( "testing" );
 * <br />&nbsp;&nbsp;
 * <br />&nbsp;&nbsp;// protect the constructor to prevent new technical departments being created
 * <br />&nbsp;&nbsp;protected TechnicalDepartment( String name )
 * <br />&nbsp;&nbsp;{
 * <br />&nbsp;&nbsp;&nbsp;&nbsp;super( name );
 * <br />&nbsp;&nbsp;}
 * <br />}
 * <br />
 * <br />
 * </code>
 * As the definition of the class suggests (<code>TechnicalDepartment extends Department</code>) a technical department
 * is a sub type of a department.
 * <br />
 * The <code>TechnicalDepartment</code>s DEVELOPMENT, SUPPORT and TESTING not only make up the sub-list of
 * technical departments but are also part of the <code>Department</code> enumeration.
 * <br />
 * i.e.
 * <br />
 * <br />
 * <code>TechnicalDepartment.ENUMERATION.valueOf( "development" )</code>
 * <br />
 * returns <code>TechnicalDepartment.DEVELOPMENT</code>
 * <br />
 * <br />
 * <code>Department.ENUMERATION.valueOf( "development" )</code>
 * <br />
 * returns <code>TechnicalDepartment.DEVELOPMENT</code>
 * <br />
 * <br />
 * <code>Department.ENUMERATION.valueOf( "sales" )</code>
 * <br />
 * returns <code>Department.SALES</code>
 * <br />
 * <br />
 * <code>TechnicalDepartment.ENUMERATION.valueOf( "sales" )</code>
 * <br />
 * throws a <code>NoSuchElementException</code>
 * <br />
 * <br />
 * @param <T> the EnumValue sub class
 * @see net.lucidviews.util.Enumeration
 * @author Stephen Battey
 * @version $Revision: 1.4.2.2 $
 * @since 1.0
 */
public abstract class EnumValue<T extends EnumValue> implements java.io.Serializable
{

	/*
	 * Local constants
	 */

	/** A value assigned to the index variable before the index of the enum
	 *  value is known. */
	protected static final int UNDEFINED_INDEX = -1;
	
	/** The name of the field that sub classes use to hold the Enumeration.
	 *  @since 1.2 */
	protected static final String ENUMERATION_FIELD_NAME = "ENUMERATION";


	/*
	 * Instance variables
	 */

	/** The user-friendly name associated with this enumeration value. */
	protected String  _textValue;

	/** A flag indicating if the text value is case sensitive. */
	protected boolean _isCaseSensitive;

	/** The index of this value within the enumeration. */
	protected int _index;


	/*
	 * Constructor(s)
	 */

	/**
	 * Create an enumeration value that can be identified by the given text.
	 * <br />
	 * <br />
	 * @param textValue the <code>String</code> that is associated with this
	 *                  enum value
	 * @param isCaseSensitive indicates if a <code>String</code> should be
	 *                        associated with this enum value only if the case
	 *                        matches that in the text value
	 */
	protected EnumValue( String textValue, boolean isCaseSensitive )
	{
		_textValue = textValue;
		_isCaseSensitive = isCaseSensitive;
		_index = UNDEFINED_INDEX;
		/* it is important to set the index as undefined first, otherwise
		   the index will default to 0, which will cause this enum value to
		   be equal to any enum value at index 0 of the enumeration. */
		
		try
		{
			_index = getEnumeration().addValue( (T)this );
		}
		catch (ClassCastException e)
		{
			// The cast "(T)this" will work as long as the sub-class was defined correctly,
			// with the sub-class specified in the parameterisation of EnumValue.
			// E.g.  public class MyEnumValue extends EnumValue<MyEnumValue>
			throw new ClassCastException( "Sub-class of EnumValue did not extend EnumValue correctly. " + e.getMessage() );
		}
	}

	/**
	 * Create an enumeration value that can be identified by the given text.
	 * <br />
	 * The text will not be treated as case sensitive.
	 * <br />
	 * <br />
	 * @param textValue the <code>String</code> that is associated with this
	 *                  enum value
	 */
	protected EnumValue( String textValue )
	{
		this( textValue, false );
	}

	/**
	 * Create an enumeration value with no user-friendly name.
	 * <br />
	 * The value can then only be cross-referenced by its index.
	 */
	protected EnumValue()
	{
		this( null, false );
	}


	/*
	 * Abstract method(s)
	 */

	/**
	 * Obtain the enumeration this value belongs to.
	 * <br />
	 * See the class JavaDoc, at the top of this page, for information on how to
	 * implement this method correctly.
	 * <br />
	 * <br />
	 * @return a list of enumeration values that this value is part of
	 */
	public abstract Enumeration<T> getEnumeration();
	
	
	/*
	 * Methods
	 */
	
	/**
	 * Get the index of this enum value within the enumeration list.
	 * <br />
	 * <br />
	 * @return the index of this enum value
	 * @since 1.2
	 */
	public int getIndex()
	{
		return _index;
	}
	
	/**
	 * A synonym for <code>getIndex</code>, defined to make switching from
	 * <code>enum</code>s to <code>EnumValue</code>s easier.
	 * <br />
	 * <br />
	 * @return the index of this enum value
	 * @since 1.2
	 * @see #getIndex()
	 */
	public int ordinal()
	{
		return _index;
	}
	
	/**
	 * Get the text representation of this enum value.
	 * <br />
	 * <br />
	 * @return the text value of this enum value
	 * @since 1.2
	 */
	public String getTextValue()
	{
		return _textValue;
	}
	
	/**
	 * A synonym for <code>getTextValue</code>, defined to make switching from
	 * <code>enum</code>s to <code>EnumValue</code>s easier.
	 * <br />
	 * <br />
	 * @return the text value of this enum value
	 * @since 1.2
	 * @see #getTextValue()
	 */
	public String name()
	{
		return _textValue;
	}
	
	/**
     * Compares this value with another enum value for order.  Returns a
     * negative integer, zero, or a positive integer as this object is less
     * than, equal to, or greater than the specified object.
     * <br />
     * Enum constants are only comparable to other enum constants of the
     * same enum type.  The natural order implemented by this
     * method is the order in which the constants are declared.
	 * <br />
	 * <br />
	 * @param other the other enum value
	 * @return an integer that describes if this value is larger, equal or
	 *         smaller than the other value 
	 * @since 1.2
	 */
	public int compareTo( T other )
	{
		return this._index - other._index;
	}
	
	/**
	 * Returns the <code>EnumValue</code> of the specified enumeration type
	 * with the specified name. The name must match with the text value of
	 * the enum value. (Extraneous whitespace characters are not permitted.)
	 * <br />
	 * <br />
	 * @param enumType the <code>Class</code> object of the
	 *                 <code>EnumValue</code> from which to return a constant
	 * @param name the text value of the enum value to return
	 * @return the <code>EnumValue</code> from the specified enumeration type
	 *         with the specified name
	 * @throws IllegalArgumentException if the specified enumeration class has
	 *                                  no constant with the specified name, or
	 *                                  the specified class does not represent
	 *                                  an enum type
	 * @throws NullPointerException if <code>enumType</code> or
	 *                              <code>name</code> is <code>null</code>
	 * @since 1.2
	 */
	public static EnumValue<?> valueOf( Class<? extends EnumValue> enumType, String name ) throws IllegalArgumentException, NullPointerException
	{
		try
		{
			Field enumerationField = enumType.getField( ENUMERATION_FIELD_NAME );
			if (!Modifier.isStatic( enumerationField.getModifiers() ))
			{
				throw new NoSuchFieldException( "The field exists but is not static." );
			}
			
			Object enumeration = enumerationField.get( null );
			if (!(enumeration instanceof Enumeration))
			{
				throw new NoSuchFieldException( "The field exists but is not a net.lucidviews.util.Enumeration." );
			}
			
			return ((Enumeration)enumeration).valueOf( name );
		}
		catch (NoSuchFieldException e)
		{
			throw new IllegalArgumentException( "Class " + enumType.getName() + " is not an enumeration. It does not declare a static Enumeration field called " + ENUMERATION_FIELD_NAME + "." );
		}
		catch (IllegalAccessException e)
		{
			throw new IllegalArgumentException( "Cannot access " + ENUMERATION_FIELD_NAME + " field in the class " + enumType.getName() + "." );
		}
    }


	/*
	 * Overridden methods of Object base class
	 */

	/** {@inheritDoc} */
	public String toString()
	{
		if (_textValue != null)
			return _textValue;
		else
			return String.valueOf( _index );
	}

	/** {@inheritDoc} */
	public int hashCode()
	{
		/* Always use the toString method to derive a hash-code.
		   Using the _index could lead to matching the hash code of the index
		   of one enum value to the hash code of the text-value of another enum
		   value. */
		String hashText;

		if (_isCaseSensitive)
			hashText = toString();
		else
			hashText = toString().toLowerCase();

		return hashText.hashCode();
	}

	/** {@inheritDoc} */
	public boolean equals( Object obj )
	{
		/* for an EnumValue object, the values must be the same instance */
		if (obj instanceof EnumValue)
			return obj == this;

		/* for other types of object, compare the string representations
		   - this will ensure we match Strings and Numbers */
		String objString = obj.toString();

		/* try comparing the user-friendly name */
		if (_textValue != null)
		{
			if (_isCaseSensitive)
			{
				if (objString.equals( _textValue ))
					return true;
			}
			else
			{
				if (objString.equalsIgnoreCase( _textValue ))
					return true;
			}
		}

		/* failing that, try comparing the index */
		if (_index != UNDEFINED_INDEX)
		{
			String indexString = Integer.toString( _index );
			if (objString.equals( indexString ))
				return true;
		}

		return false;
	}

}


/*
 * Change History
 *
 * $Log: EnumValue.java,v $
 * Revision 1.4.2.2  2008/11/15 18:52:47  Steve
 * d legal disclaimer.
 * @
 * text
 * @d25 3
 * d145 1
 * a145 *
 * Revision 1.4.2.1  2007/12/04 21:35:12  Steve
 * d legal disclaimer.
 * @
 * text
 * @d25 3
 * d145 1
 * a145 1
 *  * @@version $Revision: 1.4.2.2 $
 * d157 5
 * a161 1
 *
 * Revision 1.4  2007/10/05 19:24:33  Steve
 * Added methods to following the naming conventions used by the Java5 enums.
 *
 * Revision 1.3  2006/09/06 20:52:02  Steve
 * Added legal disclaimer.
 *
 * Revision 1.2  2006/06/12 20:50:16  Steve
 * Fixed JavaDoc x-ref.
 *
 * Revision 1.1  2006/04/17 15:33:38  Steve
 * Initial revision.
 *
 */
