/*
 * NotNull.java
 * Part of the B'am library - a collection of general-purpose utility classes.
 * 
 * Copyright (C)2008 Stephen Battey
 * Contact: resource@lucidviews.net
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.lucidviews.util;


/**
 * A wrapper class for maintaining a variable in a not-null state.
 * <br />
 * <br />
 * The wrapper class will perform checks to ensure the variable is never set to null.
 * <br />
 * Performing the null pointer check when a value is set removes the need to check for
 * a null pointer when the value is used/accessed.
 * <br />
 * It also ensures that any process fails early (at the time when an inappropriate value is computed/assigned)
 * rather than allowing the program to continue until the value is accessed.
 * <br />
 * <br />
 * @param <T> the type of variable being wrapped
 * @author Stephen Battey
 * @version $Revision: 1.1.2.2 $
 * @since 1.3
 */
public class NotNull<T>
{
	
	/*
	 * Instance variables
	 */
	
	/** The current value. */
	protected T value;
	
	
	/*
	 * Constructors
	 */
	
	/**
	 * Create a variable that is always guaranteed not null.
	 * <br />
	 * If the initial value is <code>null</code> an <code>IllegalNullValueException</code>
	 * will be thrown.
	 * <br />
	 * <br />
	 * @param value the initial value, which can't be null
	 */
	public NotNull( T value )
	{
		setValue( value );
	}
	
	
	/*
	 * Factory methods
	 */
	
	/**
	 * Create a <code>NotNull</code> variable using an initial value.
	 * <br />
	 * This is just a convenience method for creating new instances of the <code>NotNull</code> class.
	 * <br />
	 * <br />
	 * @param <T> the type of variable being wrapped
	 * @param value the initial value, which can't be null
	 * @return a <code>NotNull</code> wrapper around the original <code>value</code>
	 */
	public static <T> NotNull<T> instance( T value )
	{
		return new NotNull<T>( value );
	}
	
	
	/*
	 * Methods
	 */
	
	/**
	 * Change the value.
	 * <br />
	 * If the new value is <code>null</code> an <code>IllegalNullValueException</code>
	 * will be thrown.
	 * <br />
	 * <br />
	 * @param value the new value, which can't be null
	 */
	public void setValue( T value )
	{
		if (value == null)
			throw new IllegalNullValueException();
		
		this.value = value;
	}
	
	/**
	 * Get the value.
	 * <br />
	 * The return value is guaranteed not to be <code>null</code>.
	 * <br />
	 * <br />
	 * @return the value
	 */
	public T getValue()
	{
		return this.value;
	}
	
}


/*
 * Change History
 *
 * $Log: NotNull.java,v $
 * Revision 1.1.2.2  2008/05/25 16:21:25  Steve
 * Added 'instance' factory method.
 * Modified a few JavaDoc comments and the file header (copyright notice).
 *
 * Revision 1.1.2.1  2008/04/21 21:26:31  Steve
 * Initial revision.
 *
 */