/*
 * FloatingImage.java
 * Part of the B'am library - a collection of general-purpose utility classes.
 * 
 * Copyright (C)2006 Stephen Battey
 * Contact: resource@lucidviews.net
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

package net.lucidviews.util.gui.swing;

import net.lucidviews.util.image.SimpleImageObserver;

import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.Image;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Component;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.Collection;


/**
 * An image that can be positioned at any given co-ordinate.
 * <br />
 * Once created you will need to explicitly call <code>paint</code>, usually
 * from within the <code>paint</code> method of the parent component.
 * <br />
 * e.g.
 * <pre>public class MyPanel extends JPanel
 * {
 *
 *   ...
 *   protected static final Image THE_IMAGE
 *            = Toolkit.getDefaultToolkit().createImage( "some/file/path.gif" );
 *   ...
 *
 *   public MyPanel()
 *   {
 *     ...
 *     _floatingImage = new FloatingImage( THE_IMAGE, this );
 *     ...
 *   }
 *
 *   ...
 *
 *   public void paint( Graphics g )
 *   {
 *     super.paint( g );
 *
 *     // paint the floating image, which is a non-standard component
 *     _floatingImage.paint( g );
 *   }
 *
 * }
 * </pre>
 * <br />
 * <br />
 * @author Stephen Battey
 * @version $Revision: 1.3 $
 * @since 1.0
 */
public class FloatingImage implements MouseListener
{
	
	/*
	 * Local constants
	 */
	
	/** The initial <code>_xPos</code> value. */
	protected static final int DEFAULT_X_POS = 0;
	
	/** The initial <code>_yPos</code> value. */
	protected static final int DEFAULT_Y_POS = 0;
	
	
	/*
	 * Instance variables
	 */
	
	/** The image to be displayed. */
	protected Image     _image;
	
	/** The x position of the left edge of the image, relative to the
	 *  <code>Graphics</code> context passed to the <code>paint</code> method. */
	protected int       _xPos;
	
	/** The y position of the top edge of the image, relative to the
	 *  <code>Graphics</code> context passed to the <code>paint</code> method. */
	protected int       _yPos;
	
	/** The UI component that this image component is contained within. */
	protected Component _parentComponent;
	
	/** A flag indicating if this image is to be rendered. */
	protected boolean   _displayed;
	
	/** A collection of <code>MouseListener</code>s listening for events from
	 *  this object. */
	protected Collection _mouseListeners;
	
	
	/*
	 * Constructor(s)
	 */
	
	/**
	 * Create a floating image to be displayed at a given position, relative to
	 * the specified parent component.
	 * <br />
	 * <br />
	 * @param image the image to be rendered
	 * @param xPos the x co-ordinate of the left edge of the image, relative to
	 *             the left edge of the parent component
	 * @param yPos the y co-ordinate of the top edge of the image, relative to
	 *             the top edge of the parent component
	 * @param parentComponent the parent component
	 */
	public FloatingImage( Image image, int xPos, int yPos, Component parentComponent )
	{
		if (parentComponent == null)
			throw new IllegalArgumentException( "A FloatingImage cannot be created with a null parent component" );
		
		_image = image;
		_xPos = xPos;
		_yPos = yPos;
		_parentComponent = parentComponent;
		_displayed = true;
		_mouseListeners = new ArrayList();
		
		/* listen for events on the parent which may cascade down to this
		   component */
		_parentComponent.addMouseListener( this );
	}
	
	/**
	 * Create a floating image to be displayed at a given position, relative to
	 * the specified parent component.
	 * <br />
	 * <br />
	 * @param image the image to be rendered
	 * @param location the co-ordinates of the top-left corner of the image,
	 *                 relative to the top-left corner of the parent component
	 * @param parentComponent the parent component
	 */
	public FloatingImage( Image image, Point location, Component parentComponent )
	{
		this( image, location.x, location.y, parentComponent );
	}
	
	/**
	 * Create a floating image to be displayed at the top-left corner of the
	 * specified parent component.
	 * <br />
	 * <br />
	 * @param image the image to be rendered
	 * @param parentComponent the parent component
	 */
	public FloatingImage( Image image, Component parentComponent )
	{
		this( image, DEFAULT_X_POS, DEFAULT_Y_POS, parentComponent );
	}
	
	
	/*
	 * Implementation of MouseListener interface
	 */
	
	public void mouseClicked( MouseEvent e )
	{
		if (this.isAtLocation( e.getPoint() ))
		{
			/* notify all listeners of the click event */
			synchronized (_mouseListeners)
			{
				Iterator itr = _mouseListeners.iterator();
				while (itr.hasNext())
					((MouseListener)itr.next()).mouseClicked( e );
			}
		}
	}
	
	public void mousePressed( MouseEvent e )
	{
		if (this.isAtLocation( e.getPoint() ))
		{
			/* notify all listeners of the press event */
			synchronized (_mouseListeners)
			{
				Iterator itr = _mouseListeners.iterator();
				while (itr.hasNext())
					((MouseListener)itr.next()).mousePressed( e );
			}
		}
	}
	
	public void mouseReleased( MouseEvent e )
	{
		if (this.isAtLocation( e.getPoint() ))
		{
			/* notify all listeners of the release event */
			synchronized (_mouseListeners)
			{
				Iterator itr = _mouseListeners.iterator();
				while (itr.hasNext())
					((MouseListener)itr.next()).mouseReleased( e );
			}
		}
	}
	
	public void mouseEntered( MouseEvent e )
	{
		if (this.isAtLocation( e.getPoint() ))
		{
			/* notify all listeners of the entered event */
			synchronized (_mouseListeners)
			{
				Iterator itr = _mouseListeners.iterator();
				while (itr.hasNext())
					((MouseListener)itr.next()).mouseEntered( e );
			}
		}
	}
	
	public void mouseExited( MouseEvent e )
	{
		if (this.isAtLocation( e.getPoint() ))
		{
			/* notify all listeners of the exit event */
			synchronized (_mouseListeners)
			{
				Iterator itr = _mouseListeners.iterator();
				while (itr.hasNext())
					((MouseListener)itr.next()).mouseExited( e );
			}
		}
	}
	
	
	/*
	 * Implementation of MouseListener notifier
	 */
	
	/**
	 * Register a listener that wants to be notified of mouse events on the
	 * floating image.
	 * <br />
	 * <br />
	 * @param listener object that is to receive notifications
	 */
	public void addMouseListener( MouseListener listener )
	{
		synchronized (_mouseListeners)
		{
			if (!_mouseListeners.contains( listener ))
				_mouseListeners.add( listener );
		}
	}
	
	/**
	 * Un-register a listener that does not want to be notified of mouse events
	 * on the floating image.
	 * <br />
	 * <br />
	 * @param listener object that is no longer to receive notifications
	 */
	public void removeMouseListener( MouseListener listener )
	{
		synchronized (_mouseListeners)
		{
			if (_mouseListeners.contains( listener ))
				_mouseListeners.remove( listener );
		}
	}
	
	
	/*
	 * Access methods
	 */
	
	public void setPosition( Point position )
	{
		setPosition( position.x, position.y );
	}
	
	public void setXPosition( int xPos )
	{
		setPosition( xPos, _yPos );
	}
	
	public void setYPosition( int yPos )
	{
		setPosition( _xPos, yPos );
	}

	public Point getPosition()
	{
		return new Point( _xPos, _yPos );
	}
	
	public int getXPosition()
	{
		return _xPos;
	}
	
	public int getYPosition()
	{
		return _yPos;
	}
	
	public int getWidth()
	{
		return _image.getWidth( new SimpleImageObserver() );
	}
	
	public int getHeight()
	{
		return _image.getHeight( new SimpleImageObserver() );
	}
	
	
	/*
	 * Methods
	 */
	
	/**
	 * Determines if the image sits over a particular point.
	 * <br />
	 * The point should be relative to the top-left corner of the parent
	 * component.
	 * <br />
	 * <br />
	 * @param location a <code>Point</code> on the parent component
	 * @return a flag indicating if the image can be found at the given location
	 */
	public boolean isAtLocation( Point location )
	{
		return (   (_displayed)
				&& (location.x >= _xPos)
				&& (location.x <  _xPos + getWidth())
				&& (location.y >= _yPos)
				&& (location.y <  _yPos + getHeight())  );
	}
	
	/**
	 * Move the image to a given location.
	 * <br />
	 * <br />
	 * @param xPos the x co-ordinate of the left edge of the image, relative to
	 *             the left edge of the parent component
	 * @param yPos the y co-ordinate of the top edge of the image, relative to
	 *             the top edge of the parent component
	 */
	public void setPosition( int xPos, int yPos )
	{
		_xPos = xPos;
		_yPos = yPos;
		
		_parentComponent.repaint();
	}
	
 	/**
 	 * Set the display status of the image.
	 * <br />
	 * <br />
 	 * @param display indicates if the image is to be displayed
 	 */
	public synchronized void setDisplayed( boolean display )
	{
		boolean stateChanged = (_displayed != display);
		
		_displayed = display;
		
		if (stateChanged)
			_parentComponent.repaint();
	}
	
	/**
	 * Render the image.
	 * <br />
	 * <br />
	 * @param g the graphics context onto which the image is to be drawn
	 */
	public void paint( Graphics g )
	{
		if (_displayed)
			g.drawImage( _image, _xPos, _yPos, new SimpleImageObserver() );
	}
	
}


/*
 * Change History
 *
 * $Log: FloatingImage.java,v $
 * Revision 1.3  2006/09/06 21:07:54  Steve
 * Added legal disclaimer.
 *
 * Revision 1.2  2006/06/08 20:05:13  Steve
 * Brought into Bam library:
 *  - moved into lucidviews package
 *  - added CVS headers/footers
 *
 */
