/*
 * JFileSaver.java
 * Part of the Falken project.
 * 
 * Copyright (C)2008 Stephen Battey
 * Contact: resource@lucidviews.net
 */
package net.lucidviews.util.gui.swing;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Font;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

import net.lucidviews.util.io.FilePaths;
import net.lucidviews.util.io.Files;


/**
 * A panel that allows the user to choose a file and, optionally edit its path
 * before hitting the save button.
 * <br />
 * The main benefit of this method of choosing a file is that the user needs only
 * too choose a folder (or an initial file). If the name of the file can be
 * programmatically generated then the user does not need to browser or enter a new
 * file name, just keep hitting the save button.
 * <br />
 * <br />
 * @author Stephen Battey
 * @version $Revision: 1.1.2.1 $
 * @since 1.3
 */
public class JFileSaver extends JPanel implements ActionListener
{
	
	/*
	 * Exported constants
	 */
	
	/** The id of save events. */
	public static final int SAVE_ID = 463295;
	
	/** The command associated with save events. */
	public static final String SAVE_COMMAND = "save";
	
	
	/*
	 * Local constants
	 */
	
	/** Serialisation version id. */
	private static final long serialVersionUID = 7392555196376284391L;
	
	
	/*
	 * Instance variables
	 */
	
	/** The text field used to display/input the file path. */
	protected JTextField path;
	
	/** Button that opens the file chooser dialog. */
	protected JButton chooseButton;
	
	/** Button that triggers the file to be saved. */
	protected JButton saveButton;
	
	/** The listener that will react to the save button being pressed,
	 *  or <code>null</code> if no listener is registered. */
	protected ActionListener saveListener;
	
	
	/*
	 * Constructors
	 */
	
	/**
	 * Create a file chooser panel, defaulting to the given file location.
	 * <br />
	 * <br />
	 * @param filePath the initial path
	 */
	public JFileSaver( String filePath )
	{
		this.path = new JTextField();
		this.chooseButton = new JButton( "choose" );
		this.saveButton = new JButton( "save" );
		
		this.path.setEditable( false );
		this.path.setMargin( new Insets( 1, 1, 1, 1 ) );
		this.path.setFont( new Font( "courier", Font.PLAIN, 12 ) );
		
		this.chooseButton.addActionListener( this );
		this.saveButton.addActionListener( this );
		
		JPanel buttonPanel = new JPanel();
		buttonPanel.add( this.chooseButton );
		buttonPanel.add( this.saveButton );
		
		setLayout( new BorderLayout() );
		add( this.path, BorderLayout.CENTER );
		add( buttonPanel, BorderLayout.EAST );
		
		// setFile must be called after the UI components have been created
		setFile( new File( filePath ) );
	}
	
	
	/*
	 * Methods
	 */
	
	/**
	 * Decide whether the user can hand-edit the file path.
	 * <br />
	 * <br />
	 * @param editable <code>true</code> if the path can be edited,
	 *                 <code>false</code> if the user must use the file chooser
	 */
	public void setEditable( boolean editable )
	{
		this.path.setEditable( editable );
	}
	
	/**
	 * Set the save listener.
	 * <br />
	 * <br />
	 * @param listener the listener that will save the file
	 * @see #saveListener
	 */
	public void setSaveListener( ActionListener listener )
	{
		this.saveListener = listener;
	}
	
	/**
	 * Get the chosen file.
	 * <br />
	 * <br />
	 * @return the file that has been chosen,
	 *         which may not exist or may point to a folder
	 */
	public File getFile()
	{
		return new File( this.path.getText() );
	}
	
	/**
	 * Check if the current file chosen is valid.
	 * <br />
	 * <br />
	 * @return <code>true</code> if the file returned from <code>getFile</code> can be written to
	 */
	public boolean isValidFile()
	{
		return (!getFile().isDirectory());
	}
	
	/**
	 * Set the file chosen.
	 * <br />
	 * <br />
	 * @param file the file to be chosen
	 */
	public void setFile( File file )
	{
		this.path.setText( file.getAbsolutePath() );
		
		// move the caret to the end of the field so the file name is visible
		this.path.setCaretPosition( this.path.getText().length() );
		
		this.saveButton.setEnabled( isValidFile() );
	}
	
	/**
	 * Change the name of the file chosen.
	 * <br />
	 * Nominally, the new file will be in the same folder as the current file. This
	 * allows the file name to be changed, while keeping the user's chosen folder.
	 * <br />
	 * <br />
	 * @param fileName the name of the file
	 */
	public void setFileName( String fileName )
	{
		File file = getFile();
		File parentFolder = file.getParentFile();
		if (file.isFile())
		{
			setFile( new File( parentFolder, fileName ) );
		}
		else if (file.isDirectory())
		// important to check this first, before checking if the parent folder exists
		{
			setFile( new File( file, fileName ) );
		}
		else if (   (parentFolder != null)
		         && (parentFolder.isDirectory())  )
		{
			setFile( new File( parentFolder, fileName ) );
		}
		else
		{
			setFile( new File( file, fileName ) );
		}
	}
	
	/**
	 * Open a dialog to allow the user to choose a file.
	 * <br />
	 * <br />
	 * @return <code>JFileChooser.APPROVE_OPTION</code> if the user confirmed a new file location
	 */
	public int chooseFile()
	{
		File file = getFile();
		File existingParent = Files.getExistingParentFile( file );
		File chooseFromFolder = (existingParent != null) ? existingParent : new File( "." );
		
		JFileChooser chooser = new JFileChooser();
		chooser.setCurrentDirectory( chooseFromFolder );
		
		int chooseResult = chooser.showSaveDialog( this );
		if (chooseResult == JFileChooser.APPROVE_OPTION)
		{
			setFile( chooser.getSelectedFile() );
		}
		
		return chooseResult;
	}
	
	/**
	 * Check the file path entered is valid.
	 * <br />
	 * The path in the UI will be changed to ensure a file can be written to that location,
	 * provided the user agrees to those changes.
	 * <br />
	 * <br />
	 * @return <code>true</code> if the file path is now valid,
	 *         will be <code>false</code> if the user has chosen to cancel
	 *         the changes needed to make the file path valid
	 * @throws IOException if the file path could not be validated
	 */
	public boolean validateFile() throws IOException
	{
		File file = getFile();
		
		// It's important we check the file path is valid (doesn't contain special characters)
		// before making any other checks (which would assume the path is valid).
		File validatedFile = FilePaths.validateFilePath( file );
		boolean fileIsValid;
		try
		{
			fileIsValid = validatedFile.getCanonicalPath().equals( file.getCanonicalPath() );
		}
		catch (IOException e)
		{
			// path is invalid is the canonical path of the original 'file' cannot be created
			fileIsValid = false;
		}
		if (!fileIsValid)
		{
			
			String chosenFilePath = JOptionPane.showInputDialog( this,
			                                                     "The file path is invalid - it cannot contain any of the following characters." +
			                                                       "\n " + FilePaths.SPECIAL_CHARACTERS +
			                                                       "\n\n Please confirm a valid file path:",
			                                                     validatedFile.getPath() );
			if (chosenFilePath == null)
			{
				// user has cancelled or aborted: can't confirm the path is valid
				return false;
			}
			
			File chosenFile = new File( chosenFilePath );
			setFile( chosenFile );
			file = chosenFile;
		}
		
		// From here on it's only possible change the file using the FileChooser dialog, which
		// does not allow invalid file paths to be entered. So there's no need to re-check the
		// file path.
		
		while (file.isDirectory())
		{
			int decision = JOptionPane.showConfirmDialog( this,
			                                              "The file location chosen points to an existing directory.\nPlease choose another location.",
			                                              "Invalid file path",
			                                              JOptionPane.OK_CANCEL_OPTION,
			                                              JOptionPane.QUESTION_MESSAGE );
			if (decision != JOptionPane.OK_OPTION)
			{
				// user has cancelled or aborted: can't confirm the path is valid
				return false;
			}
			
			int newChoiceResult = chooseFile();
			if (newChoiceResult != JFileChooser.APPROVE_OPTION)
			{
				// user has cancelled or aborted: can't confirm the path is valid
				return false;
			}
			
			file = getFile();
		}
		
		return true;
	}
	
	/**
	 * Trigger the file to be saved.
	 * <br />
	 * The file path entered into the UI will be validated before any listeners are asked
	 * to perform the save. If validation fails the file will not be saved.
	 */
	public void saveFile()
	{
		try
		{
			validateFile();
		}
		catch (IOException e)
		{
			int decision = JOptionPane.showConfirmDialog( this,
                                                          "The file location could not be validated.\nThe location could be invalid or not accessible.\nTry to save anyway?",
                                                          "Error validating file location",
                                                          JOptionPane.ERROR_MESSAGE );
			if (decision != JOptionPane.OK_OPTION)
			{
				return;
			}
			// Otherwise, ignore the error and try to continue.
		}
		
		if (!ensureParentFolderExists( getFile(), this ))
		{
			return;
		}
		
		if (this.saveListener != null)
		{
			this.saveListener.actionPerformed( new ActionEvent( this, SAVE_ID, SAVE_COMMAND ) );
		}
	}
	
	
	/*
	 * Implementation of ActionListener interface
	 */
	
	/** {@inheritDoc} */
	public void actionPerformed( ActionEvent e )
	{
		if (e.getSource() == this.saveButton)
		{
			saveFile();
		}
		
		if (e.getSource() == this.chooseButton)
		{
			chooseFile();
		}
	}
	
	
	/*
	 * Utility methods
	 */
	
	/**
	 * Ensure the parent folder for a given file exists.
	 * <br />
	 * If it does not the user is asked whether it should be created.
	 * <br />
	 * <br />
	 * @param file the target file
	 * @param component the UI component used to associate question dialogs
	 * @return <code>true</code> if the parent folder for the given file exists
	 */
	public static boolean ensureParentFolderExists( File file, Component component )
	{
		File folder = file.getParentFile();
		if (   (folder != null)
		    && (!folder.exists())  )
		{
			int decision = JOptionPane.showConfirmDialog( component,
			                                              "Folder does not exist.\nCreate now?",
			                                              "Create folder?",
			                                              JOptionPane.OK_CANCEL_OPTION,
			                                              JOptionPane.QUESTION_MESSAGE );
			if (decision != JOptionPane.OK_OPTION)
			{
				// user has cancelled or aborted: can't save file if folder does not exist
				return false;
			}
			
			folder.mkdirs();
			
			if (!folder.exists())
			{
				// can't save file if we can't create the parent folder
				JOptionPane.showMessageDialog( component,
				                               "The folder could not be created.\n" + folder.getAbsolutePath(),
				                               "Can't create folder",
				                               JOptionPane.ERROR_MESSAGE );
				return false;
			}
		}
		
		return true;
	}
	
}


/*
 * Change History
 *
 * $Log: JFileSaver.java,v $
 * Revision 1.1.2.1  2008/02/24 20:20:57  Steve
 * Initial revision.
 *
 * Revision 1.4  2008/02/08 18:45:16  Steve
 * Major change to data storage - the File variable has been removed, the text box is now treated as the master copy of the file path.
 * Moved validation of file path into a separate method.
 * A valid alternative is suggested if the file path is not valid.
 *
 * Revision 1.3  2008/01/12 10:06:28  Steve
 * Implemented missing logic in setFileName method.
 *
 * Revision 1.2  2008/01/05 21:48:57  Steve
 * Added file validation handling, including
 *  - creating the parent folder if it does not exist
 *  - disabling the save button if file path is not valid
 *
 * Revision 1.1  2008/01/05 19:26:41  Steve
 * Initial revision.
 *
 */