/*
 * JFileSaverTest.java
 * Part of the Falken project.
 * 
 * Copyright (C)2008 Stephen Battey
 * Contact: resource@lucidviews.net
 */
package net.lucidviews.util.gui.swing;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;

import javax.swing.JOptionPane;

import net.lucidviews.util.io.FilePaths;


/**
 * A test harness for the <code>JFileSaver</code> class.
 * <br />
 * <br />
 * @author Stephen Battey
 * @version $Revision: 1.1.2.1 $
 * @since 0.2
 */
public class JFileSaverTest
{
	
	/*
	 * Local constants
	 */
	
	/** Location of the test data. */
	protected static final File TEST_FOLDER = new File( "./Test/JFileSaver" );
	
	/** Location of an existing folder. */
	protected static final File EXISTING_FOLDER = new File( TEST_FOLDER, "folder" );
	
	/** Location of a non-existant folder. */
	protected static final File NON_EXISTING_FOLDER = new File( TEST_FOLDER, "folder-does-not-exist" );
	
	/** Location of an existing file. */
	protected static final File EXISTING_FILE = new File( TEST_FOLDER, "file.txt" );
	
	/** Location of a non-existant file in a non-existant folder. */
	protected static final File NON_EXISTING_FILE_IN_NON_EXISTING_FOLDER = new File( NON_EXISTING_FOLDER, "file-does-not-exist" );
	
	
	/*
	 * Test harness
	 */
	
	/**
	 * Run the test.
	 * <br />
	 * <br />
	 * @param args not used
	 * @throws IOException if one or more tests could not be performed
	 */
	public static void main( String[] args ) throws IOException
	{
		System.out.println( "JFileSaver Test: Start" );
		
		testListenerIsNotified();
		testChangeFile();
		testSetFileName();
		testSaveFileWithInvalidName();
		testSaveFileInNonExistingFolder();
		testSaveAsExistingFolder();

		System.out.println( "JFileSaver Test: Complete" );
	}
	
	/**
	 * Test that listeners are notified when the file is saved.
	 */
	public static void testListenerIsNotified()
	{
		JFileSaver testSaver = new JFileSaver( "dummy" );
		
		SaveListener dummyListener = new SaveListener();
		dummyListener.expectedSource = testSaver;
		testSaver.setSaveListener( dummyListener );
		
		if (dummyListener.notified)
			System.out.println( "FAIL testListenerIsNotified 1: " );
		
		testSaver.saveFile();
		
		if (!dummyListener.notified)
			System.out.println( "FAIL testListenerIsNotified 2: " );
	}
	
	/**
	 * Test that the file path can be set using the setFile method.
	 */
	public static void testChangeFile()
	{
		JFileSaver testSaver = new JFileSaver( "dummy" );
		
		File dummyFile = new File( "./some/example/path.txt" );
		testSaver.setFile( dummyFile );
		
		File resultFile = testSaver.getFile();
		
		if (!dummyFile.getAbsolutePath().equals( resultFile.getAbsolutePath() ))
			System.out.println( "FAIL testChangeFile 1: " );
	}
	
	/**
	 * Test the various combinations of setting the file name.
	 * <br />
	 * <br />
	 * @throws IOException if this test cannot be performed
	 */
	public static void testSetFileName() throws IOException
	{
		JFileSaver testSaver = new JFileSaver( "dummy" );
		
		// current file exists, so the file name replaces name of existing file
		testSaver.setFile( EXISTING_FILE );
		testSaver.setFileName( "other.txt" );
		assertEquals( testSaver.getFile(), new File( EXISTING_FILE.getParentFile(), "other.txt" ), "testSetFileName 1" );
		
		// current file is an existing folder, so the file name is appended to the folder path
		testSaver.setFile( EXISTING_FOLDER );
		testSaver.setFileName( "other.txt" );
		assertEquals( testSaver.getFile(), new File( EXISTING_FOLDER, "other.txt" ), "testSetFileName 2" );
		
		// current file does not exist but parent folder does, so the file name replaces name of non-existing file
		testSaver.setFile( NON_EXISTING_FOLDER );
		testSaver.setFileName( "other.txt" );
		assertEquals( testSaver.getFile(), new File( NON_EXISTING_FOLDER.getParentFile(), "other.txt" ), "testSetFileName 3" );
		
		// (default case)
		// current file does not exist nor does the parent folder, so the file name is appended to the current path
		testSaver.setFile( NON_EXISTING_FILE_IN_NON_EXISTING_FOLDER );
		testSaver.setFileName( "other.txt" );
		assertEquals( testSaver.getFile(), new File( NON_EXISTING_FILE_IN_NON_EXISTING_FOLDER, "other.txt" ), "testSetFileName 4" );
	}
	
	/**
	 * Verify that the JFileSaver offers the user the chance to fix an invalid file name.
	 * <br />
	 * THIS TEST REQUIRES USER INPUT.
	 * <br />
	 * <br />
	 * @throws IOException if this test cannot be performed
	 */
	public static void testSaveFileWithInvalidName() throws IOException
	{
		final String specialCharacters = getSpecialCharactersForFileName();
		
		JFileSaver testSaver = new JFileSaver( "dummy" );
		
		testSaver.setFile( new File( EXISTING_FOLDER, "some-file\"like\".txt" ) );
		JOptionPane.showMessageDialog( testSaver,
		                               "Answer OK to the following question.",
		                               "TEST HARNESS",
		                               JOptionPane.INFORMATION_MESSAGE );
		testSaver.saveFile();
		assertEquals( testSaver.getFile(), new File( EXISTING_FOLDER, "some-file'like'.txt" ), "testSaveFileWithInvalidName 0" );
		
		testSaver.setFile( new File( EXISTING_FOLDER, "some-file" + specialCharacters + ".txt" ) );
		JOptionPane.showMessageDialog( testSaver,
		                               "Answer OK to the following question.",
		                               "TEST HARNESS",
		                               JOptionPane.INFORMATION_MESSAGE );
		testSaver.saveFile();
		assertEquals( testSaver.getFile(), new File( EXISTING_FOLDER, "some-file'.txt" ), "testSaveFileWithInvalidName 1" );
		
		testSaver.setFile( new File( EXISTING_FOLDER, "some-" + specialCharacters + "other\"-file.txt" ) );
		JOptionPane.showMessageDialog( testSaver,
		                               "Answer OK to the following question.",
		                               "TEST HARNESS",
		                               JOptionPane.INFORMATION_MESSAGE );
		testSaver.saveFile();
		assertEquals( testSaver.getFile(), new File( EXISTING_FOLDER, "some-'other'-file.txt" ), "testSaveFileWithInvalidName 2" );
	}
	
	/**
	 * Verify that the JFileSaver offers the user the chance to create a non-existing file.
	 * <br />
	 * THIS TEST REQUIRES USER INPUT.
	 */
	public static void testSaveFileInNonExistingFolder()
	{
		JFileSaver testSaver = new JFileSaver( "dummy" );
		testSaver.setFile( NON_EXISTING_FILE_IN_NON_EXISTING_FOLDER );
		
		File targetFolder = NON_EXISTING_FILE_IN_NON_EXISTING_FOLDER.getParentFile();
		if (targetFolder.exists())
			System.out.println( "FAIL testSaveFileInNonExistingFolder 1: the following folder should NOT exist! " + targetFolder.getAbsolutePath() );
		
		JOptionPane.showMessageDialog( testSaver,
		                               "Answer OK to the following question.",
		                               "TEST HARNESS",
		                               JOptionPane.INFORMATION_MESSAGE );
		
		testSaver.saveFile();
		
		if (!targetFolder.exists())
			System.out.println( "FAIL testSaveFileInNonExistingFolder 2: the following folder was not created. " + targetFolder.getAbsolutePath() );
		
		targetFolder.delete();
		if (targetFolder.exists())
			System.out.println( "FAIL testSaveFileInNonExistingFolder 3: could not delete the test folder: " + targetFolder.getAbsolutePath() );
	}
	
	/**
	 * Verify that the JFileSaver offers the user a chance to pick another file if the
	 * file selected is an existing folder.
	 * <br />
	 * THIS TEST REQUIRES USER INPUT.
	 * <br />
	 * <br />
	 * @throws IOException if this test cannot be performed
	 */
	public static void testSaveAsExistingFolder() throws IOException
	{
		final File folder = EXISTING_FOLDER;
		final String differentFileName = "gotest";
		
		JFileSaver testSaver = new JFileSaver( "dummy" );
		testSaver.setFile( folder );
		
		JOptionPane.showMessageDialog( testSaver,
                                       "Answer Cancel to the following warning.",
                                       "TEST HARNESS",
                                       JOptionPane.INFORMATION_MESSAGE );
		testSaver.saveFile();
		
		JOptionPane.showMessageDialog( testSaver,
                                       "Answer OK, to the following warning." +
                                         "\n A file select dialog should then appear, press Cancel.",
                                       "TEST HARNESS",
                                       JOptionPane.INFORMATION_MESSAGE );
		testSaver.saveFile();
		
		JOptionPane.showMessageDialog( testSaver,
                                       "Answer OK, to the following warning." +
                                         "\n A file select dialog should then appear, enter file name '" + differentFileName + "' and press Save.",
                                       "TEST HARNESS",
                                       JOptionPane.INFORMATION_MESSAGE );
		testSaver.saveFile();
		assertEquals( testSaver.getFile(), new File( folder, differentFileName ), "testSaveAsExistingFolder 1" );
	}
	
	
	/*
	 * Helper methods
	 */
	
	/**
	 * Get the set of special characters for this OS, but without the file path separator.
	 * <br />
	 * This is the <code>JFileSaver.SPECIAL_CHARACTERS</code> value, modified to remove any
	 * characters that could be interpreted as a file path separator, which cannot be used
	 * in file names (otherwise they would split the file name into a sub-folder and file name).
	 * <br />
	 * <br />
	 * @return special characters that can be used in a file name
	 */
	public static String getSpecialCharactersForFileName()
	{
		StringBuffer buffer = new StringBuffer( FilePaths.SPECIAL_CHARACTERS );
		
		int fileSeparatorIndex = buffer.indexOf( File.separator );
		while (fileSeparatorIndex > -1)
		{
			buffer.deleteCharAt( fileSeparatorIndex );
			
			fileSeparatorIndex = buffer.indexOf( File.separator );
		}
		
		int javaFileSeparatorIndex = buffer.indexOf( "/" );
		while (javaFileSeparatorIndex > -1)
		{
			buffer.deleteCharAt( javaFileSeparatorIndex );
			
			javaFileSeparatorIndex = buffer.indexOf( "/" );
		}
		
		return buffer.toString();
	}
	
	/**
	 * Check if two files point to the same location.
	 * <br />
	 * <br />
	 * @param file1 the first file
	 * @param file2 the other file
	 * @param testId a message shown to identify the test, should it fails
	 * @throws IOException if the file paths cannot be resolved into canonical form
	 */
	public static final void assertEquals( File file1, File file2, String testId ) throws IOException
	{
		if (!file1.getCanonicalPath().equals( file2.getCanonicalPath() ))
		{
			System.out.println( "FAIL " + testId + ": " + file1.getCanonicalPath() + "  /=  " + file2.getCanonicalPath() );
		}
	}
	
	/**
	 * Check if two strings are the same.
	 * <br />
	 * <br />
	 * @param message a message shown to identify the test, should it fails
	 * @param expected the expected value
	 * @param actual the actual value
	 */
	public static final void assertEquals( String message, String expected, String actual )
	{
		if (!actual.equals( expected ))
		{
			System.out.println( "FAIL " + message + ": " + expected + "  /=  " + actual );
		}
	}
	
	
	/*
	 * Helper classes
	 */
	
	/**
	 * A listener that flags when it has been notified of a save.
	 */
	protected static final class SaveListener implements ActionListener
	{
		
		/** The expected source of the action event. */
		public Object expectedSource;
		
		/** Flag indicating if this listener has been notified of the save. */
		public boolean notified;
		
		/** {@inheritDoc} */
		public void actionPerformed( ActionEvent e )
		{
			if (e.getSource() == this.expectedSource)
			{
				this.notified = true;
			}
		}
		
	}
	
}


/*
 * Change History
 *
 * $Log: JFileSaverTest.java,v $
 * Revision 1.1.2.1  2008/02/24 20:20:57  Steve
 * Initial revision.
 *
 * Revision 1.1  2008/02/08 18:17:51  Steve
 * Initial revision.
 *
 */