/*
 * JWebAddress.java
 * Part of the B'am library - a collection of general-purpose utility classes.
 * 
 * Copyright (C)2005-2008 Stephen Battey
 * Contact: resource@lucidviews.net
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.lucidviews.util.gui.swing;

import net.lucidviews.util.gui.swing.dnd.DefaultSwingDropTarget;
import net.lucidviews.util.text.Strings;

import javax.swing.border.EmptyBorder;
import javax.swing.border.Border;
import javax.swing.JOptionPane;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JPanel;
import java.awt.datatransfer.DataFlavor;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DnDConstants;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.FocusListener;
import java.awt.event.FocusEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.BorderLayout;
import java.net.URL;
import java.net.MalformedURLException;
import java.util.Collection;
import java.util.ArrayList;
import java.util.Iterator;


/**
 * A label and text-field composite for entering a web address.
 * <br />
 * <br />
 * @author Stephen Battey
 * @version $Revision: 1.1.2.1 $
 * @since 1.3
 */
public class JWebAddress extends JPanel implements ActionListener, FocusListener, KeyListener
{
	
	/*
	 * Local constants
	 */
	
	/** Serialisation version id. */
	private static final long serialVersionUID = -1843481009247561553L;
	
	/** The default label, used when none is specified. */
	private static final String DEFAULT_LABEL = "Address:";
	
	/** The default border for the label. */
	private static final Border DEFAULT_LABEL_BORDER = new EmptyBorder( 3, 40, 3, 4 );
	
	
	/*
	 * Instance variables
	 */
	
	/** Registered listeners, waiting for notification of a change of address. */
	protected Collection<ActionListener> listeners;
	
	/** The last confirmed address.
	 *  This may not match the address currently typed into the input box.
	 *  @see #inputField*/
	protected URL currentAddress;
	
	/** The label that prompts the user to enter a web address. */
	protected JLabel label;
	
	/** The input field into which the user types the web address.
	 *  NB: the text in the input field will only match the current address after
	 *      the user confirms the address (by pressing return) and the parent component
	 *      confirms the address exists and is now being used.
	 *  @see #currentAddress */
	protected JTextField inputField;
	
	
	/*
	 * Constructors
	 */
	
	/**
	 * Create a component for the input of a web address.
	 * <br />
	 * The component will display the specified text as a tool tip.
	 * The default border and label will be used.
	 * <br />
	 * <br />
	 * @param toolTipText text to display when user hovers their pointer over the web address
	 */
	public JWebAddress( String toolTipText )
	{
		this( toolTipText, DEFAULT_LABEL, DEFAULT_LABEL_BORDER );
	}
	
	/**
	 * Create a component for the input of a web address.
	 * <br />
	 * The component will display the specified tool tip and label.
	 * The default border will be used.
	 * <br />
	 * <br />
	 * @param toolTipText text to display when user hovers their pointer over the web address
	 * @param label text to display beside the input box, it should describe what to enter into the address bar
	 */
	public JWebAddress( String toolTipText, String label )
	{
		this( toolTipText, label, DEFAULT_LABEL_BORDER );
	}
	
	/**
	 * Create a component for the input of a web address.
	 * <br />
	 * The component will display the specified tool tip.
	 * The default label will be used.
	 * The default border will be used around the label.
	 * <br />
	 * <br />
	 * @param toolTipText text to display when user hovers their pointer over the web address
	 * @param labelBorder border applied around the label
	 */
	public JWebAddress( String toolTipText, Border labelBorder )
	{
		this( toolTipText, DEFAULT_LABEL, labelBorder );
	}
	
	/**
	 * Create a component for the input of a web address.
	 * <br />
	 * The component will display the specified tool tip and label.
	 * <br />
	 * <br />
	 * @param toolTipText text to display when user hovers their pointer over the web address
	 * @param label text to display beside the input box, it should describe what to enter into the address bar
	 * @param labelBorder border applied around the label
	 */
	public JWebAddress( String toolTipText, String label, Border labelBorder )
	{
		super( new BorderLayout() );
		
		this.listeners = new ArrayList<ActionListener>();
		this.currentAddress = null;
		
		this.label = new JLabel( label, JLabel.RIGHT );
		this.label.setBorder( labelBorder );
		this.label.setFocusable( true );
		add( this.label, BorderLayout.WEST );
		
		this.inputField = new JTextField( 100 );
		this.inputField.setToolTipText( toolTipText );
		this.inputField.setDropTarget( new WebAddressDropTarget( this ) );
		this.inputField.addActionListener( this );
		this.inputField.addFocusListener( this );
		this.inputField.addKeyListener( this );
		add( this.inputField, BorderLayout.CENTER );
	}
	
	
	/*
	 * Methods
	 */
	
	/**
	 * Register a listener that wants to be notified of interactions with the
	 * web address.
	 * <br />
	 * <br />
	 * @param listener object that is to receive notifications
	 */
	public void addActionListener( ActionListener listener )
	{
		synchronized (this.listeners)
		{
			this.listeners.add( listener );
		}
	}
	
	/**
	 * Un-register a listener that does not want to be notified of interactions
	 * with the web address.
	 * <br />
	 * <br />
	 * @param listener object that is no longer to receive notifications
	 */
	public void removeActionListener( ActionListener listener )
	{
		synchronized (this.listeners)
		{
			this.listeners.remove( listener );
		}
	}
	
	/**
	 * Get the current address.
	 * <br />
	 * NB: this is the last verified address, not the address being entered by the
	 * user into the input field.
	 * <br />
	 * <br />
	 * @return the most address confirmed as being used
	 */
	public URL getAddress()
 	{
		return this.currentAddress;
 	}
	
	/**
	 * Reset the text displayed to the last confirmed URL (confirmed by a call to
	 * <code>setAddress</code>.
	 * <br />
	 * <br />
	 * Call this method to reset user edits.
	 */
	public void resetAddress()
	{
		resetAddress( this.currentAddress );
	}
	
	/**
	 * Confirm the URL being used.
	 * <br />
	 * Call this method when handling a <code>WebAddressActionEvent</code> to
	 * confirm that the URL entered by the user is valid and is being used.
	 * <br />
	 * <br />
	 * @param address the URL of the file that is now being used
	 */
	public void resetAddress( URL address )
	{
		this.currentAddress = address;
		
		if (this.currentAddress == null)
			this.inputField.setText( "" );
		else
			this.inputField.setText( this.currentAddress.toString() );
		
		// move focus away from the input field
		// Should the user want to change the address they will have to click on the
		// input field, causing it to (re)gain focus and causing the focus handler
		// to highlight/select the entire field.
		this.label.requestFocus();
	}
	
	/**
	 * Set the web address.
	 * <br />
	 * NB: this only changes the address in the address bar it does not submit the address.
	 * To submit the address you will need to call the <code>go</code> method.
	 * <br />
	 * <br />
	 * @param address the web address
	 * @see #go()
	 */
	public void setAddress( String address )
	{
		this.inputField.setText( address );
	}
	
	/**
	 * Go to the location that has been entered into this web address component.
	 */
	public void go()
	{
		go( 0 );
	}
	
	/**
	 * Go to the location that has been entered into this web address component.
	 * <br />
	 * <br />
	 * @param modifiers the modifier keys (Alt, Ctrl, etc)
	 *                  that were pressed when the command to 'go' was received
	 */
	public void go( int modifiers )
	{
		try
		{
			URL newAddress = new URL( this.inputField.getText() );
			
			// remove focus from the address box to avoid the action being submitted twice
			this.label.requestFocus();
			
			// notify all listeners
			WebAddressActionEvent webAddressEvent
				= new WebAddressActionEvent( this, WebAddressActionEvent.COMMAND_GO,
				                             System.currentTimeMillis(), modifiers, newAddress );
			synchronized (this.listeners)
			{
				Iterator<ActionListener> itr = this.listeners.iterator();
				while (itr.hasNext())
					itr.next().actionPerformed( webAddressEvent );
			}
		}
		catch (MalformedURLException ex)
		{
			JOptionPane.showMessageDialog( this, "The address '" + this.inputField.getText() + "' is not a valid URL.",
			                               "Invalid address", JOptionPane.ERROR_MESSAGE );
		}
	}
	
	
	/*
	 * Implementation of ActionListener interface
	 */
	
	/** {@inheritDoc} */
	public void actionPerformed( ActionEvent e )
	{
		if (!isEnabled())
			return;
		
		if (e.getSource() != this.inputField)
			return;
		
		go( e.getModifiers() );
	}
	
	
	/*
	 * Implementation of FocusListener interface
	 */

	/** {@inheritDoc} */
	public void focusGained( FocusEvent e )
	{
		/* select the entire address string */
		this.inputField.setSelectionStart( 0 );
		this.inputField.setSelectionEnd( this.inputField.getText().length() );
	}

	/** {@inheritDoc} */
	public void focusLost( FocusEvent e )
	{
	}
	
	
	/*
	 * Implementation of KeyListener interface
	 */
	
	/** {@inheritDoc} */
	public void keyTyped( KeyEvent e )
	{
	}
	
	/** {@inheritDoc} */
	public void keyPressed( KeyEvent e )
	{
		if (e.getKeyCode() == KeyEvent.VK_ESCAPE)
			resetAddress();
	}
	
	/** {@inheritDoc} */
	public void keyReleased( KeyEvent e )
	{
	}
	
	
	/*
	 * Helper classes
	 */
	
	/**
	 * An <code>ActionEvent</code> specific to the <code>JWebAddress</code>
	 * component.
	 * <br />
	 * <code>ActionListener</code>s assigned to a <code>JWebAddress</code> object
	 * can expect to receive a <code>WebAddressActionEvent</code> as the event
	 * parameter in calls to the <code>ActionListener::actionPerformed()</code>
	 * method.
	 */
	public static class WebAddressActionEvent extends ActionEvent
	{
		
		/** Serialisation version. */
		private static final long serialVersionUID = -6512991428482876064L;
		
		/** The id of this action type. */
		public static final int ACTION_ID = 5464573;
		
		/** The (textual) command associated with this action type. */
		public static final String COMMAND_GO = "go";
		
		/** The URL of the web address . */
		protected URL webAddress;
		
		/**
		 * Create an action event to process a web address.
		 * <br />
		 * <br />
		 * @param source the object that originated the event
		 * @param command a string that may specify a command (possibly one 
		 *                of several) associated with the event
		 * @param webAddress the URL of the web address
		 */
		public WebAddressActionEvent( Object source, String command, URL webAddress )
		{
			super( source, ACTION_ID, command );
			this.webAddress = webAddress;
		}

		/**
		 * Create an action event to process a web address.
		 * <br />
		 * <br />
		 * @param source the object that originated the event
		 * @param command a string that may specify a command (possibly one 
		 *                of several) associated with the event
		 * @param when the time the event occurred
		 * @param modifiers the modifier keys held down during this action
		 * @param webAddress the URL of the web address
		 */
		public WebAddressActionEvent( Object source, String command,
		                              long when, int modifiers, URL webAddress )
		{
			super( source, ACTION_ID, command, when, modifiers );
			this.webAddress = webAddress;
		}
		
		/**
		 * Get the URL of the web address that was entered to trigger this event.
		 * <br />
		 * <br />
		 * @return the URL entered by the user
		 */
		public URL getWebAddress()
		{
			return this.webAddress;
		}
		
	}
	
	/**
	 * A <code>DropTarget</code> that accepts addresses dropped onto the input
	 * field.
	 * <br />
	 * When an address is dropped onto the input field it will replace any text
	 * currently in the address box with the new text and highlight the field
	 * ready for the user to press return and go to the new address.
	 */
	public static class WebAddressDropTarget extends DefaultSwingDropTarget
	{

		/** Serialisation version. */
		private static final long serialVersionUID = 8183501853823908677L;
		
		/** The component that will receive the URL being dropped. */
		protected JWebAddress webAddressComponent;
		
		/**
		 * Create a drop target for a web address component.
		 * <br />
		 * <br />
		 * @param webAddressComponent the component that will receive the URLs dropped onto this target
		 */
		public WebAddressDropTarget( JWebAddress webAddressComponent )
		{
			super( webAddressComponent, (DnDConstants.ACTION_COPY_OR_MOVE | DnDConstants.ACTION_LINK) );
			this.webAddressComponent = webAddressComponent;
		}
		
		/** {@inheritDoc} */
		public void drop( DropTargetDropEvent event )
		{
			super.drop( event );
			try
			{
				String text = (String)event.getTransferable().getTransferData( DataFlavor.stringFlavor );
				String singleLine = Strings.firstLine( text );
				
				this.webAddressComponent.inputField.setText( singleLine );
				this.webAddressComponent.inputField.requestFocus();
				
				event.dropComplete( true );
			}
			catch (Throwable e)
			{
				event.dropComplete( false );
			}
		}
		
	}
	
}


/*
 * Change History
 *
 * $Log: JWebAddress.java,v $
 * Revision 1.1.2.1  2008/02/18 22:02:41  Steve
 * Initial revision (taken from Falken project, originally from Bob project).
 *
 * Revision 1.1  2007/12/31 18:24:09  Steve
 * *** empty log message ***
 *
 * Original Revision 2005/11/24
 *
 */
