/*
 * DefaultDropTarget.java
 * Part of the B'am library - a collection of general-purpose utility classes.
 * 
 * Copyright (C)2005-2008 Stephen Battey
 * Contact: resource@lucidviews.net
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

package net.lucidviews.util.gui.swing.dnd;

import java.awt.datatransfer.DataFlavor;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetEvent;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DnDConstants;
import java.awt.Component;


/**
 * An extension of the <code>DropTarget</code> class that invokes state
 * changes on the drag'n'drop event.
 * <br />
 * <br />
 * This class must be sub-classed and the drag/drop methods overridden with
 * custom implementations. NB: overridden methods must call the super method
 * to invoke the state change on the event.
 * <br />
 * Sub classes will probably want to override the
 * <code>dataFlavourSupported</code> too, to control the type of data being
 * dropped onto the target.
 * <br />
 * <br />
 * @author Stephen Battey
 * @version $Revision: 1.1.2.1 $
 * @since 1.3
 */
public class DefaultDropTarget extends DropTarget
{
	
	/*
	 * Local constants
	 */
	
	/** Serialisation version id. */
	private static final long serialVersionUID = -9212470261637617473L;
	
	
	/*
	 * Instance variables
	 */
	
	/** A bit mask of flags for actions supported by this target. */
	protected int supportedActions;
	
	/** A flag to remember whether import is being allowed for each drop event sent to this target. */
	protected boolean canImport;
	
	
	/*
	 * Constructors
	 */
	
	/**
	 * Create a drop target for the specified component.
	 * <br />
	 * <br />
	 * @param component the GUI component that is to accept DnDs
	 * @param supportedActions the actions that will be supported - a bit mask of DnDConstants
	 */
	protected DefaultDropTarget( Component component, int supportedActions )
	{
		super();
		this.supportedActions = supportedActions;
		
		setComponent( component );
	}
	
	
	/*
	 * Methods
	 */
	
	/**
	 * Check if an action is supported by this drop target.
	 * <br />
	 * <br />
	 * @param action the action is taking place - a bit mask of DnDConstants
	 * @return <code>true</code> if the action is supported by this drop target
	 */
	private boolean actionSupported( int action )
	{
		return (action & this.supportedActions) != DnDConstants.ACTION_NONE;
	}
	
	/**
	 * Check if a data type is accepted by this drop target.
	 * <br />
	 * By default, all data flavours are supported. This method can be overridden
	 * to change that behaviour.
	 * <br />
	 * <br />
	 * @param component the component being targeted
	 * @param flavours the data flavour(s) of the incoming drag'n'drop event
	 * @return a Boolean value indicating if the data flavour is accepted by this
	 *         drop target
	 */
	protected boolean dataFlavourSupported( Component component, DataFlavor[] flavours )
	{
		return true;
	}
	
	
	/*
	 * Overridden methods of DropTarget
	 */
	
	/** {@inheritDoc} */
	public void dragEnter( DropTargetDragEvent e )
	{
		/* first decide whether we can allow this new drag event */
		this.canImport = dataFlavourSupported( e.getDropTargetContext().getComponent(), e.getCurrentDataFlavors() );
		
		
		int dropAction = e.getDropAction();
		
		if (this.canImport && actionSupported( dropAction ))
			e.acceptDrag( dropAction );
		else
			e.rejectDrag();
	}

	/** {@inheritDoc} */
	public void dragOver( DropTargetDragEvent e )
	{
		int dropAction = e.getDropAction();
		
		if (this.canImport)
			e.acceptDrag( dropAction );
		else
			e.rejectDrag();
	}

	/** {@inheritDoc} */
	public void dragExit( DropTargetEvent e )
	{
	}

	/** {@inheritDoc} */
	public void drop( DropTargetDropEvent e )
	{
		int dropAction = e.getDropAction();
		
		if (this.canImport && actionSupported( dropAction ))
			e.acceptDrop( dropAction );
		else
			e.rejectDrop();
	}

	/** {@inheritDoc} */
	public void dropActionChanged( DropTargetDragEvent e )
	{
		int dropAction = e.getDropAction();
		
		if (this.canImport && actionSupported( dropAction ))
			e.acceptDrag( dropAction );
		else
			e.rejectDrag();
	}
	
}


/*
 * Change History
 *
 * $Log: DefaultDropTarget.java,v $
 * Revision 1.1.2.1  2008/02/18 22:01:32  Steve
 * Initial revision (taken from Falken project, originally from Bob project).
 *
 * 
 * 
 * Revision 1.1  2007/12/31 18:24:09  Steve
 * *** empty log message ***
 *
 * Original Revision 2005/11/29
 *
 */
