/*
 * RepaintImageObserver.java
 * Part of the B'am library - a collection of general-purpose utility classes.
 * 
 * Copyright (C)2006 Stephen Battey
 * Contact: resource@lucidviews.net
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

package net.lucidviews.util.image;

import java.awt.image.ImageObserver;
import java.awt.Image;
import java.awt.Component;


/**
 * An <code>ImageObserver</code> that repaints a component once the image has
 * fully loaded.
 * <br />
 * <br />
 * @see java.awt.image.ImageObserver
 * @see java.awt.Image
 * @author Stephen Battey
 * @version $Revision: 1.4 $
 * @since 1.0
 */
public class RepaintImageObserver extends BaseImageObserver
{
	
	/*
	 * Instance variables
	 */
	
	/** The component to repaint once the image has loaded. */
	protected Component _targetComponent;
	

	
	/*
	 * Constructors
	 */
	
	/**
	 * Create an observer that repaints the target component when the observed
	 * image has loaded.
	 * <br />
	 * <br />
	 * @param targetComponent the component to be refreshed
	 */
	public RepaintImageObserver( Component targetComponent )
	{
		super();
		_targetComponent = targetComponent;
	}
	
	/**
	 * Create an observer that repaints the target component when the specified
	 * image has loaded.
	 * <br />
	 * The extra <code>image</code> parameter restricts this observer such that
	 * it only repaints the target component when the specified image has loaded
	 * and not any other image it has been assigned to observe.
	 * <br />
	 * <br />
	 * @param targetComponent the component to be refreshed
	 * @param image the image that is to be monitored
	 */
	public RepaintImageObserver( Component targetComponent, Image image )
	{
		super( image );
		_targetComponent = targetComponent;
	}
	
	
	/*
	 * Implementation of abstract method of BaseImageObserver class
	 */
	
	public boolean observedImageUpdate( Image img, int infoflags, int x, int y, int width, int height )
	{
		// check if all pixel data has loaded
		boolean pixelsLoaded = (infoflags & ImageObserver.ALLBITS) == ImageObserver.ALLBITS;

		if (pixelsLoaded)
		{
			_targetComponent.repaint();
			
			// all done
			return false;
		}
		else
		{
			// more pixels still to load
			return true;
		}
	}
	
}


/*
 * Change History
 *
 * $Log: RepaintImageObserver.java,v $
 * Revision 1.4  2006/09/06 20:52:02  Steve
 * Added legal disclaimer.
 *
 * Revision 1.3  2006/09/02 16:39:07  Steve
 * Extracted useful elements (check specific image & keep alive functionality) out to a base class (BaseImageObserver).
 *
 * Revision 1.2  2006/09/02 15:09:05  Steve
 * Implemented workaround for ImageObserver bug in JDK 1.5
 *
 * Revision 1.1  2006/06/12 20:52:36  Steve
 * Initial revision.
 *
 */
