/*
 * FilePaths.java
 * Part of the Bam project.
 * 
 * Copyright (C)2008 Stephen Battey
 * Contact: resource@lucidviews.net
 */
package net.lucidviews.util.io;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;


/**
 * A collection of methods for manipualting paths to files and folders.
 * <br />
 * <br />
 * @author Stephen Battey
 * @version $Revision: 1.1.2.1 $
 * @since 1.3
 */
public class FilePaths
{
	
	/*
	 * Constructor
	 */
	
	/**
	 * The default constructor.
	 */
	// protected the constructor to prevent anyone instantiating this class
	protected FilePaths()
	{
	}
	
	
	/*
	 * Exported constants
	 */
	
	/** A lookup table of special characters that are reserved for file system paths,
	 *  indexed by the name of the operating system to which the reserved characters apply. */
	protected static final Map<String,String> SPECIAL_CHARACTERS_BY_OS = new HashMap<String,String>();
	
	/** The special characters that are reserved for file system paths in the operating system
	 *  that is running this program. */
	// Use String rather than List<Character> to ensure characters cannot be added/removed. */
	public static final String SPECIAL_CHARACTERS;
	
	/** A default mapping of special characters to alternative, non-special characters. */
	public static final Map<Character,Character> SPECIAL_CHARACTER_REPLACEMENTS = new HashMap<Character,Character>();
	
	
	/*
	 * Static initialisation
	 */
	
	static
	{
		SPECIAL_CHARACTERS_BY_OS.put( "Windows", "/\\:*|?\"<>" );
		SPECIAL_CHARACTERS_BY_OS.put( "Linux", "&;|*?'\"'[]()$<>{}^#/\\%!~+" );
		SPECIAL_CHARACTERS_BY_OS.put( "SunOS", "&;|*?'\"'[]()$<>{}^#/\\%!~+" );
		SPECIAL_CHARACTERS_BY_OS.put( "Solaris", "&;|*?'\"'[]()$<>{}^#/\\%!~+" );
		SPECIAL_CHARACTERS_BY_OS.put( "HP-UX", "&;|*?'\"'[]()$<>{}^#/\\%!~+" );
		SPECIAL_CHARACTERS_BY_OS.put( "Mac", ":" );
		// OSs not supported (not sure about special chars):
		//   OpenVMS, OSF1, NetWare, Irix, FreeBSD, AIX
		
		SPECIAL_CHARACTERS = getSpecialChars();
		
		SPECIAL_CHARACTER_REPLACEMENTS.put( ' ', '_' );
		SPECIAL_CHARACTER_REPLACEMENTS.put( '"', '\'' );
	}
	
	
	/*
	 * Utility methods
	 */
	
	/**
	 * Check if two files point to the same location.
	 * <br />
	 * <br />
	 * @param file1 the first file
	 * @param file2 the other file
	 * @return <code>true</code> if the two <code>File</code>s reference the same location
	 * @throws IOException if the file paths cannot be resolved into canonical form
	 */
	public static final boolean isSamePath( File file1, File file2 ) throws IOException
	{
		return file1.getCanonicalPath().equals( file2.getCanonicalPath() );
	}

	/**
	 * Create a valid file path, either replacing any illegal (reserved) characters
	 * in the original path with the default replacement characters or removing the
	 * illegal characters entirely.
	 * <br />
	 * <br />
	 * @param file the original file path
	 * @return a valid file path
	 * @see #SPECIAL_CHARACTER_REPLACEMENTS
	 */
	public static File validateFilePath( File file )
	{
		StringBuffer validFilePath = new StringBuffer();
		File workingFile = file;
		while (workingFile != null)
		{
			if (validFilePath.length() > 0)
			{
				validFilePath.insert( 0, File.separatorChar );
			}
			validFilePath.insert( 0, validateFileName( workingFile.getName() ) );
			
			// keep looping
			workingFile = workingFile.getParentFile();
		}
		
		return new File( validFilePath.toString() );
	}
	
	/**
	 * Create a valid name for a file, either replacing any illegal (reserved) characters
	 * in the original name with the default replacement characters or removing the
	 * illegal characters entirely.
	 * <br />
	 * <br />
	 * @param fileName the original file name
	 * @return a valid file name
	 * @see #SPECIAL_CHARACTER_REPLACEMENTS
	 */
	public static String validateFileName( String fileName )
	{
		return validateFileName( fileName, SPECIAL_CHARACTER_REPLACEMENTS );
	}
	
	/**
	 * Create a valid name for a file, either replacing any illegal (reserved) characters
	 * in the original name with the suggested replacement characters or removing the
	 * illegal characters entirely.
	 * <br />
	 * <br />
	 * @param fileName the original file name
	 * @param replacementCharacters characters used to replace illegal characters
	 * @return a valid file name
	 */
	public static String validateFileName( String fileName, Map<Character,Character> replacementCharacters )
	{
		StringBuffer buffer = new StringBuffer( fileName );
		
		int maxCharIndex = SPECIAL_CHARACTERS.length();
		for (int i=0; i<maxCharIndex; i++)
		{
			char specialChar = SPECIAL_CHARACTERS.charAt( i );
			String specialCharStr = "" + specialChar;
			
			Character replacementChar = replacementCharacters.get( specialChar );
			String replacementCharStr = (replacementChar == null) ? null : replacementChar.toString();
			
			int specialCharIndex = buffer.indexOf( specialCharStr );
			while (specialCharIndex > -1)
			{
				if (replacementCharStr == null)
				{
					buffer.deleteCharAt( specialCharIndex );
				}
				else
				{
					buffer.replace( specialCharIndex, specialCharIndex + 1, replacementCharStr );
				}
				
				specialCharIndex = buffer.indexOf( specialCharStr );
			}
		}
		
		return buffer.toString();
	}
	
	/**
	 * Get the special (reserved) characters for file system paths
	 * for the operating system that is running this program.
	 * <br />
	 * <br />
	 * @return a string containing the special (reserved) characters
	 */
	public static String getSpecialChars()
	{
		return getSpecialChars( System.getProperty( "os.name" ) );
	}
	
	/**
	 * Get the special (reserved) characters for file system paths for a given operating system.
	 * <br />
	 * <br />
	 * @param osName the name of the operating system
	 *               (version is optional, e.g. "Windows" or "Windows XP" will both find the settings for Windows)
	 * @return a string containing the special (reserved) characters
	 * @throws UnsupportedOperationException if the operating system is unknown (not supported)
	 */
	public static String getSpecialChars( String osName ) throws UnsupportedOperationException
	{
		if (osName == null)
			throw new NullPointerException( "Cannot lookup special characters for a null OS name." );
		
		// Names of supported OSs are prefixes, to allow multiple versions of the same OS to be supported.
		// So don't perform a lookup or direct string comparison. Instead check the prefix of the osName
		// against the list of names of supported OSs.
		Iterator<String> supportedOsNameItr = SPECIAL_CHARACTERS_BY_OS.keySet().iterator();
		while (supportedOsNameItr.hasNext())
		{
			String supportedOsName = supportedOsNameItr.next();
			if (osName.startsWith( supportedOsName ))
			{
				return SPECIAL_CHARACTERS_BY_OS.get( supportedOsName );
			}
		}
		
		throw new UnsupportedOperationException( "Unknown, unsupported operating system: " + osName );
	}
	
}


/*
 * Change History
 *
 * $Log: FilePaths.java,v $
 * Revision 1.1.2.1  2008/02/24 20:11:17  Steve
 * Initial revision.
 *
 */