/*
 * Files.java
 * Part of the B'am library - a collection of general-purpose utility classes.
 * 
 * Copyright (C)2006 Stephen Battey
 * Contact: resource@lucidviews.net
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

package net.lucidviews.util.io;

import java.io.FileNotFoundException;
import java.io.File;
import java.io.FileFilter;
import java.io.FilenameFilter;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.text.NumberFormat;
import java.util.StringTokenizer;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;
import java.util.Collections;
import java.util.Comparator;


/**
 * A collection of methods for manipualting <code>File</code>s.
 * <br />
 * <br />
 * @author Stephen Battey
 * @version $Revision: 1.6.2.2 $
 * @since 1.0
 */
public class Files extends Streams
{

  /*
   * Local constants
   */

  /** The character used to separate folder paths in the classpath. */
  private static final String DEFAULT_PATH_SEPARATOR = ";";


  /*
   * Constructor(s)
   */

  /**
   * Suppresses default constructor, ensuring non-instantiability.
   */
  protected Files()
  {
  }


  /*
   * Static methods
   */
	
	/**
	 * Get the existing parent (folder) for the given file.
	 * <br />
	 * If no parent or grandparent exist for the given file path,
	 * then a <code>null</code> value is returned.
	 * <br />
	 * <br />
	 * @param file the file
	 * @return the first parent or grandparent (folder) that is found to exist
	 * @since 1.3
	 */
	public static File getExistingParentFile( File file )
	{
		File folder = file;
		
		while (   (folder != null)
		       && (!folder.isDirectory())  )
		{
			// keep going up the tree until either:
			//   * the folder is null (no more parents)
			//   * the folder exists and is a directory
			folder = folder.getParentFile();
		}
		
		return folder;
	}
	
	/**
	 * Recurse down to find the root of a file path.
	 * <br />
	 * <br />
	 * @param file a <code>File</code>
	 * @return a <code>File</code> reference to the root folder
	 * @since 1.3
	 */
	public static File deriveRootFolder( File file )
	{
		if (file == null)
			throw new IllegalArgumentException( "Cannot get the root of a null file." );
		
		File root = file;
		File parent = root.getParentFile();
		while (parent != null)
		{
			root = parent;
			parent = parent.getParentFile();
		}
		
		return root;
	}

  /**
   * Find a file with the given name, located somewhere on the classpath.
   * <br />
   * Each folder defined on the Java classpath will be searched. If the file
   * does not exist in any of those folders an exception will be thrown.
   * <br />
   * <br />
   * @param fileName the name of the file
   * @throws FileNotFoundException if the file could not be found on the
   *                               classpath
   * @return a <code>File</code> with the given name, located somewhere on the
   *         classpath, the object is guaranteed to be non-<code>null</code> and
   *         the file is guaranteed to exist
   */
  public static File findFile( String fileName ) throws FileNotFoundException
  {
    // first try using the fileName as an absolute path
    File absoluteFile = new File( fileName );
    if (absoluteFile.exists())
      return absoluteFile;

    // then try looking in the classpath
    String classpath = System.getProperty( "java.class.path", "" );
    String pathSeparator = System.getProperty( "path.separator", DEFAULT_PATH_SEPARATOR );
    StringTokenizer classpathTokenizer = new StringTokenizer( classpath, pathSeparator, true );

    File file = null;
    boolean foundFile = false;
    while (   (!foundFile)
           && (classpathTokenizer.hasMoreTokens())  )
    {
      String folderPath = classpathTokenizer.nextToken();

      // look for the named file in this folder
      file = new File( folderPath, fileName );
      foundFile = file.exists();
    }

    if (!foundFile)
      throw new FileNotFoundException( "File with name '" + fileName + "' could not be found on the classpath (" + classpath + ")." );

    return file;
  }

  /**
   * Checks if one file is a duplicate of another.
   * <br />
   * <br />
   * @param file file to test
   * @param compare second file to compare the file to
   * @return <code>true</code> if the two files are the same
   * @throws IOException error reading content of the two files being compared
   */
  public static boolean isDuplicate( File file, File compare ) throws IOException
  {
    // useful shortcut: compare size
    if (file.length() != compare.length())
      return false;

    // if both files are of the same size we need to compare content
    return compare( new FileInputStream( file ), new FileInputStream( compare ) );
  }

  /**
   * Checks if the <code>suspectedChild</code> file (or folder) is contained
   * within the specified folder or one of its sub-folders.
   * <br />
   * NB: this methods explicitly checks for descendants - if the two files are
   * the same then this method returns <code>false</code> (not a descendant).
   * <br />
   * <br />
   * @param folder the folder that could be the parent or grand-parent
   * @param suspectedChild the file or folder that is suspected to be a
   *                       descendant
   * @return boolean <code>true</code> if the <code>suspectedChild</code> is a
   *                 descendant of the <code>folder</code>
   */
  public static boolean isDescendant( File folder, File suspectedChild )
  {
    // ascend folder heirachy from the suspected child to its root folder
    File parent = suspectedChild.getParentFile();
    while (parent != null)
    {
      if (parent.equals( folder ))
    	// the folder is a parent/grand-parent of the suspected child
        return true;

      parent = parent.getParentFile();
    }

    return false;
  }

  /**
   * Create a new, blank file in the given folder.
   * <br />
   * The file created is guaranteed not to overwrite any existing file and be
   * located in the specified <code>folder</code>. The file will have a name
   * <b>based on</b> the <code>preferredFileName</code>. If a file with the
   * preferred name already exists then a numeric suffix will be added (e.g.
   * <code>./filename_2.dat</code>).
   * <br />
   * <br />
   * @param folder the directory where the new file is to be created
   * @param preferredFileName the desired name of the file
   * @return a <code>File</code> reference to a new file in the given folder
   * @throws IOException error creating the new file
   * @throws SecurityException access to write the file was denied by the system
   *                           security manager
   */
  public static File createNewFile( File folder, String preferredFileName ) throws IOException
  {
    NewFileNameModel newFileNameModel = null;
	    
	// try creating the file in the folder
    File newFile = new File( folder, preferredFileName );
    while (!newFile.createNewFile())
    {
      if (newFileNameModel == null)
        newFileNameModel = new NewFileNameModel( preferredFileName );

      // create the file using the NewFileNameModel to generate a unique name
      newFile = new File( folder, newFileNameModel.getName() );
    }

    return newFile;
  }

  /**
   * Create a new, blank file with a given name.
   * <br />
   * The file created is guaranteed not to overwrite any existing file and have
   * the specified <code>fileName</code>. The file will be placed within
   * <code>folder</code>, either directly in that folder or in a sub-folder.
   * If a file with the preferred name already exists then a sub-folder will
   * be created with a numeric suffix (e.g.
   * <code>./filename_2/filename_2.dat</code>).
   * <br />
   * <br />
   * @param fileName the name of the new file
   * @param folder the desired directory where the new file is to be created
   * @return a <code>File</code> reference to a new file in the given folder
   * @throws IOException error creating the new file
   * @throws SecurityException access to write the file was denied by the system
   *                           security manager
   */
  public static File createNewFile( String fileName, File folder ) throws IOException
  {
    String thePartOfTheNameBeforeTheExtn = null;
    int fileIndex = 1;

    // try creating the file in the folder
    File newFile = new File( folder, fileName );
    if (!newFile.createNewFile())
    {
      // decompose the file name
      int extnIndex = fileName.lastIndexOf( '.' );
      thePartOfTheNameBeforeTheExtn = fileName.substring( 0, extnIndex );

      // try creating a new sub-folder in which to store this file
      File subFolder;
      do
      {
        fileIndex ++;
        subFolder = new File( folder, thePartOfTheNameBeforeTheExtn + "_" + fileIndex );
      }
      while (!subFolder.mkdirs());

      newFile = new File( subFolder, fileName );
      newFile.createNewFile();
    }

    return newFile;
  }

  /**
   * Convert an array of <code>File</code>s to a sorted <code>List</code>.
   * <br />
   * The files are ordered using a <code>Comparator</code>.
   * <br />
   * <br />
   * @param files an array of <code>File</code>
   * @param comparator the <code>Comparator</code> used to sort the files
   * @return a <code>List</code> of sorted <code>File</code>s
   * @throws ClassCastException the specified <code>comparator</code> was not
   *                            compatible with <code>File</code> objects
   * @throws UnsupportedOperationException the generated <code>List</code> did
   *                                       not support a sort operation
   * @see net.lucidviews.util.io.FileComparator
   */
  public static List sortFiles( File[] files, Comparator<File> comparator )
  {
	// add each file to a list
	// NB: the java.util.Arrays class could be used to convert the array to a
	//     list, but the list it returns is very limited in functionality
	//     (UnSupportedOperationExceptions are thrown from most methods)
    List<File> listOfFiles = new Vector<File>( files.length );
    for (int i=0; i<files.length; i++)
      listOfFiles.add( files[i] );

    // perform the sort using the list
    Collections.sort( listOfFiles, comparator );

    return listOfFiles;
  }

  /**
   * Get a sorted list of files for a given directory.
   * <br />
   * The list returned will contain all files in the specified folder and will
   * be sorted in the order defined by the comparator.
   * <br />
   * <br />
   * @param folder the folder whose contents are to be listed
   * @param comparator a <code>Comparator</code> that will sort the files
   * @return a sorted <code>List</code> of <code>File</code>s
   * @throws SecurityException read access to the folder was denied
   * @throws ClassCastException the specified <code>comparator</code> was not
   *                            compatible with <code>File</code> objects
   * @throws UnsupportedOperationException the generated <code>List</code> did
   *                                       not support a sort operation
   * @see net.lucidviews.util.io.FileComparator
   */
  public static List listFiles( File folder, Comparator<File> comparator )
  {
    return sortFiles( folder.listFiles(), comparator );
  }

  /**
   * Get a filtered, sorted list of files for a given directory.
   * <br />
   * The list returned will contain files in the specified folder which
   * are accepted by the filter, and are sorted in the order defined by the
   * comparator.
   * <br />
   * <br />
   * @param folder the folder whose contents are to be listed
   * @param filter a filter that determines which files are added to the list
   * @param comparator a <code>Comparator</code> that will sort the files
   * @return a sorted <code>List</code> of <code>File</code>s
   * @throws SecurityException read access to the folder was denied
   * @throws ClassCastException the specified <code>comparator</code> was not
   *                            compatible with <code>File</code> objects
   * @throws UnsupportedOperationException the generated <code>List</code> did
   *                                       not support a sort operation
   * @see java.io.FilenameFilter
   * @see net.lucidviews.util.io.FileComparator
   */
  public static List listFiles( File folder, FilenameFilter filter, Comparator<File> comparator )
  {
    return sortFiles( folder.listFiles( filter ), comparator );
  }

  /**
   * Get a filtered, sorted list of files for a given directory.
   * <br />
   * The list returned will contain files in the specified folder which
   * are accepted by the filter, and are sorted in the order defined by the
   * comparator.
   * <br />
   * <br />
   * @param folder the folder whose contents are to be listed
   * @param filter a filter that determines which files are added to the list
   * @param comparator a <code>Comparator</code> that will sort the files
   * @return a sorted <code>List</code> of <code>File</code>s
   * @throws SecurityException read access to the folder was denied
   * @throws ClassCastException the specified <code>comparator</code> was not
   *                            compatible with <code>File</code> objects
   * @throws UnsupportedOperationException the generated <code>List</code> did
   *                                       not support a sort operation
   * @see java.io.FileFilter
   * @see net.lucidviews.util.io.FileComparator
   */
  public static List listFiles( File folder, FileFilter filter, Comparator<File> comparator )
  {
    return sortFiles( folder.listFiles( filter ), comparator );
  }

  /**
   * Copy a file to a given folder or file location.
   * <br />
   * The destination <code>File</code> can reference either a directory or a
   * file. In the case of a directory, a copy of the original file will be
   * placed in this directory with the same name. If a file is specified, the
   * destination file will be overwritten with the file at the
   * <code>original</code> location.
   * <br />
   * <br />
   * @param original the file to be copied
   * @param destination the destination of the copy - either a file or a directory
   * @param bufferSize the size of the buffer to use when copying the data
   * @return the new copy of the original file
   * @throws SecurityException error creating a new file
   * @throws IOException error copying the content of the file
   */
  public static File copyFile( File original, File destination, int bufferSize ) throws IOException
  {
    File fileCopy;
    if (destination.isDirectory())
      // copy the file into this directory maintaining the name of the original
      fileCopy = new File( destination, original.getName() );
    else
      fileCopy = destination;


    if (!fileCopy.createNewFile())
    {
      // the file already exists; check it can be overwritten
      if (!fileCopy.canWrite())
        throw new IOException( "Unable to copy file '" + original.getAbsolutePath() + "'. A read-only file already exists at destination path '" + fileCopy.getAbsolutePath() + "'." );
    }

    BufferedInputStream in = null;
    BufferedOutputStream out = null;
    boolean copiedOk = false;
    try
    {
      in = new BufferedInputStream( new FileInputStream( original ), bufferSize );
      out = new BufferedOutputStream( new FileOutputStream( fileCopy ), bufferSize );

	  pipe( in, out, bufferSize );

      copiedOk = true;
    }
    // ensure the streams are closed even if an error occurs
    finally
    {
	  close( in );
	  close( out );

      if (!copiedOk)
        // copy failed so delete the destination file
        fileCopy.delete();
    }

    return fileCopy;
  }

  /**
   * Copy a file to a given folder or file location.
   * <br />
   * The <code>destination</code> <code>File</code> can reference either a
   * directory or a file. In the case of a directory, a copy of the original
   * file will be placed in this directory with the same name. If a file is
   * specified, the destination file will be overwritten with the file at the
   * <code>original</code> location.
   * <br />
   * <br />
   * @param original the file to be copied
   * @param destination the destination of the copy - either a file or a
   *                    directory
   * @return the new copy of the original file
   * @throws SecurityException error creating a new file
   * @throws IOException error copying the content of the file
   */
  public static File copyFile( File original, File destination ) throws IOException
  {
    return copyFile( original, destination, DEFAULT_BUFFER_SIZE );
  }

  /**
   * Copy a file to a folder, giving the copy a new name.
   * <br />
   * If a file already exists at the new location, it will be overwritten.
   * <br />
   * <br />
   * @param original the file to be copied
   * @param destinationFolder the folder where the file is to be copied to
   * @param fileName the name of the new copy
   * @param bufferSize the size of the buffer to use when copying the data
   * @return the new copy of the original file
   * @throws SecurityException error creating a new file
   * @throws IOException error copying the content of the file
   */
  public static File copyFile( File original, File destinationFolder, String fileName, int bufferSize ) throws IOException
  {
    File destFile = new File( destinationFolder, fileName );

    return copyFile( original, destFile, bufferSize );
  }

  /**
   * Copy a file to a folder, giving the copy a new name.
   * <br />
   * If a file already exists at the new location, it will be overwritten.
   * <br />
   * <br />
   * @param original the file to be copied
   * @param destinationFolder the folder where the file is to be copied to
   * @param fileName the name of the new copy
   * @return the new copy of the original file
   * @throws SecurityException error creating a new file
   * @throws IOException error copying the content of the file
   */
  public static File copyFile( File original, File destinationFolder, String fileName ) throws IOException
  {
    return copyFile( original, destinationFolder, fileName, DEFAULT_BUFFER_SIZE );
  }

  /**
   * Copy a folder, and all its contents, to another location.
   * <br />
   * If the <code>destination</code> folder already contains a folder with the
   * same name as the <code>original</code> then the contents of the
   * <code>original</code> folder will be merged into that folder, overwriting
   * any writable files already in the <code>destination</code> (read-only
   * files will not be overwritten, but will cause this copy to be incomplete,
   * acknowledged by a <code>false</code> return value).
   * <br />
   * This copy method will recurse through all sub-folders of the
   * <code>original</code> folder. If one file or folder cannot be copied the
   * routine will continue copying the remaining contents.
   * <br />
   * This method returns <code>true</code> if all files and sub-folders were
   * successfully copied. This method returns <code>false</code> if any file or
   * folder could not be deleted.
   * <br />
   * I.e.
   * <br />
   * &bullet;&nbsp;A return value of <code>true</code> indicates the
   * <code>destination</code> folder will be an exact copy of the
   * <code>original</code>.
   * &bullet;&nbsp;A return value of <code>false</code> indicates the
   * <code>destination</code> folder may not exist.
   * &bullet;&nbsp;In the case of a security exception, there is no guarantee as
   * to which files have been copied, including the root folder.
   * <br />
   * <br />
   * @param original the folder to be copied
   * @param destination the folder where the original folder is to be copied to
   * @param filter a filter used to choose which files to copy
   * @return <code>true</code> if the folder and its contents were copied
   *         successfully, <code>false</code> otherwise
   */
  public static boolean copyFolder( File original, File destination, FilenameFilter filter )
  {
	// create a sub folder in the destination folder
    File folderCopy = new File( destination, original.getName() );
    if (!folderCopy.exists())
      folderCopy.mkdirs();

    // check the new folder was created
    if (!folderCopy.exists())
      return false;
    else
      // now copy the contents of the original into the sub folder
      return copyFolderContents( original, folderCopy, filter );
  }

  /**
   * Copy a folder, and all its contents, to another location.
   * <br />
   * If the <code>destination</code> folder already contains a folder with the
   * same name as the <code>original</code> then the contents of the
   * <code>original</code> folder will be merged into that folder, overwriting
   * any writable files already in the <code>destination</code> (read-only
   * files will not be overwritten, but will cause this copy to be incomplete,
   * acknowledged by a <code>false</code> return value).
   * <br />
   * This copy method will recurse through all sub-folders of the
   * <code>original</code> folder. If one file or folder cannot be copied the
   * routine will continue copying the remaining contents.
   * <br />
   * This method returns <code>true</code> if all files and sub-folders were
   * successfully copied. This method returns <code>false</code> if any file or
   * folder could not be deleted.
   * <br />
   * I.e.
   * <br />
   * &bullet;&nbsp;A return value of <code>true</code> indicates the
   * <code>destination</code> folder will be an exact copy of the
   * <code>original</code>.
   * &bullet;&nbsp;A return value of <code>false</code> indicates the
   * <code>destination</code> folder may not exist.
   * &bullet;&nbsp;In the case of a security exception, there is no guarantee as
   * to which files have been copied, including the root folder.
   * <br />
   * <br />
   * @param original the folder to be copied
   * @param destination the folder where the original folder is to be copied to
   * @param filter a filter used to choose which files to copy
   * @return <code>true</code> if the folder and its contents were copied
   *         successfully, <code>false</code> otherwise
   */
  public static boolean copyFolder( File original, File destination, FileFilter filter )
  {
    // create a sub folder in the destination folder
    File folderCopy = new File( destination, original.getName() );
    if (!folderCopy.exists())
      folderCopy.mkdirs();

    // check the new folder was created
    if (!folderCopy.exists())
      return false;
    else
      // now copy the contents of the original into the sub folder
      return copyFolderContents( original, folderCopy, filter );
  }

  /**
   * Copy a folder, and all its contents, to another location.
   * <br />
   * If the <code>destination</code> folder already contains a folder with the
   * same name as the <code>original</code> then the contents of the
   * <code>original</code> folder will be merged into that folder, overwriting
   * any writable files already in the <code>destination</code> (read-only
   * files will not be overwritten, but will cause this copy to be incomplete,
   * acknowledged by a <code>false</code> return value).
   * <br />
   * This copy method will recurse through all sub-folders of the
   * <code>original</code> folder. If one file or folder cannot be copied the
   * routine will continue copying the remaining contents.
   * <br />
   * This method returns <code>true</code> if all files and sub-folders were
   * successfully copied. This method returns <code>false</code> if any file or
   * folder could not be deleted.
   * <br />
   * I.e.
   * <br />
   * &bullet;&nbsp;A return value of <code>true</code> indicates the
   * <code>destination</code> folder will be an exact copy of the
   * <code>original</code>.
   * &bullet;&nbsp;A return value of <code>false</code> indicates the
   * <code>destination</code> folder may not exist.
   * &bullet;&nbsp;In the case of a security exception, there is no guarantee as
   * to which files have been copied, including the root folder.
   * <br />
   * <br />
   * @param original the folder to be copied
   * @param destination the folder where the original folder is to be copied to
   * @return <code>true</code> if the folder and its contents were copied
   *         successfully, <code>false</code> otherwise
   */
  public static boolean copyFolder( File original, File destination )
  {
    return copyFolder( original, destination, (FileFilter)null );
  }

  /**
   * Copy the contents of a folder to another location.
   * <br />
   * The contents of the <code>original</code> folder will overwrite any
   * writable files already in the <code>destination</code> (read-only files
   * will not be overwritten, but will cause this copy to be incomplete,
   * acknowledged by a <code>false</code> return value).
   * <br />
   * This copy method will recurse through all sub-folders of the
   * <code>original</code> folder. If one file or folder cannot be copied the
   * routine will continue copying the remaining contents.
   * <br />
   * This method returns <code>true</code> if all files and sub-folders were
   * successfully copied. This method returns <code>false</code> if any file or
   * folder could not be deleted.
   * <br />
   * I.e.
   * <br />
   * &bullet;&nbsp;A return value of <code>true</code> indicates the
   * <code>destination</code> folder will contain all the files and folder in
   * the <code>original</code>.
   * &bullet;&nbsp;A return value of <code>false</code> indicates the
   * <code>destination</code> folder may not contain any of the files or folders
   * from the <code>original</code>.
   * &bullet;&nbsp;In the case of a security exception, there is no guarantee as
   * to which files have been copied.
   * <br />
   * <br />
   * @param original the folder whose content is to be copied
   * @param destination the folder where the content is to be copied to
   * @param filter a filter used to choose which files to copy
   * @return <code>true</code> if the contents were copied successfully,
   *         <code>false</code> otherwise
   */
  public static boolean copyFolderContents( File original, File destination, FilenameFilter filter )
  {
    boolean success = true;

    // loop through all files and sub-folders of the original folder
    File ref;
    File[] files = original.listFiles( filter );
    for (int i=0; i<files.length; i++)
    {
      ref = files[i];

      try
      {
        if (ref.isFile())
          copyFile( ref, destination, ref.getName() );
        else
          success &= copyFolder( ref, destination, filter );
      }
      catch (Exception e)
      {
        // record the failure but continue
        success = false;
      }
    }

    return success;
  }

  /**
   * Copy the contents of a folder to another location.
   * <br />
   * The contents of the <code>original</code> folder will overwrite any
   * writable files already in the <code>destination</code> (read-only files
   * will not be overwritten, but will cause this copy to be incomplete,
   * acknowledged by a <code>false</code> return value).
   * <br />
   * This copy method will recurse through all sub-folders of the
   * <code>original</code> folder. If one file or folder cannot be copied the
   * routine will continue copying the remaining contents.
   * <br />
   * This method returns <code>true</code> if all files and sub-folders were
   * successfully copied. This method returns <code>false</code> if any file or
   * folder could not be deleted.
   * <br />
   * I.e.
   * <br />
   * &bullet;&nbsp;A return value of <code>true</code> indicates the
   * <code>destination</code> folder will contain all the files and folder in
   * the <code>original</code>.
   * &bullet;&nbsp;A return value of <code>false</code> indicates the
   * <code>destination</code> folder may not contain any of the files or folders
   * from the <code>original</code>.
   * &bullet;&nbsp;In the case of a security exception, there is no guarantee as
   * to which files have been copied.
   * <br />
   * <br />
   * @param original the folder whose content is to be copied
   * @param destination the folder where the content is to be copied to
   * @param filter a filter used to choose which files to copy
   * @return <code>true</code> if the contents were copied successfully,
   *         <code>false</code> otherwise
   */
  public static boolean copyFolderContents( File original, File destination, FileFilter filter )
  {
    boolean success = true;

    // loop through all files and sub-folders of the original folder
    File ref;
    File[] files = original.listFiles( filter );
    for (int i=0; i<files.length; i++)
    {
      ref = files[i];

      try
      {
        if (ref.isFile())
          copyFile( ref, destination, ref.getName() );
        else
          success &= copyFolder( ref, destination, filter );
      }
      catch (Exception e)
      {
        // record the failure but continue
        success = false;
      }
    }

    return success;
  }

  /**
   * Copy the contents of a folder to another location.
   * <br />
   * The contents of the <code>original</code> folder will overwrite any
   * writable files already in the <code>destination</code> (read-only files
   * will not be overwritten, but will cause this copy to be incomplete,
   * acknowledged by a <code>false</code> return value).
   * <br />
   * This copy method will recurse through all sub-folders of the
   * <code>original</code> folder. If one file or folder cannot be copied the
   * routine will continue copying the remaining contents.
   * <br />
   * This method returns <code>true</code> if all files and sub-folders were
   * successfully copied. This method returns <code>false</code> if any file or
   * folder could not be deleted.
   * <br />
   * I.e.
   * <br />
   * &bullet;&nbsp;A return value of <code>true</code> indicates the
   * <code>destination</code> folder will contain all the files and folder in
   * the <code>original</code>.
   * &bullet;&nbsp;A return value of <code>false</code> indicates the
   * <code>destination</code> folder may not contain any of the files or folders
   * from the <code>original</code>.
   * &bullet;&nbsp;In the case of a security exception, there is no guarantee as
   * to which files have been copied.
   * <br />
   * <br />
   * @param original the folder whose content is to be copied
   * @param destination the folder where the content is to be copied to
   * @return <code>true</code> if the contents were copied successfully,
   *         <code>false</code> otherwise
   */
  public static boolean copyFolderContents( File original, File destination )
  {
    return copyFolderContents( original, destination, (FileFilter)null );
  }

  /**
   * Delete a folder and all files and sub-folders.
   * <br />
   * This method returns <code>true</code> if all files and sub-folders were
   * successfully deleted. This method returns <code>false</code> if any file or
   * folder could not be deleted.
   * <br />
   * I.e.
   * <br />
   * &bullet;&nbsp;A return value of <code>true</code> indicates the
   * <code>folder</code> will no longer exist.
   * &bullet;&nbsp;A return value of <code>false</code> indicates the
   * <code>folder</code> will still exist, however there is no guarantee that
   * any of its files or sub-folders will still exist.
   * &bullet;&nbsp;In the case of a security exception, there is no guarantee as
   * to which files have been deleted, including the specified
   * <code>folder</code>.
   * <br />
   * <br />
   * @param folder the folder to be deleted
   * @return boolean value indicating whether the delete was totally successful
   * @throws SecurityException delete access to the folder was denied by the
   *                           security manager
   */
  public static boolean deleteFolder( File folder )
  {
    if (folder.isFile())
      return false;

    boolean success = true;
    File ref;

    // loop through all files and sub-folders
    File[] files = folder.listFiles();
    for (int i=0; i<files.length; i++)
    {
      ref = files[i];

      if (ref.isFile())
        success &= ref.delete();
      else
        success &= deleteFolder( ref );
    }

    // once all the content is deleted, delete the folder itself
    success &= folder.delete();

    return success;
  }
  
  /**
   * Create a patch for an old set of files.
   * <br />
   * This method will compare the content of two folders and deduce which files
   * in the 'current' set of files need to be copied into the 'old' set of files
   * in order to bring the old set of files up-to-date.
   * <br />
   * No changes will be made to the old set of files, i.e. this method does not
   * <i>perform</i> a patch, it <i>creates</i> a patch by copying the new or
   * modified files into a blank 'destination' folder. The patch is created in
   * such a way that copying the 'destination' folder over the 'old' folder will
   * patch the old set of files and bring them up-to-date with the 'current'
   * set of files.
   * <br />
   * <br />
   * This method will automatically recurse into sub folders and include
   * modified files in the patch. (The definition of 'modified' being those files
   * whose timestamp is more recent than the old copy of the file).
   * <br />
   * <br />
   * This method does not throw an I/O exception. If a file cannot be copied for
   * any reason it will be added to a list of failed files. The process then
   * continues until all files have been checked. The list of failed files is
   * then returned. You should check the list is empty to ensure the patch
   * has completed successfully.
   * <br />
   * <br />
   * @param currentFolder the root folder containing the current set of files
   *                      (latest version)
   * @param oldFolder the root folder containing the old set of files
   * @param destFolder the folder where the patch is to be created (this folder
   *                   need not exist, it will automatically be created) it is
   *                   recommended the folder either does not exist or is empty
   *                   before this method is called
   * @return a <code>List</code> of <code>File</code> references to files in the
   *         'current' folder that could not be copied/included in the patch due
   *         to an I/O exception - the value is never <code>null</code> - always
   *         check the list is empty to ensure the patch was complete
   */
  public static List createPatch( File currentFolder, File oldFolder, File destFolder )
  {
	  return createPatch( currentFolder, true, oldFolder, destFolder, true, FileDateComparator.CHRONOLOGICAL );
  }
  
  /**
   * Create a patch for an old set of files.
   * <br />
   * This method will compare the content of two folders and deduce which files
   * in the 'current' set of files need to be copied into the 'old' set of files
   * in order to bring the old set of files up-to-date.
   * <br />
   * No changes will be made to the old set of files, i.e. this method does not
   * <i>perform</i> a patch, it <i>creates</i> a patch by copying the new or
   * modified files into a blank 'destination' folder. The patch is created in
   * such a way that copying the 'destination' folder over the 'old' folder will
   * patch the old set of files and bring them up-to-date with the 'current'
   * set of files.
   * <br />
   * <br />
   * This method will automatically recurse into sub folders. It will only
   * include modified files in the patch if instructed to, by setting the
   * <code>patchModifiedFiles</code> parameter to <code>true</code>. (The
   * definition of 'modified' being those files whose timestamp is more recent
   * than the old copy of the file).
   * <br />
   * <br />
   * This method does not throw an I/O exception. If a file cannot be copied for
   * any reason it will be added to a list of failed files. The process then
   * continues until all files have been checked. The list of failed files is
   * then returned. You should check the list is empty to ensure the patch
   * has completed successfully.
   * <br />
   * <br />
   * @param currentFolder the root folder containing the current set of files
   *                      (latest version)
   * @param oldFolder the root folder containing the old set of files
   * @param destFolder the folder where the patch is to be created (this folder
   *                   need not exist, it will automatically be created) it is
   *                   recommended the folder either does not exist or is empty
   *                   before this method is called
   * @param patchModifiedFiles indicates if modified files should be included
   *                           in the patch
   * @return a <code>List</code> of <code>File</code> references to files in the
   *         'current' folder that could not be copied/included in the patch due
   *         to an I/O exception - the value is never <code>null</code> - always
   *         check the list is empty to ensure the patch was complete
   */
  public static List createPatch( File currentFolder, File oldFolder, File destFolder, boolean patchModifiedFiles )
  {
	  return createPatch( currentFolder, true, oldFolder, destFolder, patchModifiedFiles, FileDateComparator.CHRONOLOGICAL );
  }
  
  /**
   * Create a patch for an old set of files.
   * <br />
   * This method will compare the content of two folders and deduce which files
   * in the 'current' set of files need to be copied into the 'old' set of files
   * in order to bring the old set of files up-to-date.
   * <br />
   * No changes will be made to the old set of files, i.e. this method does not
   * <i>perform</i> a patch, it <i>creates</i> a patch by copying the new or
   * modified files into a blank 'destination' folder. The patch is created in
   * such a way that copying the 'destination' folder over the 'old' folder will
   * patch the old set of files and bring them up-to-date with the 'current'
   * set of files.
   * <br />
   * <br />
   * This method will only recurse into sub folders if instructed to, by setting
   * the <code>recursive</code> parameter to <code>true</code>. It will
   * automatically include modified files in the patch. (The definition of
   * 'modified' being those files whose timestamp is more recent than the old
   * copy of the file).
   * <br />
   * <br />
   * This method does not throw an I/O exception. If a file cannot be copied for
   * any reason it will be added to a list of failed files. The process then
   * continues until all files have been checked. The list of failed files is
   * then returned. You should check the list is empty to ensure the patch
   * has completed successfully.
   * <br />
   * <br />
   * @param currentFolder the root folder containing the current set of files
   *                      (latest version)
   * @param recursive indicates if the patch should include sub-folders
   * @param oldFolder the root folder containing the old set of files
   * @param destFolder the folder where the patch is to be created (this folder
   *                   need not exist, it will automatically be created) it is
   *                   recommended the folder either does not exist or is empty
   *                   before this method is called
   * @return a <code>List</code> of <code>File</code> references to files in the
   *         'current' folder that could not be copied/included in the patch due
   *         to an I/O exception - the value is never <code>null</code> - always
   *         check the list is empty to ensure the patch was complete
   */
  public static List createPatch( File currentFolder, boolean recursive, File oldFolder, File destFolder )
  {
	  return createPatch( currentFolder, recursive, oldFolder, destFolder, true, FileDateComparator.CHRONOLOGICAL );
  }
  
  /**
   * Create a patch for an old set of files.
   * <br />
   * This method will compare the content of two folders and deduce which files
   * in the 'current' set of files need to be copied into the 'old' set of files
   * in order to bring the old set of files up-to-date.
   * <br />
   * No changes will be made to the old set of files, i.e. this method does not
   * <i>perform</i> a patch, it <i>creates</i> a patch by copying the new or
   * modified files into a blank 'destination' folder. The patch is created in
   * such a way that copying the 'destination' folder over the 'old' folder will
   * patch the old set of files and bring them up-to-date with the 'current'
   * set of files.
   * <br />
   * <br />
   * This method will only recurse into sub folders if instructed to, by setting
   * the <code>recursive</code> parameter to <code>true</code>. It will only
   * include modified files in the patch if instructed to, by setting the
   * <code>patchModifiedFiles</code> parameter to <code>true</code>. (The
   * definition of 'modified' being those files whose timestamp is more recent
   * than the old copy of the file).
   * <br />
   * <br />
   * This method does not throw an I/O exception. If a file cannot be copied for
   * any reason it will be added to a list of failed files. The process then
   * continues until all files have been checked. The list of failed files is
   * then returned. You should check the list is empty to ensure the patch
   * has completed successfully.
   * <br />
   * <br />
   * @param currentFolder the root folder containing the current set of files
   *                      (latest version)
   * @param recursive indicates if the patch should include sub-folders
   * @param oldFolder the root folder containing the old set of files
   * @param destFolder the folder where the patch is to be created (this folder
   *                   need not exist, it will automatically be created) it is
   *                   recommended the folder either does not exist or is empty
   *                   before this method is called
   * @param patchModifiedFiles indicates if modified files should be included
   *                           in the patch
   * @return a <code>List</code> of <code>File</code> references to files in the
   *         'current' folder that could not be copied/included in the patch due
   *         to an I/O exception - the value is never <code>null</code> - always
   *         check the list is empty to ensure the patch was complete
   */
  public static List createPatch( File currentFolder, boolean recursive, File oldFolder, File destFolder, boolean patchModifiedFiles )
  {
	  return createPatch( currentFolder, recursive, oldFolder, destFolder, patchModifiedFiles, FileDateComparator.CHRONOLOGICAL );
  }
  
  /**
   * Create a patch for an old set of files.
   * <br />
   * This method will compare the content of two folders and deduce which files
   * in the 'current' set of files need to be copied into the 'old' set of files
   * in order to bring the old set of files up-to-date.
   * <br />
   * No changes will be made to the old set of files, i.e. this method does not
   * <i>perform</i> a patch, it <i>creates</i> a patch by copying the new or
   * modified files into a blank 'destination' folder. The patch is created in
   * such a way that copying the 'destination' folder over the 'old' folder will
   * patch the old set of files and bring them up-to-date with the 'current'
   * set of files.
   * <br />
   * <br />
   * This method will only recurse into sub folders if instructed to, by setting
   * the <code>recursive</code> parameter to <code>true</code>. It will only
   * include modified files in the patch if instructed to, by setting the
   * <code>patchModifiedFiles</code> parameter to <code>true</code>.
   * <br />
   * Whether a file has been modified is determined by the
   * <code>fileComparator</code>. If the <code>fileComparator</code> concludes
   * the current file is 'greater' than the old file, then the file is
   * considered more recent and therefore needs replacing.
   * <br />
   * <br />
   * This method does not throw an I/O exception. If a file cannot be copied for
   * any reason it will be added to a list of failed files. The process then
   * continues until all files have been checked. The list of failed files is
   * then returned. You should check the list is empty to ensure the patch
   * has completed successfully.
   * <br />
   * <br />
   * @param currentFolder the root folder containing the current set of files
   *                      (latest version)
   * @param recursive indicates if the patch should include sub-folders
   * @param oldFolder the root folder containing the old set of files
   * @param destFolder the folder where the patch is to be created (this folder
   *                   need not exist, it will automatically be created) it is
   *                   recommended the folder either does not exist or is empty
   *                   before this method is called
   * @param patchModifiedFiles indicates if modified files should be included
   *                           in the patch
   * @param fileComparator the comparator used to determine which file is more
   *                       recent and therefore if it needs to be patched
   * @return a <code>List</code> of <code>File</code> references to files in the
   *         'current' folder that could not be copied/included in the patch due
   *         to an I/O exception - the value is never <code>null</code> - always
   *         check the list is empty to ensure the patch was complete
   */
  public static List<File> createPatch( File currentFolder, boolean recursive, File oldFolder, File destFolder, boolean patchModifiedFiles, Comparator<File> fileComparator )
  {
	  List<File> failedFiles = new ArrayList<File>();
	  
	  // loop through all the current files
	  File[] currentFiles = currentFolder.listFiles();
	  for (int fileI=0; fileI<currentFiles.length; fileI++)
	  {
		  File currentFile = currentFiles[fileI];
		  
		  if (currentFile.isDirectory())
		  {
			  if (recursive)
			  {
				  // recurse down into this folder
				  String subFolderName = currentFile.getName();
				  failedFiles.addAll( createPatch( currentFile,
                                                   recursive,
				                                   new File( oldFolder, subFolderName ),
				                                   new File( destFolder, subFolderName ),
				                                   patchModifiedFiles,
				                                   fileComparator ) );
			  }
		  }
		  else
		  {
			  File oldFile = new File( oldFolder, currentFile.getName() );
			  
			  boolean needPatch;
			  if (!oldFile.exists())
			  {
				  // completely new file - always needs to be in the patch
				  needPatch = true;
			  }
			  else
			  {
				  if (patchModifiedFiles)
					  // patch the modified file if it is newer than the old one
					  needPatch = fileComparator.compare( currentFile, oldFile ) > 0;
				  else
					  // don't patch any modified files
					  needPatch = false;
			  }
			  
			  if (needPatch)
			  {
				  // ensure dest folder exists
				  if (!destFolder.exists())
					  destFolder.mkdirs();
				  
				  File destFile = new File( destFolder, currentFile.getName() );
				  try
				  {
					  Files.copyFile( currentFile, destFile );
				  }
				  catch (IOException e)
				  {
					  failedFiles.add( currentFile );
				  }
			  }
		  }
	  }
	  
	  return failedFiles;
  }


  /*
   * Static helper class
   */
  
  /**
   * Helper class that creates a new file name based on some rules/configuration.
   */
  public static class NewFileNameModel
  {

    /*
     * Member variables
     */

    private String _fileName;
    private String _fileIndexSeparator;
    private int _fileIndex;
    private NumberFormat _fileIndexFormat;
    private String _fileExtn;


    /*
     * Constructor(s)
     */

    /**
     * Create a new file-name model ready to construct a new file name.
     * <br />
     * <br />
     * @param preferredFileName the preferred name for the new file
     * @since 1.0
     */
    public NewFileNameModel( String preferredFileName )
    {
      String thePartOfTheNameBeforeTheExtn;

      /* break down the file name into two parts */
      int extnIndex = preferredFileName.lastIndexOf( '.' );
      if (extnIndex > -1)
      {
        thePartOfTheNameBeforeTheExtn = preferredFileName.substring( 0, extnIndex );
        _fileExtn = preferredFileName.substring( extnIndex );
      }
      else
      {
        thePartOfTheNameBeforeTheExtn = preferredFileName;
        _fileExtn = "";
      }

      /* check if the name ends with a numeric value */
      if (   (thePartOfTheNameBeforeTheExtn.length() > 0)
          && (Character.isDigit( thePartOfTheNameBeforeTheExtn.charAt( thePartOfTheNameBeforeTheExtn.length()-1 ) ))  )
      {
        int lastCharacterIndex = thePartOfTheNameBeforeTheExtn.length() - 1;
        int digitCount = 0;
        while (   (lastCharacterIndex > -1)
               && (Character.isDigit( thePartOfTheNameBeforeTheExtn.charAt( lastCharacterIndex ) ))  )
        {
          digitCount ++;
          lastCharacterIndex --;
        }
        int firstDigitIndex = lastCharacterIndex + 1;

        _fileName = thePartOfTheNameBeforeTheExtn.substring( 0, firstDigitIndex );
        _fileIndexSeparator = "";
        try
        {
          _fileIndex = (int)Long.parseLong( thePartOfTheNameBeforeTheExtn.substring( firstDigitIndex ) );
          _fileIndexFormat = NumberFormat.getInstance();
          _fileIndexFormat.setMinimumIntegerDigits( digitCount );
        }
        catch (NumberFormatException e)
        {
          /* error parsing the value - use default values */
          _fileName = thePartOfTheNameBeforeTheExtn;
          _fileIndexSeparator = "_";
          _fileIndexFormat = NumberFormat.getInstance();
          _fileIndex = 1;
        }
      }
      else
      {
        /* not indexed, use a new index and an underscore to separate the name from its index */
        _fileName = thePartOfTheNameBeforeTheExtn;
        _fileIndexSeparator = "_";
        _fileIndexFormat = NumberFormat.getInstance();
        _fileIndex = 1;
      }
    }


    /*
     * Methods
     */

    /**
     * Creates a new file name, modifying the preferred file name slightly (by
     * adding an index) and incrementing the index each time this method is
     * called such that each call returns a different file name.
     * <br />
     * <br />
     * @return a new file name
     * @since 1.0
     */
    public String getName()
    {
      /* return the preferred file name, postfixed with a numeric counter */
      _fileIndex ++;
      return _fileName + _fileIndexSeparator + _fileIndexFormat.format( _fileIndex ) + _fileExtn;
    }

  }

}


/*
 * Change History
 *
 * $Log: Files.java,v $
 * Revision 1.6.2.2  2008/02/24 20:11:48  Steve
 * d some comments.
 * Added createPatch method(s).
 *
 * Revision 1.6.2.1  2008/01/19 16:34:45  Steve
 * d some comments.
 * Added createPatch method(s).
 * @
 * text
 * @d *
 * Revision 1.6  2006/09/16 17:35:46  Steve
 * Fixed bug in copyFolder/copyFolderContents method caused by using mkdir instead of mkdirs.
 *
 * Revision 1.5  2006/09/16 16:49:15  Steve
 * Fixed some comments.
 * Added createPatch method(s).
 *
 * Revision 1.4  2006/09/06 20:52:02  Steve
 * Added legal disclaimer.
 *
 * Revision 1.3  2006/04/27 21:07:16  Steve
 * Removed un-necessary imports.
 *
 * Revision 1.2  2006/04/13 21:01:11  Steve
 * Added file footer, which contains the revision history.
 *
 * Revision 1.1  2006/04/12 21:37:21  Steve
 * Initial revision.
 *
 */
