/*
 * FilesTest.java
 * Part of the B'am library - a collection of general-purpose utility classes.
 * 
 * Copyright (C)2006 Stephen Battey
 * Contact: resource@lucidviews.net
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

package net.lucidviews.util.io;

import junit.framework.TestCase;

import java.io.IOException;
import java.io.File;
import java.util.List;


/**
 * Test harness for the <code>Files</code> class.
 * <br />
 * <br />
 * @author Stephen Battey
 * @version $Revision: 1.5 $
 * @since 1.0
 */
public class FilesTest extends TestCase
{
	
	/*
	 * Local constants
	 */
	
	/** the folder where the test data is stored */
	protected static final File TEST_DATA_FOLDER = new File( "./Test/Files" );
	
	/** folder for the create-patch test case, where the current set of files are stored */
	protected static final File PATCH_CURRENT_FOLDER = new File( TEST_DATA_FOLDER, "CreatePatch/CurrentFiles" );
	
	/** folder for the create-patch test case, where the old set of files are stored */
	protected static final File PATCH_OLD_FOLDER = new File( TEST_DATA_FOLDER, "CreatePatch/OldFiles" );
	
	/** folder for the create-patch test case, where the patch should be created */
	protected static final File PATCH_DEST_FOLDER = new File( TEST_DATA_FOLDER, "CreatePatch/PatchedFiles" );
	
	protected static final String PATCH_NEW_FILE = "newfile.html";
	protected static final String PATCH_NEW_FILE_SUB_FOLDER = "sub3/sub4/newfile.html";
	protected static final String PATCH_MODIFIED_FILE = "modified.txt";
	protected static final String PATCH_MODIFIED_FILE_SUB_FOLDER = "sub1/modified.txt";
	protected static final String PATCH_UNCHANGED_FILE = "unchanged.dat";
	protected static final String PATCH_UNCHANGED_FILE_SUB_FOLDER = "sub1/unchanged.dat";
	protected static final String PATCH_MODIFIED_FOLDER = "sub1";
	protected static final String PATCH_UNCHANGED_FOLDER = "sub2";
	protected static final String PATCH_NEW_FOLDER = "sub3";
	
	
	/*
	 * Static methods
	 */
	
	/**
	 * Runs the test in the command console.
	 * <br />
	 * @param args not used
	 */
	public static void main( String[] args )
	{
		junit.textui.TestRunner.run( FilesTest.class );
	}
	
	
	/*
	 * Test cases
	 */
	
	public void testIsDuplicate()
	{
		File testFolder = new File( TEST_DATA_FOLDER, "IsDuplicate" );
		
		File base = new File( testFolder, "base.txt" );
		File baseCopy = new File( testFolder, "Copy of base.txt" );
		File notBaseCopy = new File( testFolder, "not-base.txt" );
		
		// compare two identical files
		try
		{
			if (!Files.isDuplicate( base, baseCopy ))
				fail( "Base & copy were not found to be duplicates." );
		}
		catch (Exception e)
		{
			fail( "Unexpected exception while comparing base & copy." + e );
		}
		
		// compare two different files
		try
		{
			if (Files.isDuplicate( base, notBaseCopy ))
				fail( "Base & not-base were found to be duplicates." );
		}
		catch (Exception e)
		{
			fail( "Unexpected exception while comparing base & not-base." + e );
		}
	}

	public void testIsDescendant()
	{
		File testFolder = new File( TEST_DATA_FOLDER, "IsDescendant" );

		File descFile = new File( testFolder, "Sub1/Sub11/Sub111/sub.txt" );
		File descFolder = new File( testFolder, "Sub2/Sub21/Sub211/Sub2111" );

		// a file that is a descendant
		try
		{
			if (!Files.isDescendant( testFolder, descFile ))
				fail( "file descendant not detected." );
		}
		catch (Exception e)
		{
			fail( "Unexpected exception while checking file descendant." + e );
		}

		// a folder that is a descendant
		try
		{
			if (!Files.isDescendant( testFolder, descFolder ))
				fail( "folder descendant not detected." );
		}
		catch (Exception e)
		{
			fail( "Unexpected exception while checking folder descendant." + e );
		}

		// not a descendant
		try
		{
			if (Files.isDescendant( descFolder, testFolder ))
				fail( "non-descendant folder detected." );
		}
		catch (Exception e)
		{
			fail( "Unexpected exception while checking non-descendant folder." + e );
		}
	}

	public void subtestDeleteFolder() throws Exception
	// This test case depends on the 'copy folder' test case running before it.
	// Therefore it is *not* prefixed by 'test' and is called at the end of the
	// copy folder test case.
	{
		File testFolder = new File( TEST_DATA_FOLDER, "DeleteFolder" );

		File src = new File( testFolder, "Data" );
		File dest = new File( testFolder, "WorkingCopy" );

		// check the destination does not already exist
		if (new File( dest, "Data" ).exists())
		{
			fail( "the WorkingCopy folder already contains a Data folder - please delete the WorkingCopy/Data folder and try again." );
		}
		else
		{
			// make a copy of the test data
			if (!Files.copyFolder( src, dest ))
			{
				fail( "unable to copy test data to a new location" );
			}
			else
			{
				// check the copy contains the expected files and folders
				File copy = new File( dest, "Data" );

				if (!copy.exists())
					fail( "the copy of the Data folder was not created." );
				if (!new File( copy, "f0.txt" ).exists())
					fail( "the file in the root of the copied folder was not created." );
				if (!new File( copy, "f1" ).exists())
					fail( "a sub-folder was not created." );
				if (!new File( copy, "f1/f1.txt" ).exists())
					fail( "a file in a sub-folder was not created." );
				if (!new File( copy, "f2/f21" ).exists())
					fail( "a sub-sub-folder was not created." );
				if (!new File( copy, "f2/f21/f2.txt" ).exists())
					fail( "a file in a sub-sub-folder was not created." );
	
				if (!Files.deleteFolder( copy ))
				{
					fail( "deleteFolder was unsuccessful." );
				}
				else
				{
					if (copy.exists())
						fail( "the copy of the Data folder was not deleted." );
					if (new File( copy, "f0.txt" ).exists())
						fail( "the file in the root of the copied folder was not deleted." );
					if (new File( copy, "f1" ).exists())
						fail( "a sub-folder was not deleted." );
					if (new File( copy, "f1/f1.txt" ).exists())
						fail( "a file in a sub-folder was not deleted." );
					if (new File( copy, "f2/f21" ).exists())
						fail( "a sub-sub-folder was not deleted." );
					if (new File( copy, "f2/f21/f2.txt" ).exists())
						fail( "a file in a sub-sub-folder was not deleted." );
				}
			}
		}
		
		// run dependent test cases
		subtestCreateNewFileWithPrefixedName();
		subtestCreateNewFileInSubFolder();
		subtestCopyFile();
	}

	public void subtestCopyFile() throws Exception
	// This test case depends on the 'delete folder' test case running before it.
	// Therefore it is *not* prefixed by 'test' and is called at the end of the
	// delete folder test case.
	{
		File testFolder = new File( TEST_DATA_FOLDER, "CopyFile" );

		// prepare the destination folder
		File destFolder = new File( testFolder, "Dest" );
		if (destFolder.exists())
		{
			if (!Files.deleteFolder( destFolder ))
			{
				fail( "the Dest folder could not be cleaned." );
			}
		}
		if (!destFolder.mkdirs())
		{
			fail( "the Dest folder could not be created." );
		}

		// try copy - keeping the name
		try
		{
			File f1 = new File( testFolder, "CannonStreet.jpg" );
			File f1Copy = Files.copyFile( f1, destFolder );
			if (!f1Copy.getName().equals( "CannonStreet.jpg" ))
				fail( "the copy of f1 did not have the correct name." );
			else if (!f1Copy.exists())
				fail( "the copy of f1 was not created." );
			else if (!Files.isDuplicate( f1, f1Copy ))
				fail( "the copied f1 file was not identical." );
		}
		catch (Exception e)
		{
			fail( "Unexpected exception while creating f1 copy." + e );
		}

		// try copy - changing the name
		try
		{
			File f2 = new File( testFolder, "ICC_NIA.jpg" );
			File f2Copy = Files.copyFile( f2, destFolder, "Canals.jpg" );
			if (!f2Copy.getName().equals( "Canals.jpg" ))
				fail( "the copy of f2 did not have the correct name." );
			else if (!f2Copy.exists())
				fail( "the copy of f2 was not created." );
			else if (!Files.isDuplicate( f2, f2Copy ))
				fail( "the copied f2 file was not identical." );
		}
		catch (Exception e)
		{
			fail( "Unexpected exception while creating f2 copy." + e );
		}
	}

	public void testCopyFolder() throws Exception
	{
		File testFolder = new File( TEST_DATA_FOLDER, "CopyFolder" );

		// check the src contains the expected files and folders
		File src = new File( testFolder, "Data" );
		if (!src.exists())
			fail( "the Data folder was not setup correctly." );
		if (!new File( src, "f0.txt" ).exists())
			fail( "the Data folder was not setup correctly." );
		if (!new File( src, "f1" ).exists())
			fail( "the Data folder was not setup correctly." );
		if (!new File( src, "f1/f1.txt" ).exists())
			fail( "the Data folder was not setup correctly." );
		if (!new File( src, "f2/f21" ).exists())
			fail( "the Data folder was not setup correctly." );
		if (!new File( src, "f2/f21/f2.txt" ).exists())
			fail( "the Data folder was not setup correctly." );

		// check the destination folder does not already exist
		File dest = new File( testFolder, "WorkingCopy" );
		File destData = new File( dest, "Data" );
		if (destData.exists())
		{
			Files.deleteFolder( destData );
		}
		if (destData.exists())
		{
			fail( "the WorkingCopy folder already contains a Data folder - please delete the WorkingCopy/Data folder and try again." );
		}
		else
		{
			// copy the folder - by using copyFolder we also test copyFolderContents
			if (!Files.copyFolder( src, dest ))
			{
				fail( "unable to copy test data from '" + src.getAbsolutePath() + "' to a new location: '" + dest.getAbsolutePath() + "'" );
			}
			else
			{
				File copy = new File( dest, "Data" );

				if (!copy.exists())
					fail( "the copy of the Data folder was not created." );
				if (!new File( copy, "f0.txt" ).exists())
					fail( "the file in the root of the copied folder was not created." );
				if (!new File( copy, "f1" ).exists())
					fail( "a sub-folder was not created." );
				if (!new File( copy, "f1/f1.txt" ).exists())
					fail( "a file in a sub-folder was not created." );
				if (!new File( copy, "f2/f21" ).exists())
					fail( "a sub-sub-folder was not created." );
				if (!new File( copy, "f2/f21/f2.txt" ).exists())
					fail( "a file in a sub-sub-folder was not created." );
			}
		}
		
		// run dependent test case
		subtestDeleteFolder();
	}
	
	public void testCreatePatch() throws IOException
	{
		File destFolder = new File( PATCH_DEST_FOLDER, "default" );
		
		// ensure dest folder is clean before commencing
		if (destFolder.exists())
			Files.deleteFolder( destFolder );
		if (destFolder.exists())
			fail( "Unable to clean destFolder: " + destFolder.getAbsolutePath() );
		
		List failedFiles = Files.createPatch( PATCH_CURRENT_FOLDER, PATCH_OLD_FOLDER, destFolder );
		
		// ensure patch was complete
		assertEquals( 0, failedFiles.size() );
		
		// check new files are patched
		assertFileExists( destFolder, PATCH_NEW_FILE );
		// and in sub-folder
		assertFileExists( destFolder, PATCH_NEW_FILE_SUB_FOLDER );
		
		// check modified files are patched
		File orgFile1 = new File( PATCH_CURRENT_FOLDER, PATCH_MODIFIED_FILE );
		assertFileEquals( destFolder, PATCH_MODIFIED_FILE, orgFile1 );
		// and in sub-folder
		File orgFile2 = new File( PATCH_CURRENT_FOLDER, PATCH_MODIFIED_FILE_SUB_FOLDER );
		assertFileEquals( destFolder, PATCH_MODIFIED_FILE_SUB_FOLDER, orgFile2 );
		
		// check un-modified files are not patched
		assertFileNotExists( destFolder, PATCH_UNCHANGED_FILE );
		// and in sub-folder
		assertFileNotExists( destFolder, PATCH_UNCHANGED_FILE_SUB_FOLDER );
		
		// check sub-folders with no changes are not created
		assertFileNotExists( destFolder, PATCH_UNCHANGED_FOLDER );
	}
	
	public void testCreatePatchNoRecursion() throws IOException
	{
		File destFolder = new File( PATCH_DEST_FOLDER, "no-recursion" );
		
		// ensure dest folder is clean before commencing
		if (destFolder.exists())
			Files.deleteFolder( destFolder );
		if (destFolder.exists())
			fail( "Unable to clean destFolder: " + destFolder.getAbsolutePath() );
		
		List failedFiles = Files.createPatch( PATCH_CURRENT_FOLDER, false, PATCH_OLD_FOLDER, destFolder );
		
		// ensure patch was complete
		assertEquals( 0, failedFiles.size() );
		
		// check new files are patched
		assertFileExists( destFolder, PATCH_NEW_FILE );
		// but not in sub-folder
		assertFileNotExists( destFolder, PATCH_NEW_FILE_SUB_FOLDER );
		
		// check modified files are patched
		File orgFile1 = new File( PATCH_CURRENT_FOLDER, PATCH_MODIFIED_FILE );
		assertFileEquals( destFolder, PATCH_MODIFIED_FILE, orgFile1 );
		// but not in sub-folder
		assertFileNotExists( destFolder, PATCH_MODIFIED_FILE_SUB_FOLDER );
		
		// check un-modified files are not patched
		assertFileNotExists( destFolder, PATCH_UNCHANGED_FILE );
		// and not in sub-folder
		assertFileNotExists( destFolder, PATCH_UNCHANGED_FILE_SUB_FOLDER );
		
		// check sub-folders are not created
		assertFileNotExists( destFolder, PATCH_NEW_FOLDER );
		assertFileNotExists( destFolder, PATCH_MODIFIED_FOLDER );
		assertFileNotExists( destFolder, PATCH_UNCHANGED_FOLDER );
	}
	
	public void testCreatePatchNotModifiedFiles() throws IOException
	{
		File destFolder = new File( PATCH_DEST_FOLDER, "new-files-only" );
		
		// ensure dest folder is clean before commencing
		if (destFolder.exists())
			Files.deleteFolder( destFolder );
		if (destFolder.exists())
			fail( "Unable to clean destFolder: " + destFolder.getAbsolutePath() );
		
		List failedFiles = Files.createPatch( PATCH_CURRENT_FOLDER, PATCH_OLD_FOLDER, destFolder, false );
		
		// ensure patch was complete
		assertEquals( 0, failedFiles.size() );
		
		// check new files are patched
		assertFileExists( destFolder, PATCH_NEW_FILE );
		// and in sub-folder
		assertFileExists( destFolder, PATCH_NEW_FILE_SUB_FOLDER );
		
		// check modified files are not patched
		assertFileNotExists( destFolder, PATCH_MODIFIED_FILE );
		// and in sub-folder
		assertFileNotExists( destFolder, PATCH_MODIFIED_FILE_SUB_FOLDER );
		
		// check un-modified files are not patched
		assertFileNotExists( destFolder, PATCH_UNCHANGED_FILE );
		// and in sub-folder
		assertFileNotExists( destFolder, PATCH_UNCHANGED_FILE_SUB_FOLDER );
		
		// check sub-folders with no changes are not created
		assertFileNotExists( destFolder, PATCH_MODIFIED_FOLDER );
		assertFileNotExists( destFolder, PATCH_UNCHANGED_FOLDER );
	}
	
	public void subtestCreateNewFileWithPrefixedName() throws Exception
	// This test case depends on the 'delete folder' test case running before it.
	// Therefore it is *not* prefixed by 'test' and is called at the end of the
	// delete folder test case.
	{
		final File TEST_FOLDER = new File( TEST_DATA_FOLDER, "CreateNewFile/PrefixName" );
		final String EXPECTED_FOLDER_PATH = TEST_FOLDER.getAbsolutePath() + File.separator;
		
		// ensure test folder is clean before commencing
		if (TEST_FOLDER.exists())
			Files.deleteFolder( TEST_FOLDER );
		TEST_FOLDER.mkdirs();
		
		
		// 1: file name does not contain a number
		{
			final String TEST_FILE_PREFIX = "sample";
			final String TEST_FILE_SUFFIX = ".html";
			final String TEST_FILE_NAME = TEST_FILE_PREFIX + TEST_FILE_SUFFIX;
			
			File file1 = Files.createNewFile( TEST_FOLDER, TEST_FILE_NAME );
			assertFilePathEquals( file1, EXPECTED_FOLDER_PATH + TEST_FILE_NAME );
			
			File file2 = Files.createNewFile( TEST_FOLDER, TEST_FILE_NAME );
			assertFilePathEquals( file2, EXPECTED_FOLDER_PATH + TEST_FILE_PREFIX + "_2" + TEST_FILE_SUFFIX );
			
			File file3 = Files.createNewFile( TEST_FOLDER, TEST_FILE_NAME );
			assertFilePathEquals( file3, EXPECTED_FOLDER_PATH + TEST_FILE_PREFIX + "_3" + TEST_FILE_SUFFIX );
			
			File file4 = Files.createNewFile( TEST_FOLDER, TEST_FILE_NAME );
			assertFilePathEquals( file4, EXPECTED_FOLDER_PATH + TEST_FILE_PREFIX + "_4" + TEST_FILE_SUFFIX );
			
			File file5 = Files.createNewFile( TEST_FOLDER, TEST_FILE_NAME );
			assertFilePathEquals( file5, EXPECTED_FOLDER_PATH + TEST_FILE_PREFIX + "_5" + TEST_FILE_SUFFIX );
		}
		
		// 2: file name ends with a number
		{
			final String TEST_FILE_PREFIX = "sample";
			final String TEST_FILE_INDEX = "98";
			final String TEST_FILE_SUFFIX = ".dat";
			final String TEST_FILE_NAME = TEST_FILE_PREFIX + TEST_FILE_INDEX + TEST_FILE_SUFFIX;
			
			File file1 = Files.createNewFile( TEST_FOLDER, TEST_FILE_NAME );
			assertFilePathEquals( file1, EXPECTED_FOLDER_PATH + TEST_FILE_NAME );
			
			File file2 = Files.createNewFile( TEST_FOLDER, TEST_FILE_NAME );
			assertFilePathEquals( file2, EXPECTED_FOLDER_PATH + TEST_FILE_PREFIX + "99" + TEST_FILE_SUFFIX );
			
			File file3 = Files.createNewFile( TEST_FOLDER, TEST_FILE_NAME );
			assertFilePathEquals( file3, EXPECTED_FOLDER_PATH + TEST_FILE_PREFIX + "100" + TEST_FILE_SUFFIX );
			
			File file4 = Files.createNewFile( TEST_FOLDER, TEST_FILE_NAME );
			assertFilePathEquals( file4, EXPECTED_FOLDER_PATH + TEST_FILE_PREFIX + "101" + TEST_FILE_SUFFIX );
			
			File file5 = Files.createNewFile( TEST_FOLDER, TEST_FILE_NAME );
			assertFilePathEquals( file5, EXPECTED_FOLDER_PATH + TEST_FILE_PREFIX + "102" + TEST_FILE_SUFFIX );
		}
		
		// 2: file name ends with a number
		{
			final String TEST_FILE_PREFIX = "";
			final String TEST_FILE_INDEX = "098";
			final String TEST_FILE_SUFFIX = ".txt";
			final String TEST_FILE_NAME = TEST_FILE_PREFIX + TEST_FILE_INDEX + TEST_FILE_SUFFIX;
			
			File file1 = Files.createNewFile( TEST_FOLDER, TEST_FILE_NAME );
			assertFilePathEquals( file1, EXPECTED_FOLDER_PATH + TEST_FILE_NAME );
			
			File file2 = Files.createNewFile( TEST_FOLDER, TEST_FILE_NAME );
			assertFilePathEquals( file2, EXPECTED_FOLDER_PATH + TEST_FILE_PREFIX + "099" + TEST_FILE_SUFFIX );
			
			File file3 = Files.createNewFile( TEST_FOLDER, TEST_FILE_NAME );
			assertFilePathEquals( file3, EXPECTED_FOLDER_PATH + TEST_FILE_PREFIX + "100" + TEST_FILE_SUFFIX );
			
			File file4 = Files.createNewFile( TEST_FOLDER, TEST_FILE_NAME );
			assertFilePathEquals( file4, EXPECTED_FOLDER_PATH + TEST_FILE_PREFIX + "101" + TEST_FILE_SUFFIX );
			
			File file5 = Files.createNewFile( TEST_FOLDER, TEST_FILE_NAME );
			assertFilePathEquals( file5, EXPECTED_FOLDER_PATH + TEST_FILE_PREFIX + "102" + TEST_FILE_SUFFIX );
		}
	}
	
	public void subtestCreateNewFileInSubFolder() throws Exception
	{
		final File TEST_FOLDER = new File( TEST_DATA_FOLDER, "CreateNewFile/PrefixFolder" );
		final String EXPECTED_FOLDER_PATH = TEST_FOLDER.getAbsolutePath() + File.separator;
		final String TEST_FILE_PREFIX = "sample";
		final String TEST_FILE_NAME = TEST_FILE_PREFIX + ".html";
		
		// ensure test folder is clean before commencing
		if (TEST_FOLDER.exists())
			Files.deleteFolder( TEST_FOLDER );
		TEST_FOLDER.mkdirs();
		
		
		File file1 = Files.createNewFile( TEST_FILE_NAME, TEST_FOLDER );
		assertFilePathEquals( file1, EXPECTED_FOLDER_PATH + TEST_FILE_NAME );
		
		File file2 = Files.createNewFile( TEST_FILE_NAME, TEST_FOLDER );
		assertFilePathEquals( file2, EXPECTED_FOLDER_PATH + TEST_FILE_PREFIX + "_2" + File.separator + TEST_FILE_NAME );
		
		File file3 = Files.createNewFile( TEST_FILE_NAME, TEST_FOLDER );
		assertFilePathEquals( file3, EXPECTED_FOLDER_PATH + TEST_FILE_PREFIX + "_3" + File.separator + TEST_FILE_NAME );
		
		File file4 = Files.createNewFile( TEST_FILE_NAME, TEST_FOLDER );
		assertFilePathEquals( file4, EXPECTED_FOLDER_PATH + TEST_FILE_PREFIX + "_4" + File.separator + TEST_FILE_NAME );
		
		File file5 = Files.createNewFile( TEST_FILE_NAME, TEST_FOLDER );
		assertFilePathEquals( file5, EXPECTED_FOLDER_PATH + TEST_FILE_PREFIX + "_5" + File.separator + TEST_FILE_NAME );
	}
	
	
	/*
	 * Assertion methods for checking files
	 */
	
	public static void assertFilePathEquals( File file, String expectedPath )
	{
		assertEquals( expectedPath, file.getAbsolutePath() );
	}
	
	public static void assertFileExists( File file )
	{
		assertTrue( "Expected file to exist at path: '" + file.getAbsolutePath() + "'.", file.exists() );
	}
	
	public static void assertFileExists( File folder, String path )
	{
		assertFileExists( new File( folder, path ) );
	}
	
	public static void assertFileNotExists( File file )
	{
		assertFalse( "Unexpected file exists at path: '" + file.getAbsolutePath() + "'.", file.exists() );
	}
	
	public static void assertFileNotExists( File folder, String path )
	{
		assertFileNotExists( new File( folder, path ) );
	}
	
	public static void assertFileEquals( File file, File otherFile ) throws IOException
	{
		assertTrue( Files.isDuplicate( file, otherFile ) );
	}
	
	public static void assertFileEquals( File folder, String path, File otherFile ) throws IOException
	{
		assertFileEquals( new File( folder, path ), otherFile );
	}
	
	public static void assertFileEquals( File file, File otherFolder, String otherPath ) throws IOException
	{
		assertFileEquals( file, new File( otherFolder, otherPath ) );
	}
	
	public static void assertFileEquals( File folder, String path, File otherFolder, String otherPath ) throws IOException
	{
		assertFileEquals( new File( folder, path ), new File( otherFolder, otherPath ) );
	}
	
}


/*
 * Change History
 *
 * $Log: FilesTest.java,v $
 * Revision 1.5  2006/09/16 17:36:06  Steve
 * Added more information to one of the failure messages.
 *
 * Revision 1.4  2006/09/16 16:50:04  Steve
 * Modified test cases to conform to JUnit testing.
 * Added test cases for createPatch method(s).
 *
 * Revision 1.3  2006/09/06 20:52:02  Steve
 * Added legal disclaimer.
 *
 * Revision 1.2  2006/04/13 21:02:03  Steve
 * Removed un-necessary imports.
 *
 * Revision 1.1  2006/04/12 21:37:38  Steve
 * Initial revision.
 *
 */
