/*
 * Strings.java
 * Part of the B'am library - a collection of general-purpose utility classes.
 * 
 * Copyright (C)2006 Stephen Battey
 * Contact: resource@lucidviews.net
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

package net.lucidviews.util.text;

import net.lucidviews.util.EnumValue;
import net.lucidviews.util.Enumeration;

import java.awt.Color;
import java.io.IOException;
import java.io.File;
import java.util.NoSuchElementException;
import java.util.Iterator;
import java.util.Collection;
import java.util.StringTokenizer;


/**
 * A collection of utility methods for performing operations on
 * <code>String</code>s.
 * <br />
 * <br />
 * @author Stephen Battey
 * @version $Revision: 1.7.2.1 $
 * @since 1.0
 */
public class Strings
{
	
	/*
	 * Exported constants
	 */
	
	/** Special default value that ensures a valid Boolean value is parsed
	 *  (i.e. to be used when no default value is applicable).
	 *  @see #parseBoolean(Boolean,String) */ 
	public static final Boolean BOOLEAN_MUST_BE_VALID = new Boolean( false );
	
	/** Special default value that ensures a valid Integer value is parsed
	 *  (i.e. to be used when no default value is applicable).
	 *  @see #parseLong(Long, String) */
	public static final Long LONG_MUST_BE_VALID = new Long( 0 );
	
	/** Special default value that ensures a valid floating-point value is
	 *  parsed (i.e. to be used when no default value is applicable).
	 *  @see #parseDouble(Double, String) */
	public static final Double DOUBLE_MUST_BE_VALID = new Double( 0 );
	
	/** Special default value that ensures a valid File is parsed and references
	 *  a file that already exists on the file system.
	 *  @see #parseFile(File, String) */
	public static final File FILE_MUST_EXIST = new File( "" );
	
	/** Special default value that ensures a valid File is parsed and references
	 *  an existing file or a location where a new file can be created.
	 *  @see #parseFile(File, String) */
	public static final File ENSURE_FILE_EXISTS = new File( "" );
	
	/** Special default value that ensures a valid File is parsed and references
	 *  a folder that already exists on the file system.
	 *  @see #parseFolder(File, String) */
	public static final File FOLDER_MUST_EXIST = new File( "" );
	
	/** Special default value that ensures a valid File is parsed and references
	 *  an existing folder or a location where a new folder can be created.
	 *  @see #parseFolder(File, String) */
	public static final File ENSURE_FOLDER_EXISTS = new File( "" );
	
	/** Special default value that ensures a valid File is parsed and references
	 *  a location within an existing folder on the file system.
	 *  @see #parseFile(File, String)
	 *  @see #parseFolder(File, String) */
	public static final File PARENT_FOLDER_MUST_EXIST = new File( "" );
	
	/** Special default value that ensures a valid File is parsed and references
	 *  a location within an existing folder or in a folder that can be created.
	 *  @see #parseFile(File, String)
	 *  @see #parseFolder(File, String) */
	public static final File ENSURE_PARENT_FOLDER_EXISTS = new File( "" );
	
	/** Special default value that ensures a valid Class is parsed
	 *  (i.e. to be used when no default value is applicable).
	 *  @see #parseClass(Class, String) */
	public static final Class CLASS_MUST_BE_VALID = ClassMustBeValid.class;
	
	/** Special default value that ensures a valid Enumeration value is parsed
	 *  (i.e. to be used when no default value is applicable).
	 *  @see #parseEnumValue(EnumValue, String, Enumeration) */
	public static final EnumValue ENUM_VALUE_MUST_BE_VALID = new EnumValueMustBeValid();
	
	/** Special default value that ensures a valid Object is parsed
	 *  (i.e. to be used when no default value is applicable).
	 *  @see #parseObject(Object, String, Collection) */
	public static final Object OBJECT_MUST_BE_VALID = new Object();
	
	/** Special default value that ensures a valid Color is parsed
	 *  (i.e. to be used when no default value is applicable).
	 *  @see #parseColor(Color, String) */
	public static final Color COLOR_MUST_BE_VALID = new Color( 0 );
	
	/** Bitmask for extracting the alpha component from a argb value. */
	public static final long COLOR_ALPHA_MASK = 0xff000000;
	
	/** The character used to prefix hex colour codes. */
	public static final char COLOR_HEX_PREFIX = '#';
	
	/** The maximum length of a hex colour string (exlucing '#' prefix)
	 *  without an alpha component. */
	public static final char COLOR_HEX_LENGTH_WITHOUT_ALPHA = 6;
	
	/** A string containing all the characters Java treats as whitespace. */
	public static final String WHITESPACE_CHARS = " \t\n\r\f";
	
	
	/*
	 * Constructor(s)
	 */
	
	/**
	 * The default constructor is protected to prevent instantiation.
	 */
	protected Strings()
	{
	}
	
	
	/*
	 * Static methods
	 */
	
	/**
	 * Parse a <code>Boolean</code> value from a <code>String</code>.
	 * <br />
	 * If the <code>string</code> is invalid then an exception will be thrown.
	 * <br />
	 * The return value is guaranteed to be non-<code>null</code>. Thus, the
	 * following line of code will never cause a
	 * <code>NullPointerException</code>:
	 * <code>
	 * <br />boolean value = Strings.parseBoolean( <i>string</i> ).booleanValue();
	 * </code>
	 * <br />
	 * Although it may throw a <code>ParseValueException</code> if the
	 * <code><i>string</i></code> is invalid.
	 * <br />
	 * <br />
	 * @param string the <code>String</code> containing the Boolean value
	 * @return the <code>Boolean</code> value represented by the
	 *         <code>string</code>
	 * @throws ParseValueException if the <code>String</code> value could not be
	 *                             parsed as a valid <code>Boolean</code> value
	 */
	public static Boolean parseBoolean( String string ) throws ParseValueException
	{
		return parseBoolean( BOOLEAN_MUST_BE_VALID, string );
	}
	
	/**
	 * Parse a <code>Boolean</code> value from a <code>String</code>, returning
	 * the default value if the string was invalid.
	 * <br />
	 * If the <code>string</code> is invalid and the <code>defaultValue</code>
	 * is set to the constant <code>Strings.BOOLEAN_MUST_BE_VALID</code>, an
	 * exception will be thrown.
	 * <br />
	 * The return value is guaranteed to be non-<code>null</code>, provided the
	 * <code>defaultValue</code> is not <code>null</code>. Thus, none of the
	 * following lines of code will cause a <code>NullPointerException</code>:
	 * <code>
	 * <br />Boolean value = Strings.parseBoolean( <i>default</i>, <i>string</i> );
	 * <br />boolean value = Strings.parseBoolean( <i>default</i>, <i>string</i> ).booleanValue();
	 * <br />boolean value = Strings.parseBoolean( Strings.BOOLEAN_MUST_BE_VALID, <i>string</i> ).booleanValue();
	 * </code>
	 * <br />
	 * Although the last line may throw a <code>ParseValueException</code> if
	 * the <code><i>string</i></code> is invalid.
	 * <br />
	 * <br />
	 * @param defaultValue the default value to be used if the
	 *                     <code>string</code> is not valid
	 * @param string the <code>String</code> containing the Boolean value
	 * @return the <code>Boolean</code> value represented by the
	 *         <code>string</code>, or the default value if the
	 *         <code>string</code> is invalid
	 * @throws ParseValueException if the <code>String</code> value must be
	 *                             valid but could not be parsed as a valid
	 *                             <code>Boolean</code> value
	 */
	public static Boolean parseBoolean( Boolean defaultValue, String string ) throws ParseValueException
	{
		if (   (string.equalsIgnoreCase( "true" ))
		    || (string.equalsIgnoreCase( "yes" ))   )
			return Boolean.TRUE;
		
		if (   (string.equalsIgnoreCase( "false" ))
		    || (string.equalsIgnoreCase( "no" ))   )
			return Boolean.FALSE;
		
		if (defaultValue == BOOLEAN_MUST_BE_VALID)
			throw new ParseValueException( "Boolean value could not be parsed from String '" + string + "'. Acceptable values are 'true', 'false', 'yes', 'no'." );
		
		return defaultValue;
	}
	
	/**
	 * Parse a <code>Boolean</code> value from a <code>String</code>, returning
	 * the default value if the string was invalid.
	 * <br />
	 * The return value is guaranteed to be non-<code>null</code>, provided the
	 * <code>defaultValue</code> is not <code>null</code>. Thus, none of the
	 * following lines of code will cause a <code>NullPointerException</code>:
	 * <code>
	 * <br />Boolean value = Strings.parseBoolean( <i>string</i>, <i>default</i> );
	 * <br />boolean value = Strings.parseBoolean( <i>string</i>, <i>default</i> ).booleanValue();
	 * <br />
	 * <br />
	 * @param string the <code>String</code> containing the Boolean value
	 * @param defaultValue the default value to be used if the
	 *                     <code>string</code> is not valid
	 * @return the <code>Boolean</code> value represented by the
	 *         <code>string</code>, or the default value if the
	 *         <code>string</code> is invalid
	 */
	public static Boolean parseBoolean( String string, Boolean defaultValue )
	{
		if (   (string.equalsIgnoreCase( "true" ))
		    || (string.equalsIgnoreCase( "yes" ))   )
			return Boolean.TRUE;
		
		if (   (string.equalsIgnoreCase( "false" ))
		    || (string.equalsIgnoreCase( "no" ))   )
			return Boolean.FALSE;
		
		return defaultValue;
	}
	
	/**
	 * Parse a <code>boolean</code> value from a <code>String</code>, returning
	 * the default value if the string was invalid.
	 * <br />
	 * <br />
	 * @param string the <code>String</code> containing the Boolean value
	 * @param defaultValue the default value to be used if the
	 *                     <code>string</code> is not valid
	 * @return the <code>boolean</code> value represented by the
	 *         <code>string</code>, or the default value if the
	 *         <code>string</code> is invalid
	 */
	public static boolean parseBoolean( String string, boolean defaultValue )
	{
		Boolean defaultBoolean = (defaultValue) ? Boolean.TRUE : Boolean.FALSE;
		return parseBoolean( string, defaultBoolean ).booleanValue();
	}

	/**
	 * Parse a <code>Long</code> value from a <code>String</code>.
	 * <br />
	 * If the <code>string</code> is invalid then an exception will be thrown.
	 * <br />
	 * The return value is guaranteed to be non-<code>null</code>. Thus, the
	 * following line of code will never cause a
	 * <code>NullPointerException</code>:
	 * <code>
	 * <br />long value = Strings.parseLong( <i>string</i> ).longValue();
	 * <br />int value = Strings.parseLong( <i>string</i> ).intValue();
	 * </code>
	 * <br />
	 * Although they may throw a <code>ParseValueException</code> if the
	 * <code><i>string</i></code> is invalid.
	 * <br />
	 * <br />
	 * @param string the <code>String</code> containing the numeric, integer
	 *               value
	 * @return the <code>Long</code> value represented by the
	 *         <code>string</code>
	 * @throws ParseValueException if the <code>String</code> could not be
	 *                             parsed as a valid <code>Long</code> value
	 */
	public static Long parseLong( String string ) throws ParseValueException
	{
		return parseLong( LONG_MUST_BE_VALID, string );
	}
	
	/**
	 * Parse a <code>Long</code> value from a <code>String</code>, returning
	 * the default value if the string was invalid.
	 * <br />
	 * If the <code>string</code> is invalid and the <code>defaultValue</code>
	 * is set to the constant <code>Strings.LONG_MUST_BE_VALID</code>, an
	 * exception will be thrown.
	 * <br />
	 * The return value is guaranteed to be non-<code>null</code>, provided the
	 * <code>defaultValue</code> is not <code>null</code>. Thus, none of the
	 * following lines of code will cause a <code>NullPointerException</code>:
	 * <code>
	 * <br />Long value = Strings.parseLong( <i>default</i>, <i>string</i> );
	 * <br />long value = Strings.parseLong( <i>default</i>, <i>string</i> ).longValue();
	 * <br />int value = Strings.parseLong( <i>default</i>, <i>string</i> ).intValue();
	 * <br />int value = Strings.parseLong( Strings.LONG_MUST_BE_VALID, <i>string</i> ).intValue();
	 * </code>
	 * <br />
	 * Although the last line may throw a <code>ParseValueException</code> if
	 * the <code><i>string</i></code> is invalid.
	 * <br />
	 * <br />
	 * @param defaultValue the default value to be used if the
	 *                     <code>string</code> is not valid
	 * @param string the <code>String</code> containing the numeric, integer
	 *               value
	 * @return the <code>Long</code> value represented by the
	 *         <code>string</code>, or the default value if the
	 *         <code>string</code> is invalid
	 * @throws ParseValueException if the <code>String</code> value must be
	 *                             valid but could not be parsed as a valid
	 *                             <code>Long</code> value
	 */
	public static Long parseLong( Long defaultValue, String string ) throws ParseValueException
	{
		try
		{
			return new Long( string );
		}
		catch (NumberFormatException e)
		{
			if (defaultValue == LONG_MUST_BE_VALID)
				throw new ParseValueException( "Long value could not be parsed from String '" + string + "'.", e );
			
			return defaultValue;
		}
	}
	
	/**
	 * Parse a <code>Long</code> value from a <code>String</code>, returning
	 * the default value if the string was invalid.
	 * <br />
	 * If the <code>string</code> is invalid and the <code>defaultValue</code>
	 * is set to the constant <code>Strings.LONG_MUST_BE_VALID</code>, an
	 * exception will be thrown.
	 * <br />
	 * The return value is guaranteed to be non-<code>null</code>, provided the
	 * <code>defaultValue</code> is not <code>null</code>. Thus, none of the
	 * following lines of code will cause a <code>NullPointerException</code>:
	 * <code>
	 * <br />Long value = Strings.parseLong( <i>string</i>, <i>default</i> );
	 * <br />long value = Strings.parseLong( <i>string</i>, <i>default</i> ).longValue();
	 * <br />int value = Strings.parseLong( <i>string</i>, <i>default</i> ).intValue();
	 * </code>
	 * <br />
	 * <br />
	 * @param string the <code>String</code> containing the numeric, integer
	 *               value
	 * @param defaultValue the default value to be used if the
	 *                     <code>string</code> is not valid
	 * @return the <code>Long</code> value represented by the
	 *         <code>string</code>, or the default value if the
	 *         <code>string</code> is invalid
	 */
	public static Long parseLong( String string, Long defaultValue )
	{
		try
		{
			return new Long( string );
		}
		catch (NumberFormatException e)
		{
			return defaultValue;
		}
	}
	
	/**
	 * Parse a <code>Long</code> value from a <code>String</code>, returning
	 * the default value if the string was invalid.
	 * <br />
	 * <br />
	 * @param string the <code>String</code> containing the numeric integer
	 *               value
	 * @param defaultValue the default value to be used if the
	 *                     <code>string</code> is not valid
	 * @return the <code>long</code> value represented by the
	 *         <code>string</code>, or the default value if the
	 *         <code>string</code> is invalid
	 */
	public static long parseLong( String string, long defaultValue )
	{
		try
		{
			return new Long( string ).longValue();
		}
		catch (NumberFormatException e)
		{
			return defaultValue;
		}
	}
	
	/**
	 * Parse a <code>Double</code> value from a <code>String</code>.
	 * <br />
	 * If the <code>string</code> is invalid then an exception will be thrown.
	 * <br />
	 * The return value is guaranteed to be non-<code>null</code>. Thus, the
	 * following line of code will never cause a
	 * <code>NullPointerException</code>:
	 * <code>
	 * <br />double value = Strings.parseDouble( <i>string</i> ).doubleValue();
	 * <br />float value = Strings.parseDouble( <i>string</i> ).floatValue();
	 * </code>
	 * <br />
	 * Although they may throw a <code>ParseValueException</code> if the
	 * <code><i>string</i></code> is invalid.
	 * <br />
	 * <br />
	 * @param string the <code>String</code> containing the numeric,
	 *               floating-point value
	 * @return the <code>Double</code> value represented by the
	 *         <code>string</code>
	 * @throws ParseValueException if the <code>String</code> could not be
	 *                             parsed as a valid <code>Double</code> value
	 */
	public static Double parseDouble( String string ) throws ParseValueException
	{
		return parseDouble( DOUBLE_MUST_BE_VALID, string );
	}
	
	/**
	 * Parse a <code>Double</code> value from a <code>String</code>, returning
	 * the default value if the string was invalid.
	 * <br />
	 * If the <code>string</code> is invalid and the <code>defaultValue</code>
	 * is set to the constant <code>Strings.DOUBLE_MUST_BE_VALID</code>, an
	 * exception will be thrown.
	 * <br />
	 * The return value is guaranteed to be non-<code>null</code>, provided the
	 * <code>defaultValue</code> is not <code>null</code>. Thus, none of the
	 * following lines of code will cause a <code>NullPointerException</code>:
	 * <code>
	 * <br />Double value = Strings.parseDouble( <i>default</i>, <i>string</i> );
	 * <br />double value = Strings.parseDouble( <i>default</i>, <i>string</i>, ).doubleValue();
	 * <br />float value = Strings.parseDouble( <i>default</i>, <i>string</i> ).floatValue();
	 * <br />float value = Strings.parseDouble( Strings.DOUBLE_MUST_BE_VALID, <i>string</i> ).floatValue();
	 * </code>
	 * <br />
	 * Although the last line may throw a <code>ParseValueException</code> if
	 * the <code><i>string</i></code> is invalid.
	 * <br />
	 * <br />
	 * @param defaultValue the default value to be used if the
	 *                     <code>string</code> is not valid
	 * @param string the <code>String</code> containing the numeric,
	 *               floating-point value
	 * @return the <code>Double</code> value represented by the
	 *         <code>string</code>, or the default value if the
	 *         <code>string</code> is invalid
	 * @throws ParseValueException if the <code>String</code> value must be
	 *                             valid but could not be parsed as a valid
	 *                             <code>Double</code> value
	 */
	public static Double parseDouble( Double defaultValue, String string ) throws ParseValueException
	{
		try
		{
			return new Double( string );
		}
		catch (NumberFormatException e)
		{
			if (defaultValue == DOUBLE_MUST_BE_VALID)
				throw new ParseValueException( "Double value could not be parsed from String '" + string + "'.", e );
			
			return defaultValue;
		}
	}
	
	/**
	 * Parse a <code>Double</code> value from a <code>String</code>, returning
	 * the default value if the string was invalid.
	 * <br />
	 * The return value is guaranteed to be non-<code>null</code>, provided the
	 * <code>defaultValue</code> is not <code>null</code>. Thus, none of the
	 * following lines of code will cause a <code>NullPointerException</code>:
	 * <code>
	 * <br />Double value = Strings.parseDouble( <i>string</i>, <i>default</i> );
	 * <br />double value = Strings.parseDouble( <i>string</i>, <i>default</i> ).doubleValue();
	 * <br />float value = Strings.parseDouble( <i>string</i>, <i>default</i> ).floatValue();
	 * </code>
	 * <br />
	 * <br />
	 * @param string the <code>String</code> containing the numeric,
	 *               floating-point value
	 * @param defaultValue the default value to be used if the
	 *                     <code>string</code> is not valid
	 * @return the <code>Double</code> value represented by the
	 *         <code>string</code>, or the default value if the
	 *         <code>string</code> is invalid
	 */
	public static Double parseDouble( String string, Double defaultValue )
	{
		try
		{
			return new Double( string );
		}
		catch (NumberFormatException e)
		{
			return defaultValue;
		}
	}
	
	/**
	 * Parse a <code>Double</code> value from a <code>String</code>, returning
	 * the default value if the string was invalid.
	 * <br />
	 * <br />
	 * @param string the <code>String</code> containing the numeric,
	 *               floating-point value
	 * @param defaultValue the default value to be used if the
	 *                     <code>string</code> is not valid
	 * @return the <code>Double</code> value represented by the
	 *         <code>string</code>, or the default value if the
	 *         <code>string</code> is invalid
	 */
	public static double parseDouble( String string, double defaultValue )
	{
		try
		{
			return new Double( string ).doubleValue();
		}
		catch (NumberFormatException e)
		{
			return defaultValue;
		}
	}
	
	/**
	 * Parse a <code>File</code> value from a <code>String</code>.
	 * <br />
	 * If the path does not reference a file then an exception will be thrown.
	 * <br />
	 * The return value is guaranteed to be non-<code>null</code>.
	 * <br />
	 * <br />
	 * @param string the <code>String</code> value containing the file path
	 * @return the <code>File</code> at the system path specified by the
	 *         <code>string</code>
	 * @throws ParseValueException if a <code>File</code> could not be found at
	 *                             the location specified in the
	 *                             <code>String</code>
	 */
	public static File parseFile( String string ) throws ParseValueException
	{
		return parseFile( FILE_MUST_EXIST, string );
	}
	
	/**
	 * Parse a <code>File</code> value from a <code>String</code>, returning
	 * the default value if the string was invalid.
	 * <br />
	 * The input <code>string</code> should specify the path to a file, either
	 * absolute or relative to the given folder.
	 * <br />
	 * If the path does not reference a file and the <code>defaultValue</code>
	 * is set to the constant <code>Strings.FILE_MUST_EXIST</code>, an exception
	 * will be thrown.
	 * <br />
	 * If the path does not reference a file and the <code>defaultValue</code>
	 * is set to the constant <code>Strings.ENSURE_FILE_EXISTS</code>, a new,
	 * empty file will be created and a reference to that new file returned.
	 * <br />
	 * If the path references a file in a folder that does not exist and the
	 * <code>defaultValue</code> is set to the constant
	 * <code>Strings.PARENT_FOLDER_MUST_EXIST</code>, an exception will be
	 * thrown.
	 * <br />
	 * If the path references a file in a folder that does not exist and the
	 * <code>defaultValue</code> is set to the constant
	 * <code>Strings.ENSURE_PARENT_FOLDER_EXISTS</code>, the parent folder will
	 * be created and a reference to an inexistant file (in a existant folder)
	 * will be returned.
	 * <br />
	 * The return value is guaranteed to be non-<code>null</code>, provided the
	 * <code>defaultValue</code> is not <code>null</code>.
	 * <br />
	 * <br />
	 * @param defaultValue the default value to be used if the
	 *                     <code>string</code> is not valid
	 * @param string the <code>String</code> value containing the file path
	 * @param relativeToFolder the base location to use if the file path
	 *                         defined by the <code>string</code> is relative,
	 *                         rather than an absolute path
	 * @return the <code>File</code> referenced by the path in the
	 *         <code>string</code>, or the default value if the
	 *         <code>string</code> is invalid
	 * @throws ParseValueException if the path specified by the
	 *                             <code>string</code> value must be valid but
	 *                             could not be parsed as a valid
	 *                             <code>File</code>
	 */
	public static File parseFile( File defaultValue, String string, File relativeToFolder ) throws ParseValueException
	{
		File file = new File( string );
		
		/* check if the file path is absolute */
		if (   (!file.isAbsolute())
		    && (relativeToFolder != null)  )
			file = new File( relativeToFolder, string );
		
		if (defaultValue == FILE_MUST_EXIST)
		{
			if (!file.exists())
				throw new ParseValueException( "The path '" + file.getAbsolutePath() + "' references a file that does not exist." );
		}
		else if (defaultValue == ENSURE_FILE_EXISTS)
		{
			if (!file.exists())
			{
				try
				{
					file.createNewFile();
				}
				catch (IOException e)
				{
					throw new ParseValueException( "The path '" + file.getAbsolutePath() + "' references a file that does not exist and could not be created.", e );
				}
			}
		}
		else if (defaultValue == PARENT_FOLDER_MUST_EXIST)
		{
			File parent = file.getParentFile();
			/* handle files in root folder */
			if (parent != null)
			{
				if (!parent.exists())
					throw new ParseValueException( "The path '" + file.getAbsolutePath() + "' references a file in a folder '" + parent.getAbsolutePath() + "' that does not exist." );
			}
		}
		else if (defaultValue == PARENT_FOLDER_MUST_EXIST)
		{
			File parent = file.getParentFile();
			/* handle files in root folder */
			if (parent != null)
			{
				if (!parent.exists())
				{
					parent.mkdirs();
					if (!parent.exists())
						throw new ParseValueException( "The path '" + file.getAbsolutePath() + "' references a file in a folder '" + parent.getAbsolutePath() + "' that does not exist and could not be created." );
				}
			}
		}
		else
		{
			if (!file.exists())
				file = defaultValue;
		}
		
		/* the file must be a file, not a folder (although allow any value for the default) */
		if (   (file != defaultValue)
		    && (!file.isFile())  )
			throw new ParseValueException( "The path '" + file.getAbsolutePath() + "' references a folder, not a file." );
		
		return file;
	}
	
	/**
	 * Parse a <code>File</code> value from a <code>String</code>, returning
	 * the default value if the string was invalid.
	 * <br />
	 * The input <code>string</code> should specify the path to a file, either
	 * absolute or relative to the current working folder.
	 * <br />
	 * If the path does not reference a file and the <code>defaultValue</code>
	 * is set to the constant <code>Strings.FILE_MUST_EXIST</code>, an exception
	 * will be thrown.
	 * <br />
	 * If the path does not reference a file and the <code>defaultValue</code>
	 * is set to the constant <code>Strings.ENSURE_FILE_EXISTS</code>, a new,
	 * empty file will be created and a reference to that new file returned.
	 * <br />
	 * If the path references a file in a folder that does not exist and the
	 * <code>defaultValue</code> is set to the constant
	 * <code>Strings.PARENT_FOLDER_MUST_EXIST</code>, an exception will be
	 * thrown.
	 * <br />
	 * If the path references a file in a folder that does not exist and the
	 * <code>defaultValue</code> is set to the constant
	 * <code>Strings.ENSURE_PARENT_FOLDER_EXISTS</code>, the parent folder will
	 * be created and a reference to an inexistant file (in a existant folder)
	 * will be returned.
	 * <br />
	 * The return value is guaranteed to be non-<code>null</code>, provided the
	 * <code>defaultValue</code> is not <code>null</code>.
	 * <br />
	 * <br />
	 * @param defaultValue the default value to be used if the
	 *                     <code>string</code> is not valid
	 * @param string the <code>String</code> value containing the file path
	 * @return the <code>File</code> referenced by the path in the
	 *         <code>string</code>, or the default value if the
	 *         <code>string</code> is invalid
	 * @throws ParseValueException if the path specified by the
	 *                             <code>string</code> value must be valid but
	 *                             could not be parsed as a valid
	 *                             <code>File</code>
	 */
	public static File parseFile( File defaultValue, String string ) throws ParseValueException
	{
		return parseFile( defaultValue, string, null );
	}
	
	/**
	 * Parse a <code>File</code> value from a <code>String</code>, returning
	 * the default value if the string was invalid.
	 * <br />
	 * The input <code>string</code> should specify the path to a file, either
	 * absolute or relative to the given folder.
	 * <br />
	 * If the path does not reference a file and the <code>defaultValue</code>
	 * is set to the constant <code>Strings.ENSURE_FILE_EXISTS</code>, a new,
	 * empty file will be created and a reference to that new file returned.
	 * Otherwise, if the path does not reference a file, the default value will
	 * be returned.
	 * <br />
	 * If the path references a file in a folder that does not exist and the
	 * <code>defaultValue</code> is set to the constant
	 * <code>Strings.ENSURE_PARENT_FOLDER_EXISTS</code>, the parent folder will
	 * be created and a reference to an inexistant file (in a existant folder)
	 * will be returned. Otherwise, if the path references a file in a folder
	 * that does not exist, the default value will be returned.
	 * <br />
	 * The return value is guaranteed to be non-<code>null</code>, provided the
	 * <code>defaultValue</code> is not <code>null</code>.
	 * <br />
	 * <br />
	 * @param string the <code>String</code> value containing the file path
	 * @param relativeToFolder the base location to use if the file path
	 *                         defined by the <code>string</code> is relative,
	 *                         rather than an absolute path
	 * @param defaultValue the default value to be used if the
	 *                     <code>string</code> is not valid
	 * @return the <code>File</code> referenced by the path in the
	 *         <code>string</code>, or the default value if the
	 *         <code>string</code> is invalid
	 */
	public static File parseFile( String string, File relativeToFolder, File defaultValue )
	{
		try
		{
			return parseFile( defaultValue, string, relativeToFolder );
		}
		catch (ParseValueException e)
		{
			return defaultValue;
		}
	}
	
	/**
	 * Parse a <code>File</code> value from a <code>String</code>, returning
	 * the default value if the string was invalid.
	 * <br />
	 * The input <code>string</code> should specify the path to a file, either
	 * absolute or relative to the current working folder.
	 * <br />
	 * If the path does not reference a file and the <code>defaultValue</code>
	 * is set to the constant <code>Strings.ENSURE_FILE_EXISTS</code>, a new,
	 * empty file will be created and a reference to that new file returned.
	 * Otherwise, if the path does not reference a file, the default value will
	 * be returned.
	 * <br />
	 * If the path references a file in a folder that does not exist and the
	 * <code>defaultValue</code> is set to the constant
	 * <code>Strings.ENSURE_PARENT_FOLDER_EXISTS</code>, the parent folder will
	 * be created and a reference to an inexistant file (in a existant folder)
	 * will be returned. Otherwise, if the path references a file in a folder
	 * that does not exist, the default value will be returned.
	 * <br />
	 * The return value is guaranteed to be non-<code>null</code>, provided the
	 * <code>defaultValue</code> is not <code>null</code>.
	 * <br />
	 * <br />
	 * @param string the <code>String</code> value containing the file path
	 * @param defaultValue the default value to be used if the
	 *                     <code>string</code> is not valid
	 * @return a <code>File</code> referencing the location of the path in the
	 *         <code>string</code>, or the default value if the
	 *         <code>string</code> is invalid
	 */
	public static File parseFile( String string, File defaultValue )
	{
		return parseFile( string, null, defaultValue );
	}
	
	/**
	 * Parse a <code>File</code> value from a <code>String</code>.
	 * <br />
	 * If the path does not reference a folder then an exception will be thrown.
	 * <br />
	 * The return value is guaranteed to be non-<code>null</code>.
	 * <br />
	 * <br />
	 * @param string the <code>String</code> value containing the folder path
	 * @return the <code>File</code> at the system path specified by the
	 *         <code>string</code>
	 * @throws ParseValueException if a <code>File</code> could not be found at
	 *                             the location specified in the
	 *                             <code>String</code>
	 */
	public static File parseFolder( String string ) throws ParseValueException
	{
		return parseFolder( FOLDER_MUST_EXIST, string );
	}
	
	/**
	 * Parse a <code>File</code> value from a <code>String</code>, returning
	 * the default value if the string was invalid.
	 * <br />
	 * The input <code>string</code> should specify the path to a folder, either
	 * absolute or relative to the given folder.
	 * <br />
	 * If the path does not reference a folder and the <code>defaultValue</code>
	 * is set to the constant <code>Strings.FOLDER_MUST_EXIST</code>, an
	 * exception will be thrown.
	 * <br />
	 * If the path does not reference a folder and the <code>defaultValue</code>
	 * is set to the constant <code>Strings.ENSURE_FOLDER_EXISTS</code>, the
	 * folder will be created and a reference to that new folder returned.
	 * <br />
	 * If the path references a sub-folder of a folder that does not exist and
	 * the <code>defaultValue</code> is set to the constant
	 * <code>Strings.PARENT_FOLDER_MUST_EXIST</code>, an exception will be
	 * thrown.
	 * <br />
	 * If the path references a sub-folder of a folder that does not exist and
	 * the <code>defaultValue</code> is set to the constant
	 * <code>Strings.ENSURE_PARENT_FOLDER_EXISTS</code>, the parent folder will
	 * be created and a reference to an inexistant folder (in a existant folder)
	 * will be returned.
	 * <br />
	 * The return value is guaranteed to be non-<code>null</code>, provided the
	 * <code>defaultValue</code> is not <code>null</code>.
	 * <br />
	 * <br />
	 * @param defaultValue the default value to be used if the
	 *                     <code>string</code> is not valid
	 * @param string the <code>String</code> value containing the folder path
	 * @param relativeToFolder the base location to use if the folder path
	 *                         defined by the <code>string</code> is relative,
	 *                         rather than an absolute path
	 * @return a <code>File</code> referencing the location of the path in the
	 *         <code>string</code>, or the default value if the
	 *         <code>string</code> is invalid
	 * @throws ParseValueException if the path specified by the
	 *                             <code>string</code> value must be valid but
	 *                             could not be parsed as a valid
	 *                             <code>File</code>
	 */
	public static File parseFolder( File defaultValue, String string, File relativeToFolder ) throws ParseValueException
	{
		File file = new File( string );
		
		/* check if the file path is absolute */
		if (   (!file.isAbsolute())
		    && (relativeToFolder != null)  )
			file = new File( relativeToFolder, string );
		
		if (defaultValue == FILE_MUST_EXIST)
		{
			if (!file.exists())
				throw new ParseValueException( "The path '" + file.getAbsolutePath() + "' references a folder that does not exist." );
		}
		else if (defaultValue == ENSURE_FOLDER_EXISTS)
		{
			if (!file.exists())
			{
				file.mkdirs();
				if (!file.exists())
					throw new ParseValueException( "The path '" + file.getAbsolutePath() + "' references a folder that does not exist and could not be created." );
			}
		}
		else if (defaultValue == PARENT_FOLDER_MUST_EXIST)
		{
			File parent = file.getParentFile();
			/* handle files in root folder */
			if (parent != null)
			{
				if (!parent.exists())
					throw new ParseValueException( "The path '" + file.getAbsolutePath() + "' references a sub-folder of folder '" + parent.getAbsolutePath() + "' that does not exist." );
			}
		}
		else if (defaultValue == PARENT_FOLDER_MUST_EXIST)
		{
			File parent = file.getParentFile();
			/* handle files in root folder */
			if (parent != null)
			{
				if (!parent.exists())
				{
					parent.mkdirs();
					if (!parent.exists())
						throw new ParseValueException( "The path '" + file.getAbsolutePath() + "' references a sub-folder of folder '" + parent.getAbsolutePath() + "' that does not exist and could not be created." );
				}
			}
		}
		else
		{
			if (!file.exists())
				file = defaultValue;
		}
		
		/* the file must be a folder, not a file (although allow any value for the default) */
		if (   (file != defaultValue)
		    && (!file.isDirectory())  )
			throw new ParseValueException( "The path '" + file.getAbsolutePath() + "' references a file, not a folder." );
		
		return file;
	}
	
	/**
	 * Parse a <code>File</code> value from a <code>String</code>, returning
	 * the default value if the string was invalid.
	 * <br />
	 * The input <code>string</code> should specify the path to a folder, either
	 * absolute or relative to the current working folder.
	 * <br />
	 * If the path does not reference a folder and the <code>defaultValue</code>
	 * is set to the constant <code>Strings.FOLDER_MUST_EXIST</code>, an
	 * exception will be thrown.
	 * <br />
	 * If the path does not reference a folder and the <code>defaultValue</code>
	 * is set to the constant <code>Strings.ENSURE_FOLDER_EXISTS</code>, the
	 * folder will be created and a reference to that new folder returned.
	 * <br />
	 * If the path references a sub-folder of a folder that does not exist and
	 * the <code>defaultValue</code> is set to the constant
	 * <code>Strings.PARENT_FOLDER_MUST_EXIST</code>, an exception will be
	 * thrown.
	 * <br />
	 * If the path references a sub-folder of a folder that does not exist and
	 * the <code>defaultValue</code> is set to the constant
	 * <code>Strings.ENSURE_PARENT_FOLDER_EXISTS</code>, the parent folder will
	 * be created and a reference to an inexistant folder (in a existant folder)
	 * will be returned.
	 * <br />
	 * The return value is guaranteed to be non-<code>null</code>, provided the
	 * <code>defaultValue</code> is not <code>null</code>.
	 * <br />
	 * <br />
	 * @param defaultValue the default value to be used if the
	 *                     <code>string</code> is not valid
	 * @param string the <code>String</code> value containing the folder path
	 * @return a <code>File</code> referencing the location of the path in the
	 *         <code>string</code>, or the default value if the
	 *         <code>string</code> is invalid
	 * @throws ParseValueException if the path specified by the
	 *                             <code>string</code> value must be valid but
	 *                             could not be parsed as a valid
	 *                             <code>File</code>
	 */
	public static File parseFolder( File defaultValue, String string ) throws ParseValueException
	{
		return parseFolder( defaultValue, string, null );
	}
	
	/**
	 * Parse a <code>File</code> value from a <code>String</code>, returning
	 * the default value if the string was invalid.
	 * <br />
	 * The input <code>string</code> should specify the path to a folder, either
	 * absolute or relative to the given folder.
	 * <br />
	 * If the path does not reference a folder and the <code>defaultValue</code>
	 * is set to the constant <code>Strings.ENSURE_FOLDER_EXISTS</code>, the
	 * folder will be created and a reference to that new folder returned.
	 * Otherwise, if the path does not reference a folder, the default value
	 * will be returned.
	 * <br />
	 * If the path references a sub-folder of a folder that does not exist and
	 * the <code>defaultValue</code> is set to the constant
	 * <code>Strings.ENSURE_PARENT_FOLDER_EXISTS</code>, the parent folder will
	 * be created and a reference to an inexistant folder (in a existant folder)
	 * will be returned. Otherwise, if the path references a sub-folder of a
	 * folder that does not exist, the default value will be returned.
	 * <br />
	 * The return value is guaranteed to be non-<code>null</code>, provided the
	 * <code>defaultValue</code> is not <code>null</code>.
	 * <br />
	 * <br />
	 * @param string the <code>String</code> value containing the folder path
	 * @param relativeToFolder the base location to use if the folder path
	 *                         defined by the <code>string</code> is relative,
	 *                         rather than an absolute path
	 * @param defaultValue the default value to be used if the
	 *                     <code>string</code> is not valid
	 * @return a <code>File</code> referencing the location of the path in the
	 *         <code>string</code>, or the default value if the
	 *         <code>string</code> is invalid
	 */
	public static File parseFolder( String string, File relativeToFolder, File defaultValue )
	{
		try
		{
			return parseFolder( defaultValue, string, relativeToFolder );
		}
		catch (ParseValueException e)
		{
			return defaultValue;
		}
	}
	
	/**
	 * Parse a <code>File</code> value from a <code>String</code>, returning
	 * the default value if the string was invalid.
	 * <br />
	 * The input <code>string</code> should specify the path to a folder, either
	 * absolute or relative to the current working folder.
	 * <br />
	 * If the path does not reference a folder and the <code>defaultValue</code>
	 * is set to the constant <code>Strings.ENSURE_FOLDER_EXISTS</code>, the
	 * folder will be created and a reference to that new folder returned.
	 * Otherwise, if the path does not reference a folder, then the default
	 * value will be returned.
	 * <br />
	 * If the path references a sub-folder of a folder that does not exist and
	 * the <code>defaultValue</code> is set to the constant
	 * <code>Strings.ENSURE_PARENT_FOLDER_EXISTS</code>, the parent folder will
	 * be created and a reference to an inexistant folder (in a existant folder)
	 * will be returned. Otherwise, if the path references a sub-folder of a
	 * folder that does not exist, the default value will be returned.
	 * <br />
	 * The return value is guaranteed to be non-<code>null</code>, provided the
	 * <code>defaultValue</code> is not <code>null</code>.
	 * <br />
	 * <br />
	 * @param defaultValue the default value to be used if the
	 *                     <code>string</code> is not valid
	 * @param string the <code>String</code> value containing the folder path
	 * @return a <code>File</code> referencing the location of the path in the
	 *         <code>string</code>, or the default value if the
	 *         <code>string</code> is invalid
	 */
	public static File parseFolder( String string, File defaultValue )
	{
		return parseFolder( string, null, defaultValue );
	}
	
	/**
	 * Parse a Java <code>Class</code> from a <code>String</code>.
	 * <br />
	 * The input <code>string</code> should specify the full, absolute name of a
	 * class.
	 * <br />
	 * If the class name does not reference a class then an exception will be
	 * thrown.
	 * <br />
	 * The return value is guaranteed to be non-<code>null</code>.
	 * <br />
	 * <br />
	 * @param string the <code>String</code> value containing the class name
	 * @return a Java <code>Class</code> found at the given location defined in
	 *         the <code>string</code>
	 * @throws ParseValueException if the path specified by the
	 *                             <code>string</code> value could not be parsed
	 *                             as a valid <code>Class</code>
	 */
	public static Class parseClass( String string ) throws ParseValueException
	{
		return parseClass( CLASS_MUST_BE_VALID, string );
	}
	
	/**
	 * Parse a Java <code>Class</code> from a <code>String</code>, returning
	 * the default value if the string was invalid.
	 * <br />
	 * The input <code>string</code> should specify the name of a class, with
	 * the package path, either absolute or relative to the given package.
	 * <br />
	 * If the class name does not reference a class and the
	 * <code>defaultValue</code> is set to the constant
	 * <code>Strings.CLASS_MUST_BE_VALID</code>, an exception will be thrown.
	 * <br />
	 * The return value is guaranteed to be non-<code>null</code>, provided the
	 * <code>defaultValue</code> is not <code>null</code>.
	 * <br />
	 * <br />
	 * @param defaultValue the default value to be used if the
	 *                     <code>string</code> is not valid
	 * @param string the <code>String</code> value containing the class name
	 * @param relativeToPackage the base location to use if the class name
	 *                          defined by the <code>string</code> is not an
	 *                          absolute path
	 * @return a Java <code>Class</code> found at the given location defined in
	 *         the <code>string</code>, or the default value if the
	 *         <code>string</code> is invalid
	 * @throws ParseValueException if the path specified by the
	 *                             <code>string</code> value must be valid but
	 *                             could not be parsed as a valid
	 *                             <code>Class</code>
	 */
	public static Class parseClass( Class defaultValue, String string, String relativeToPackage ) throws ParseValueException
	{
		Class classValue;
		
		/* check the class exists */
		try
		{
			classValue = Class.forName( string );
		}
		catch (Throwable e)
		{
			if (relativeToPackage == null)
				throw new ParseValueException( "Could not find class '" + string + "'.", e );
			
			// offset the class name from the relative package path
			String offsetPackagePath;
			if (relativeToPackage.endsWith( "." ))
				offsetPackagePath = relativeToPackage + string;
			else
				offsetPackagePath = relativeToPackage + "." + string;
			
			/* check if class exists offset to this package */
			try
			{
				classValue = Class.forName( offsetPackagePath );
			}
			catch (Throwable ne)
			{
				if (defaultValue == CLASS_MUST_BE_VALID)
					throw new ParseValueException( "Could not find class '" + string + "', neither absolute (" + string + ") nor relative to the given package (" + offsetPackagePath + ").", e );

				classValue = defaultValue;
			}
		}
		
		return classValue;
	}
	
	/**
	 * Parse a Java <code>Class</code> from a <code>String</code>, returning
	 * the default value if the string was invalid.
	 * <br />
	 * The input <code>string</code> should specify the full, absolute name of
	 * a class.
	 * <br />
	 * If the class name does not reference a class and the
	 * <code>defaultValue</code> is set to the constant
	 * <code>Strings.CLASS_MUST_BE_VALID</code>, an exception will be thrown.
	 * <br />
	 * The return value is guaranteed to be non-<code>null</code>, provided the
	 * <code>defaultValue</code> is not <code>null</code>.
	 * <br />
	 * <br />
	 * @param defaultValue the default value to be used if the
	 *                     <code>string</code> is not valid
	 * @param string the <code>String</code> value containing the class name
	 * @return a Java <code>Class</code> found at the given location defined in
	 *         the <code>string</code>, or the default value if the
	 *         <code>string</code> is invalid
	 * @throws ParseValueException if the path specified by the
	 *                             <code>string</code> value must be valid but
	 *                             could not be parsed as a valid
	 *                             <code>Class</code>
	 */
	public static Class parseClass( Class defaultValue, String string ) throws ParseValueException
	{
		return parseClass( defaultValue, string, null );
	}
	
	/**
	 * Parse a Java <code>Class</code> from a <code>String</code>, returning
	 * the default value if the string was invalid.
	 * <br />
	 * The input <code>string</code> should specify the name of a class, with
	 * the package path, either absolute or relative to the given package.
	 * <br />
	 * If the class name does not reference a class then the default value will
	 * be returned.
	 * <br />
	 * The return value is guaranteed to be non-<code>null</code>, provided the
	 * <code>defaultValue</code> is not <code>null</code>.
	 * <br />
	 * <br />
	 * @param string the <code>String</code> value containing the class name
	 * @param relativeToPackage the base location to use if the class name
	 *                          defined by the <code>string</code> is not an
	 *                          absolute path
	 * @param defaultValue the default value to be used if the
	 *                     <code>string</code> is not valid
	 * @return a Java <code>Class</code> found at the given location defined in
	 *         the <code>string</code>, or the default value if the
	 *         <code>string</code> is invalid
	 */
	public static Class parseClass( String string, String relativeToPackage, Class defaultValue )
	{
		try
		{
			return parseClass( defaultValue, string, relativeToPackage );
		}
		catch (ParseValueException e)
		{
			return defaultValue;
		}
	}
	
	/**
	 * Parse a Java <code>Class</code> from a <code>String</code>, returning
	 * the default value if the string was invalid.
	 * <br />
	 * The input <code>string</code> should specify the full, absolute name of
	 * a class.
	 * <br />
	 * If the class name does not reference a class then the default value will
	 * be returned.
	 * <br />
	 * The return value is guaranteed to be non-<code>null</code>, provided the
	 * <code>defaultValue</code> is not <code>null</code>.
	 * <br />
	 * <br />
	 * @param string the <code>String</code> value containing the class name
	 * @param defaultValue the default value to be used if the
	 *                     <code>string</code> is not valid
	 * @return a Java <code>Class</code> found at the given location defined in
	 *         the <code>string</code>, or the default value if the
	 *         <code>string</code> is invalid
	 */
	public static Class parseClass( String string, Class defaultValue )
	{
		return parseClass( string, null, defaultValue );
	}
	
	/**
	 * Parse an enumeration value of the given <code>Enumeration</code> type
	 * from a <code>String</code>.
	 * <br />
	 * The input <code>string</code> should specify one of the values in the
	 * <code>enumeration</code>.
	 * <br />
	 * If the string is not valid then an exception will be thrown.
	 * <br />
	 * The return value is guaranteed to be non-<code>null</code>.
	 * <br />
	 * <br />
	 * @param string the <code>String</code> value containing the enum value
	 * @param enumeration the enumeration type
	 * @return the <code>EnumValue</code> referenced by the <code>string</code>
	 * @throws ParseValueException if the <code>string</code> could not be
	 *                             parsed as a valid <code>EnumValue</code>
	 */
	public static EnumValue parseEnumValue( String string, Enumeration enumeration ) throws ParseValueException
	{
		return parseEnumValue( ENUM_VALUE_MUST_BE_VALID, string, enumeration );
	}
	
	/**
	 * Parse an enumeration value of the given <code>Enumeration</code> type
	 * from a <code>String</code>, returning the default value if the string was
	 * invalid.
	 * <br />
	 * The input <code>string</code> should specify one of the values in the
	 * <code>enumeration</code>.
	 * <br />
	 * If the string is not valid and the <code>defaultValue</code> is set to
	 * the constant <code>Strings.ENUM_VALUE_MUST_BE_VALID</code>, an exception
	 * will be thrown.
	 * <br />
	 * The return value is guaranteed to be non-<code>null</code>, provided the
	 * <code>defaultValue</code> is not <code>null</code>.
	 * <br />
	 * <br />
	 * @param defaultValue the default value to be used if the
	 *                     <code>string</code> is not valid
	 * @param string the <code>String</code> value containing the enum value
	 * @param enumeration the enumeration type
	 * @return the <code>EnumValue</code> referenced by the <code>string</code>,
	 *         or the default value if the <code>string</code> is invalid
	 * @throws ParseValueException if the <code>string</code> must reference an
	 *                             enumeration value but could not be parsed as
	 *                             a valid <code>EnumValue</code>
	 */
	public static EnumValue parseEnumValue( EnumValue defaultValue, String string, Enumeration enumeration ) throws ParseValueException
	{
		EnumValue parsedValue = parseEnumValue( string, enumeration, defaultValue );
		
		if (parsedValue == ENUM_VALUE_MUST_BE_VALID)
			throw new ParseValueException( "The string '" + string + "' does not match any value from the " + enumeration.getType().getName() + " enumeration." );
		
		return parsedValue;
	}
	
	/**
	 * Parse an enumeration value of the given <code>Enumeration</code> type
	 * from a <code>String</code>, returning the default value if the string was
	 * invalid.
	 * <br />
	 * The input <code>string</code> should specify one of the values in the
	 * <code>enumeration</code>.
	 * <br />
	 * If the string is not valid then the default value will be returned.
	 * <br />
	 * The return value is guaranteed to be non-<code>null</code>, provided the
	 * <code>defaultValue</code> is not <code>null</code>.
	 * <br />
	 * <br />
	 * @param string the <code>String</code> value containing the enum value
	 * @param enumeration the enumeration type
	 * @param defaultValue the default value to be used if the
	 *                     <code>string</code> is not valid
	 * @return the <code>EnumValue</code> referenced by the <code>string</code>,
	 *         or the default value if the <code>string</code> is invalid
	 */
	public static EnumValue parseEnumValue( String string, Enumeration enumeration, EnumValue defaultValue )
	{
		try
		{
			return enumeration.lookupValue( string );
		}
		catch (NoSuchElementException e)
		{
			return defaultValue;
		}
	}
	
	/**
	 * Parse an object from a <code>String</code>.
	 * <br />
	 * The input <code>string</code> should relate to one of the objects in the
	 * a collection of accepted values.
	 * <br />
	 * If the string is not valid then an exception will be thrown.
	 * <br />
	 * The return value is guaranteed to be non-<code>null</code>.
	 * <br />
	 * <br />
	 * @param string the <code>String</code> value
	 * @param validObjects a collection of acceptable objects
	 * @return one of the <code>Object</code>s from the <code>validValues</code>
	 *         collection, referenced by the <code>string</code>
	 * @throws ParseValueException if the <code>string</code> could not be
	 *                             parsed as a valid <code>Object</code>
	 */
	public static Object parseObject( String string, Collection validObjects ) throws ParseValueException
	{
		return parseObject( OBJECT_MUST_BE_VALID, string, validObjects );
	}
	
	/**
	 * Parse an object from a <code>String</code>, returning the default value
	 * if the string was invalid.
	 * <br />
	 * The input <code>string</code> should relate to one of the objects in the
	 * a collection of accepted values.
	 * <br />
	 * If the string is not valid and the <code>defaultValue</code> is set to
	 * the constant <code>Strings.OBJECT_MUST_BE_VALID</code>, an exception will
	 * be thrown.
	 * <br />
	 * The return value is guaranteed to be non-<code>null</code>, provided the
	 * <code>defaultValue</code> is not <code>null</code>.
	 * <br />
	 * <br />
	 * @param defaultValue the default value to be used if the
	 *                     <code>string</code> is not valid
	 * @param string the <code>String</code> value
	 * @param validObjects a collection of acceptable objects
	 * @return one of the <code>Object</code>s from the <code>validValues</code>
	 *         collection, referenced by the <code>string</code>,
	 *         or the default value if the <code>string</code> is invalid
	 * @throws ParseValueException if the <code>string</code> must reference one
	 *                             of the valid values but could not be parsed
	 *                             as a valid <code>Object</code>
	 */
	public static Object parseObject( Object defaultValue, String string, Collection validObjects ) throws ParseValueException
	{
		Object parsedValue = parseObject( string, validObjects, defaultValue );
		
		if (parsedValue == OBJECT_MUST_BE_VALID)
			throw new ParseValueException( "The string '" + string + "' does not match any of the valid values: " + validObjects + "." );
		
		return parsedValue;
	}
	
	/**
	 * Parse an object from a <code>String</code>, returning the default value
	 * if the string was invalid.
	 * <br />
	 * The input <code>string</code> should relate to one of the objects in the
	 * a collection of accepted values.
	 * <br />
	 * If the string is not valid, the default value will be returned.
	 * <br />
	 * The return value is guaranteed to be non-<code>null</code>, provided the
	 * <code>defaultValue</code> is not <code>null</code>.
	 * <br />
	 * <br />
	 * @param string the <code>String</code> value
	 * @param validObjects a collection of acceptable objects
	 * @param defaultValue the default value to be used if the
	 *                     <code>string</code> is not valid
	 * @return one of the <code>Object</code>s from the <code>validValues</code>
	 *         collection, referenced by the <code>string</code>,
	 *         or the default value if the <code>string</code> is invalid
	 */
	public static Object parseObject( String string, Collection validObjects, Object defaultValue )
	{
		/* iterate through the set of valid values to find a match */
		Iterator validValueItr = validObjects.iterator();
		Object validValue = null;
		while (validValueItr.hasNext())
		{
			validValue = validValueItr.next();

			/* check for a match with this valid value */
			// NB: it is critical to use 'validValue.equals( string )' and not
			//     'string.equals( validValue )', since the latter will almost
			//     always be false, but the valid-value object may allow
			//     comparison with a String and find an equality
			if (validValue.equals( string ))
				return validValue;
		}
		
		return defaultValue;
	}
	
	/**
	 * Parse a <code>Color</code> from a <code>String</code>, returning
	 * the default value if the string was invalid.
	 * <br />
	 * <br />
	 * The input <code>string</code> should specify a colour in either:
	 * <br />
	 * &#149; decimal format:<code> "[AAA] RRR GGG BBB" </code>where
	 * <code>AAA</code>, <code>RRR</code>, <code>GGG</code> and <code>BBB</code>
	 * are 16-bit decimal values (0-255) for the alpha, red, green and blue
	 * components of the colour (alpha optional) each separated by one or more
	 * whitespace characters
	 * <br />
	 * &#149; hexadecimal format:<code> "#[AA]RRGGBB" </code> where
	 * <code>AA</code>, <code>RR</code>, <code>GG</code> and <code>BB</code> are
	 * 16-bit hex values for the alpha, red, green and blue components of the
	 * colour (alpha optional)
	 * <br />
	 * &#149; equivalent decimal value: a single decimal value that equates to
	 * a hexadecimal value in <code> "#[AA]RRGGBB" </code> format
	 * <br />
	 * <br />
	 * If the string does not conform to this format then an exception will be
	 * thrown.
	 * <br />
	 * The return value is guaranteed to be non-<code>null</code>.
	 * <br />
	 * <br />
	 * @param string the <code>String</code> value containing the color
	 *               specification
	 * @return a <code>Color</code> object for the color specified by the
	 *         <code>string</code>
	 * @throws ParseValueException if the string does not conform to the
	 *                             required format
	 */
	public static Color parseColor( String string ) throws ParseValueException
	{
		return parseColor( COLOR_MUST_BE_VALID, string );
	}
	
	/**
	 * Parse a <code>Color</code> from a <code>String</code>, returning
	 * the default value if the string was invalid.
	 * <br />
	 * <br />
	 * The input <code>string</code> should specify a colour in either:
	 * <br />
	 * &#149; decimal format:<code> "[AAA] RRR GGG BBB" </code>where
	 * <code>AAA</code>, <code>RRR</code>, <code>GGG</code> and <code>BBB</code>
	 * are 16-bit decimal values (0-255) for the alpha, red, green and blue
	 * components of the colour (alpha optional) each separated by one or more
	 * whitespace characters
	 * <br />
	 * &#149; hexadecimal format:<code> "#[AA]RRGGBB" </code> where
	 * <code>AA</code>, <code>RR</code>, <code>GG</code> and <code>BB</code> are
	 * 16-bit hex values for the alpha, red, green and blue components of the
	 * colour (alpha optional)
	 * <br />
	 * &#149; equivalent decimal value: a single decimal value that equates to
	 * a hexadecimal value in <code> "#[AA]RRGGBB" </code> format
	 * <br />
	 * <br />
	 * If the string does not conform to this format and the
	 * <code>defaultValue</code> is set to the constant
	 * <code>Strings.COLOR_MUST_BE_VALID</code>, an exception will be thrown.
	 * <br />
	 * The return value is guaranteed to be non-<code>null</code>, provided the
	 * <code>defaultValue</code> is not <code>null</code>.
	 * <br />
	 * <br />
	 * @param string the <code>String</code> value containing the color
	 *               specification
	 * @param defaultValue the default value to be used if the
	 *                     <code>string</code> does not specify a color
	 * @return a <code>Color</code> object for the color specified by the
	 *         <code>string</code>, or the default value if the
	 *         <code>string</code> is invalid
	 * @throws ParseValueException if the color specified by the
	 *                             <code>string</code> value must be valid but
	 *                             the string does not conform to the required
	 *                             format
	 */
	public static Color parseColor( Color defaultValue, String string ) throws ParseValueException
	{
		Color colorValue;
		
		/* check the class exists */
		try
		{
			if (string.charAt( 0 ) == COLOR_HEX_PREFIX)
			{
				// remove # prefix
				string = string.substring( 1 );

				// NB: parse argb as a long (not an int)
				//     since Integer.parseInt can't handle 32-bit values
				long argb = Long.parseLong( string, 16 );
				boolean hasAlpha = string.length() > COLOR_HEX_LENGTH_WITHOUT_ALPHA;
				
				colorValue = new Color( (int)argb, hasAlpha );
			}
			else
			{
				if (Strings.containsAny( string, WHITESPACE_CHARS ))
				{
					int[] colourComponents = parseInts( string, 0, 255 );
					switch (colourComponents.length)
					{
						case 0:
						case 1:
						case 2:
							throw new ParseValueException( "Expected 3 or 4 decimal numbers but only " + colourComponents.length + " numbers were defined." );
						case 3:
							colorValue = new Color( colourComponents[0], colourComponents[1], colourComponents[2] );
						break;
						default:
							colorValue = new Color( colourComponents[1], colourComponents[2], colourComponents[3], colourComponents[0] );
						break;
					}
				}
				else
				{
					// NB: parse argb as a long (not an int)
					//     since Integer.parseInt can't handle 32-bit values
					long argb = Long.parseLong( string );
					// can't determine if alpha is missing, assume it is specified
					
					colorValue = new Color( (int)argb, true );
				}
			}
		}
		catch (Throwable e)
		{
			if (defaultValue == COLOR_MUST_BE_VALID)
				throw new ParseValueException( "Could not create Color from definition string '" + string + "'.", e );
			
			colorValue = defaultValue;
		}
		
		return colorValue;
	}
	
	/**
	 * Parse a <code>Color</code> from a <code>String</code>, returning
	 * the default value if the string was invalid.
	 * <br />
	 * <br />
	 * The input <code>string</code> should specify a colour in either:
	 * <br />
	 * &#149; decimal format:<code> "[AAA] RRR GGG BBB" </code>where
	 * <code>AAA</code>, <code>RRR</code>, <code>GGG</code> and <code>BBB</code>
	 * are 16-bit decimal values (0-255) for the alpha, red, green and blue
	 * components of the colour (alpha optional) each separated by one or more
	 * whitespace characters
	 * <br />
	 * &#149; hexadecimal format:<code> "#[AA]RRGGBB" </code> where
	 * <code>AA</code>, <code>RR</code>, <code>GG</code> and <code>BB</code> are
	 * 16-bit hex values for the alpha, red, green and blue components of the
	 * colour (alpha optional)
	 * <br />
	 * &#149; equivalent decimal value: a single decimal value that equates to
	 * a hexadecimal value in <code> "#[AA]RRGGBB" </code> format
	 * <br />
	 * <br />
	 * If the string does not conform to this format then the default value will
	 * be returned.
	 * <br />
	 * The return value is guaranteed to be non-<code>null</code>, provided the
	 * <code>defaultValue</code> is not <code>null</code>.
	 * <br />
	 * <br />
	 * @param string the <code>String</code> value containing the color
	 *               specification
	 * @param defaultValue the default value to be used if the
	 *                     <code>string</code> does not specify a color
	 * @return a <code>Color</code> object for the color specified by the
	 *         <code>string</code>, or the default value if the
	 *         <code>string</code> is invalid
	 */
	public static Color parseColor( String string, Color defaultValue )
	{
		try
		{
			return parseColor( defaultValue, string );
		}
		catch (ParseValueException e)
		{
			return defaultValue;
		}
	}
	
	/**
	 * Serialize a <code>Color</code> as a <code>String</code>.
	 * <br />
	 * The colour components are encoded into a hex string in the following
	 * format: <code>#[<i>AA</i>]<i>RRGGBB</i></code> (alpha, red, green, blue).
	 * <br />
	 * <br />
	 * @param value the <code>Color</code> value
	 * @return a <code>String</code> representing the given colour
	 */
	public static String serializeColor( Color value )
	{
		StringBuffer buf = new StringBuffer( 9 );
		// expect 4 2-digit components and # symbol
		
		buf.append( COLOR_HEX_PREFIX );
		
		String hexStr = Integer.toHexString( value.getRGB() );
		// ensure correct length
		buf.append( rightAlign( hexStr, 8, '0' ) );
		
		return buf.toString();
	}
	
	/**
	 * Parse a list of integer values from a given string.
	 * <br />
	 * Each value will be separated by one or more whitespace characters.
	 * <br />
	 * The string must only contain numeric values. Any text will cause a parse
	 * exception.
	 * <br />
	 * <br />
	 * @param string the string to be parsed
	 * @return an array of integer values parsed from the string
	 * @throws ParseValueException if any value is outside the specified range
	 *                             or the string contains any non-numeric text
	 */
	public static final int[] parseInts( String string ) throws ParseValueException
	{
		return parseInts( string, WHITESPACE_CHARS, Integer.MIN_VALUE, Integer.MAX_VALUE );
	}
	
	/**
	 * Parse a list of integer values from a given string.
	 * <br />
	 * Each value will be separated by one or more of the characters in the
	 * delimiters string.
	 * <br />
	 * The string must only contain numeric values. Any text will cause a parse
	 * exception.
	 * <br />
	 * <br />
	 * @param string the string to be parsed
	 * @param delimiters a list of characters, any of which may separate the
	 *                   integer values in the input string
	 * @return an array of integer values parsed from the string
	 * @throws ParseValueException if any value is outside the specified range
	 *                             or the string contains any non-numeric text
	 */
	public static final int[] parseInts( String string, String delimiters ) throws ParseValueException
	{
		return parseInts( string, delimiters, Integer.MIN_VALUE, Integer.MAX_VALUE );
	}
	
	/**
	 * Parse a list of integer values from a given string.
	 * <br />
	 * Each value will be separated by one or more whitespace characters.
	 * <br />
	 * Each integer value must be within the specified range, otherwise an
	 * exception will be thrown.
	 * <br />
	 * The string must only contain numeric values. Any text will cause a parse
	 * exception.
	 * <br />
	 * <br />
	 * @param string the string to be parsed
	 * @param minValue the minimum value for each individual number
	 * @param maxValue the maximum value for each individual number
	 * @return an array of integer values parsed from the string
	 * @throws ParseValueException if any value is outside the specified range
	 *                             or the string contains any non-numeric text
	 */
	public static final int[] parseInts( String string, int minValue, int maxValue ) throws ParseValueException
	{
		return parseInts( string, WHITESPACE_CHARS, minValue, maxValue );
	}
	
	/**
	 * Parse a list of integer values from a given string.
	 * <br />
	 * Each value will be separated by one or more of the characters in the
	 * delimiters string.
	 * <br />
	 * Each integer value must be within the specified range, otherwise an
	 * exception will be thrown.
	 * <br />
	 * The string must only contain numeric values. Any text will cause a parse
	 * exception.
	 * <br />
	 * <br />
	 * @param string the string to be parsed
	 * @param delimiters a list of characters, any of which may separate the
	 *                   integer values in the input string
	 * @param minValue the minimum value for each individual number
	 * @param maxValue the maximum value for each individual number
	 * @return an array of integer values parsed from the string
	 * @throws ParseValueException if any value is outside the specified range
	 *                             or the string contains any non-numeric text
	 */
	public static final int[] parseInts( String string, String delimiters, int minValue, int maxValue ) throws ParseValueException
	{
		StringTokenizer tokeniser = new StringTokenizer( string, delimiters );
		
		// create an array of the correct size
		int count = tokeniser.countTokens();
		int[] values = new int[count];
		
		// parse each value in the string
		int value;
		int valueIndex = 0;
		while (tokeniser.hasMoreTokens())
		{
			String token = tokeniser.nextToken();
			
			try
			{
				value = Integer.parseInt( token );
				
				if (   (value < minValue)
				    || (value > maxValue)  )
					throw new RuntimeException( "Number must be in the range " + minValue + ".." + maxValue + "." );
				
				values[valueIndex] = value;
				valueIndex ++;
			}
			catch (Exception e)
			{
				throw new ParseValueException( "Invalid integer value '" + token + "' found in value string '" + string + ".", e );
			}
		}
		
		return values;
	}
	
	/**
	 * Right-align a text string, padding the left portion of the string with
	 * the specified padding character.
	 * <br />
	 * <br />
	 * @param string the original piece of text
	 * @param length the length of the resultant string
	 * @param pad the character to be used to pad the string to the desired
	 *            length
	 * @param truncate set to <code>true</code> if the original string should be
	 *                 truncated to the desired length
	 * @return the padded string
	 */
	public static String rightAlign( String string, int length, char pad, boolean truncate )
	{
		StringBuffer buf = new StringBuffer( length );
		
		// build required amount of padding
		for (int i=length-string.length(); i>0; i--)
		{
			buf.append( pad );
		}
		
		if (   (truncate)
		    && (string.length() > length)  )
			// truncate as necessary
			buf.append( string.substring( string.length() - length ) );
		else
			buf.append( string );
		
		return buf.toString();
	}
	
	/**
	 * Right-align a text string, padding the left portion of the string with
	 * spaces.
	 * <br />
	 * <br />
	 * @param string the original piece of text
	 * @param length the length of the resultant string
	 * @param truncate set to <code>true</code> if the original string should be
	 *                 truncated to the desired length
	 * @return the padded string
	 */
	public static String rightAlign( String string, int length, boolean truncate )
	{
		return rightAlign( string, length, ' ', truncate );
	}
	
	/**
	 * Right-align a text string, padding the left portion of the string with
	 * the specified padding character and truncating the string if necessary.
	 * <br />
	 * <br />
	 * @param string the original piece of text
	 * @param length the length of the resultant string
	 * @param pad the character to be used to pad the string to the desired
	 *            length
	 * @return the padded string
	 */
	public static String rightAlign( String string, int length, char pad )
	{
		return rightAlign( string, length, pad, true );
	}
	
	/**
	 * Right-align a text string, padding the left portion of the string with
	 * spaces and truncating the string if necessary.
	 * <br />
	 * <br />
	 * @param string the original piece of text
	 * @param length the length of the resultant string
	 * @return the padded string
	 */
	public static String rightAlign( String string, int length )
	{
		return rightAlign( string, length, true );
	}
	
	/**
	 * Check if a string contains any of a given set of characters.
	 * <br />
	 * This method will look for each of the characters in turn. Thus, for
	 * optimisation you may want to order the characters in the
	 * <code>chars</code> string such that the characters more likely to match
	 * appear towards the start.
	 * <br />
	 * <br />
	 * @param string the string to test
	 * @param chars the characters to look for
	 * @return <code>true</code> if the string contains any of the characters
	 */
	public static boolean containsAny( String string, String chars )
	{
		for (int i=0; i<chars.length(); i++)
		{
			// is this character in the string
			if (string.indexOf( chars.charAt( i ) ) > -1)
				return true;
		}
		
		return false;
	}
	
	/**
	 * Check if a string contains all of a given set of characters.
	 * <br />
	 * This method will look for each of the characters in turn. Thus, for
	 * optimisation you may want to order the characters in the
	 * <code>chars</code> string such that the characters least likely to match
	 * appear towards the start.
	 * <br />
	 * <br />
	 * @param string the string to test
	 * @param chars the characters to look for
	 * @return <code>true</code> if the string contains all of the characters
	 */
	public static boolean containsAll( String string, String chars )
	{
		for (int i=0; i<chars.length(); i++)
		{
			// is this character missing from the string
			if (string.indexOf( chars.charAt( i ) ) == -1)
				return false;
		}
		
		return true;
	}
	
	/**
	 * Extract the first line from a text string.
	 * <br />
	 * The line will be terminated by a new-line or form-feed character - or the
	 * end of the string.
	 * <br />
	 * <br />
	 * @param text the text to be sampled
	 * @return the first line in the text string
	 * @since 1.3
	 */
	public static String firstLine( String text )
	{
		return nextLine( text, 0 );
	}
	
	/**
	 * Extract a line from a text string.
	 * <br />
	 * The line will begin at the specified index and be terminated by a new-line
	 * or form-feed character - or the end of the string.
	 * <br />
	 * <br />
	 * @param text the text to be sampled
	 * @param fromIndex the index of the start of the line within the text string
	 * @return the next line in the text string
	 * @since 1.3
	 */
	public static String nextLine( String text, int fromIndex )
	{
		int nIndex = text.indexOf( '\n', fromIndex );
		int rIndex = text.indexOf( '\r', fromIndex );
		
		if (nIndex == -1)
		{
			if (rIndex == -1)
				return text.substring( fromIndex );
			else
				return text.substring( fromIndex, rIndex );
		}
		else
		{
			if (rIndex == -1)
				return text.substring( fromIndex, nIndex );
			else
				return text.substring( fromIndex, Math.min( nIndex, rIndex ) );
		}
	}
	
	
	/*
	 * Inner classes
	 */

	/**
	 * Helper class required for exported constant CLASS_MUST_BE_VALID, which
	 * must be unique/different to avoid confusion with any other value a client
	 * may use.
	 */
	protected static class ClassMustBeValid
	{
	}
	
	/**
	 * Helper class required for exported constant ENUM_VALUE_MUST_BE_VALID,
	 * which must be unique/different to avoid confusion with any other value a
	 * client may use.
	 */
	protected static class EnumValueMustBeValid extends EnumValue<EnumValueMustBeValid>
	{
		/** Serialisation version id. */
		private static final long serialVersionUID = -5121412110436726247L;
		
		/** Enumeration holding the values in this enum. */
		public static final Enumeration<EnumValueMustBeValid> ENUMERATION = new Enumeration<EnumValueMustBeValid>();
		
		/** {@inheritDoc} */
		public Enumeration<EnumValueMustBeValid> getEnumeration()
		{
			return ENUMERATION;
		}
		
		// make the constructor protected to protect this enum
		protected EnumValueMustBeValid()
		{
		}
	}
	
}


/*
 * Change History
 *
 * $Log: Strings.java,v $
 * Revision 1.7.2.1  2008/02/18 22:00:08  Steve
 * d JavaDoc links.
 * @
 * text
 * @d3 2
 * d6 15
 * d43 1
 * a43 1
 *  * @@ver *
 * Revision 1.7  2006/09/06 20:52:02  Steve
 * Added legal disclaimer.
 *
 * Revision 1.6  2006/09/02 16:54:49  Steve
 * Fixed JavaDoc links.
 *
 * Revision 1.5  2006/08/19 16:35:50  Steve
 * The parseColor method now processes a space-separated list of decimal values.
 * Added methods: parseInts, containsAll, containsAny.
 *
 * Revision 1.4  2006/08/11 22:02:00  Steve
 * Added parse and serialize method for Color objects.
 *
 * Revision 1.3  2006/05/14 18:12:24  Steve
 * Overloaded parse methods to include interfaces that do not throw exceptions.
 * Updated some JavaDoc comments.
 *
 * Revision 1.2  2006/05/03 21:14:30  Steve
 * Initial revision.
 *
 * Revision 1.1  2006/04/12 16:15:25  Steve
 * Initial revision.
 *
 */
