/*
 * StringsTest.java
 * Part of the B'am library - a collection of general-purpose utility classes.
 * 
 * Copyright (C)2006 Stephen Battey
 * Contact: resource@lucidviews.net
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.lucidviews.util.text;

import junit.framework.TestCase;

import java.awt.Color;


/**
 * Test harness for the <code>Strings</code> class.
 * <br />
 * <br />
 * @author Stephen Battey
 * @version $Revision: 1.3.2.1 $
 * @since 1.0
 */
public class StringsTest extends TestCase
{

	/*
	 * Static methods
	 */
	
	/**
	 * Runs the test in the command console.
	 * <br />
	 * @param args not used
	 */
	public static void main( String[] args )
	{
		junit.textui.TestRunner.run( StringsTest.class );
	}
	
	
	/*
	 * Overridden methods of TestCase base class
	 */
	
	/** {@inheritDoc} */
	public void setUp() throws Exception
	{
		super.setUp();
	}

	
	/*
	 * Methods
	 */
	
	/**
	 * Test parsing a colour value from a string.
	 * <br />
	 * <br />
	 * @throws Exception if this test cannot be performed
	 */
	public void testParseColor() throws Exception
	{
		Color result;
		long decValue;
		
		// hex no alpha (detects no alpha, defaults to 255)
		result = Strings.parseColor( "#80C0ff" );
		assertEquals( 0xFF, result.getAlpha() );
		assertEquals( 0x80, result.getRed() );
		assertEquals( 0xC0, result.getGreen() );
		assertEquals( 0xff, result.getBlue() );
		
		// hex with alpha
		result = Strings.parseColor( "#b00045e7" );
		assertEquals( 0xb0, result.getAlpha() );
		assertEquals( 0x00, result.getRed() );
		assertEquals( 0x45, result.getGreen() );
		assertEquals( 0xe7, result.getBlue() );
		
		// hex with zero alpha
		result = Strings.parseColor( "#000045e7" );
		assertEquals( 0x00, result.getAlpha() );
		assertEquals( 0x00, result.getRed() );
		assertEquals( 0x45, result.getGreen() );
		assertEquals( 0xe7, result.getBlue() );
		
		// hex with single-digit alpha
		result = Strings.parseColor( "#90045e7" );
		assertEquals( 0x09, result.getAlpha() );
		assertEquals( 0x00, result.getRed() );
		assertEquals( 0x45, result.getGreen() );
		assertEquals( 0xe7, result.getBlue() );
		
		// dec no alpha (assumes alpha is specified as 0)
		decValue = 0x80C0ff;
		result = Strings.parseColor( String.valueOf( decValue ) );
		assertEquals( 0x00, result.getAlpha() );
		assertEquals( 0x80, result.getRed() );
		assertEquals( 0xC0, result.getGreen() );
		assertEquals( 0xff, result.getBlue() );
		
		// dec with alpha
		decValue = 0xb00045e7;
		result = Strings.parseColor( String.valueOf( decValue ) );
		assertEquals( 0xb0, result.getAlpha() );
		assertEquals( 0x00, result.getRed() );
		assertEquals( 0x45, result.getGreen() );
		assertEquals( 0xe7, result.getBlue() );
		
		// dec with zero alpha
		decValue = 0x000045e7;
		result = Strings.parseColor( String.valueOf( decValue ) );
		assertEquals( 0x00, result.getAlpha() );
		assertEquals( 0x00, result.getRed() );
		assertEquals( 0x45, result.getGreen() );
		assertEquals( 0xe7, result.getBlue() );
		
		// triple dec
		result = Strings.parseColor( "255 254 253" );
		assertEquals( 255, result.getAlpha() );
		assertEquals( 255, result.getRed() );
		assertEquals( 254, result.getGreen() );
		assertEquals( 253, result.getBlue() );
		
		// triple dec, 1 & 2 digit values
		result = Strings.parseColor( "90 0 10" );
		assertEquals( 255, result.getAlpha() );
		assertEquals( 90, result.getRed() );
		assertEquals( 0, result.getGreen() );
		assertEquals( 10, result.getBlue() );
		
		// quad dec, non-zero alpha
		result = Strings.parseColor( "128 200 201 202" );
		assertEquals( 128, result.getAlpha() );
		assertEquals( 200, result.getRed() );
		assertEquals( 201, result.getGreen() );
		assertEquals( 202, result.getBlue() );
		
		// quad dec, zero alpha
		result = Strings.parseColor( "0 127 128 129" );
		assertEquals( 0, result.getAlpha() );
		assertEquals( 127, result.getRed() );
		assertEquals( 128, result.getGreen() );
		assertEquals( 129, result.getBlue() );
		
		// quad dec, extra spaces
		result = Strings.parseColor( "  128  9    56 129  " );
		assertEquals( 128, result.getAlpha() );
		assertEquals( 9, result.getRed() );
		assertEquals( 56, result.getGreen() );
		assertEquals( 129, result.getBlue() );
		
		// additional numbers
		result = Strings.parseColor( "  128  9    56 129  47  175 " );
		assertEquals( 128, result.getAlpha() );
		assertEquals( 9, result.getRed() );
		assertEquals( 56, result.getGreen() );
		assertEquals( 129, result.getBlue() );
		
		// too few numbers
		try
		{
			result = Strings.parseColor( "  128  9  " );
			fail( "Expected a ParseValueException" );
		}
		catch (ParseValueException e)
		{
			// as expected
		}
		catch (Exception e)
		{
			fail( "Unexpected exception: " + e );
		}
		
		// number out of range
		try
		{
			result = Strings.parseColor( "128 567 48" );
			fail( "Expected a ParseValueException" );
		}
		catch (ParseValueException e)
		{
			// as expected
		}
		catch (Exception e)
		{
			fail( "Unexpected exception: " + e );
		}
	}
	
	/**
	 * Test serialising a colour value to a string.
	 * <br />
	 * <br />
	 * @throws Exception if this test cannot be performed
	 */
	public void testSerializeColor() throws Exception
	{
		Color color;
		String result;
		
		color = Strings.parseColor( "#FFFFFFFF" );
		result = Strings.serializeColor( color );
		assertEquals( "#ffffffff", result );
		
		color = Strings.parseColor( "#b00045e7" );
		result = Strings.serializeColor( color );
		assertEquals( "#b00045e7", result );
		
		color = Strings.parseColor( "#000045e7" );
		result = Strings.serializeColor( color );
		assertEquals( "#000045e7", result );
		
		color = Strings.parseColor( "#0045e7" );
		result = Strings.serializeColor( color );
		assertEquals( "#ff0045e7", result );
		
		color = Strings.parseColor( "#FFFFFF" );
		result = Strings.serializeColor( color );
		assertEquals( "#ffffffff", result );
		
		color = Strings.parseColor( "#FFFFFFF" );
		result = Strings.serializeColor( color );
		assertEquals( "#0fffffff", result );
	}
	
	/**
	 * Test the nextLine method.
	 */
	public void testNextLine()
	{
		assertEquals( "First line", Strings.nextLine( "First line\nSecond line.", 0 ) );
		assertEquals( "First line", Strings.nextLine( "First line\rSecond line.", 0 ) );
		assertEquals( "First line", Strings.nextLine( "First line\r\nSecond line.", 0 ) );
		assertEquals( "First line", Strings.nextLine( "First line\n\rSecond line.", 0 ) );

		assertEquals( "Second line.", Strings.nextLine( "First line\nSecond line.", 11 ) );
		assertEquals( "Second line.", Strings.nextLine( "First line\rSecond line.", 11 ) );
		assertEquals( "Second line.", Strings.nextLine( "First line\r\nSecond line.", 12 ) );
		assertEquals( "Second line.", Strings.nextLine( "First line\n\rSecond line.", 12 ) );
	}
	
}
